/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  calibration.c
* Description:  Contains routines for all calibration preparation routines
*
* History:
* 21-Jul-03     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <cpl.h>
#include <string.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "memoryHandling.h"
#include "calibration.h"
#include "diagnostics.h"
#include "errorHandling.h"
#include "statistics.h"
#include "midiFitsUtility.h"
#include "qfits_table.h"

/**********************************************************
*   Global Variables
**********************************************************/

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getCalibratedChWave
* Input/Output: See function arguments to avoid duplication
* Description:    Retrieves the table of channel frequencies, either from the database
*                or from the current directory. Format for the database is simply
*                a table of pairs of floats. The one in the current directory has
*                the following format:
*
*                Record 1    Grism ID
*                Record 2    Beam Combiner
*                Record 3    Number of records
*                Record 4    Value1 value2
*                .
*                .
*                Record n    value1 value2
*
* History:
* 09-Sep-05 (csabet) Created
******************************************************************************/
void getCalibratedChWave (
    MidiFiles    *fileNames,    // In: Pointer to MIDI file structure
    ImageFormat    *format,    // In: Image format
    float       **waveCal,    // Ou: Wavelength calibration
    int            *error)        // Ou: Error status

{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "getCalibratedChWave";
    char        *fileString, *givenGrismId, *givenBeamCombiner;
    FILE        *filePtr=NULL;
    int            i, numOfRecords;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 1;
    
    //    Check if correct wavelength calibration file exists in the calibration database
    //    If calibration is not in the database then get it from the current directory
    fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sprintf (fileString , "%s_%s_%s.dat", fileNames->waveCalibName, format->grismId, format->beamCombiner);
    if ((filePtr = fopen (fileString, "r")) != NULL)    // It is in the database
    {
        //    Read from file and load into array
        if (diagnostic)cpl_msg_info(cpl_func,"Trying to extract Wavelength Calibration from %s \n", fileString);
        fprintf (midiReportPtr, "Trying to extract Wavelength Calibration from %s \n", fileString);
        numOfRecords = 0;
        
        if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
            while (fscanf (filePtr, "%f %f %f %f\n", 
                &(waveCal[0][numOfRecords]), &(waveCal[1][numOfRecords]),
                &(waveCal[2][numOfRecords]), &(waveCal[3][numOfRecords])) != EOF) numOfRecords++;
        else
            while (fscanf (filePtr, "%f %f\n", 
                &(waveCal[0][numOfRecords]), &(waveCal[1][numOfRecords])) != EOF) numOfRecords++;
                
        fclose (filePtr);
        if (numOfRecords == format->iXWidth)
        {
            *error = 0;
            if (diagnostic)cpl_msg_info(cpl_func,"Found a wavelength calibration file %s with %d rows of floats \n", 
                fileString, numOfRecords);
            fprintf (midiReportPtr, "Found a wavelength calibration file %s with %d rows of floats \n", 
                fileString, numOfRecords);
        }
        else
        {
            *error = 1;
            sprintf (midiMessage, "Inconsistent calibration data in %s. Data ignored", fileString);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        }
    }
    else 
    {
        *error = 1;
        sprintf (midiMessage, "Cannot find calibration data file ... %s", fileString);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
    free (fileString);

    if (*error)
    {
        //    Allocate memory
        *error = 0;
        givenGrismId = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
        givenBeamCombiner = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

        //    Check if wavelength calibration is available. Try to open the file for reading
        if (diagnostic)cpl_msg_info(cpl_func,"Trying to extract Wavelength Calibration from %s \n", fileNames->wclNameRead);
        fprintf (midiReportPtr, "Trying to extract Wavelength Calibration from %s \n", fileNames->wclNameRead);
        if ((filePtr = fopen (fileNames->wclNameRead, "r")) == NULL)
        {
            *error = 1;
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                "Cannot find the Wavelength Calibration file\n             Data will be computed from the interferometry file");
        }
        else *error = 0;    // File exists
    
        //    Read Grism ID
        if (!(*error))
        {
            if (fscanf (filePtr, "%s \n", givenGrismId) == EOF)
            {
                *error = 1;
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                    "Cannot read Grism ID from\n             Data will be computed from the interferometry file");
                fclose (filePtr);
                if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->wclNameRead);
            }
            else
            {
                //    Check if the given Grism ID is the same as that of the current batch
                if (strcmp (format->grismId, givenGrismId) != 0)
                {
                    *error = 1;
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                        "Inconsistent Grism ID\n             Data will be computed from the interferometry file");
                    fclose (filePtr);
                    if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->wclNameRead);
                }
            }
           cpl_msg_info(cpl_func,"Given grism ID          = %s. Expected %s \n", givenGrismId, format->grismId);
            fprintf (midiReportPtr, "Given grism ID          = %s. Expected %s \n", givenGrismId, format->grismId);
        }

        //    Read Beam Combiner
        if (!(*error))
        {
            if (fscanf (filePtr, "%s \n", givenBeamCombiner) == EOF)
            {
                *error = 1;
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                    "Cannot read Beam Combiner from\n             Data will be computed from the interferometry file");
                fclose (filePtr);
                if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->wclNameRead);
            }
            else
            {
                //    Check if the given Beam Combiner is the same as that of the current batch
                if (strcmp (format->beamCombiner, givenBeamCombiner) != 0)
                {
                    *error = 1;
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                        "Inconsistent Beam Combiner\n             Data will be computed from the interferometry file");
                    fclose (filePtr);
                    if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->wclNameRead);
                }
            }
           cpl_msg_info(cpl_func,"Given Beam Combiner     = %s. Expected %s \n", givenBeamCombiner, format->beamCombiner);
            fprintf (midiReportPtr, "Given Beam Combiner     = %s. Expected %s \n", givenBeamCombiner, format->beamCombiner);
        }
    
        //    If so far so good then get the number of records 
        if (!(*error))
        {
            if (fscanf (filePtr, "%d\n", &numOfRecords) == EOF)
            {
                *error = 1;
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                    "Cannot read Number of Records\n             Data will be computed from the interferometry file");
                if (filePtr) fclose (filePtr);
                if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->wclNameRead);
            }
            else 
            {
                //    Check if number of records is consistent
                if (numOfRecords != format->iXWidth)
                {
                    *error = 1;
                    sprintf (midiMessage, "Inconsistent Number of Records from %s. Expected %d \n", 
                        fileNames->wclNameRead, format->iXWidth); 
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    if (filePtr) fclose (filePtr);
                    if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->wclNameRead);
                }
            }
           cpl_msg_info(cpl_func,"Given Number of Records = %d. Expected %d \n", numOfRecords, format->iXWidth);
            fprintf (midiReportPtr, "Given Number of Records = %d. Expected %d \n", numOfRecords, format->iXWidth);
        }
        
        //    If all is well then try to load the data into the array
        if (!(*error))
        {
            for (i = 0; i < format->iXWidth; i++)
            {
                if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
                {
                    if (fscanf (filePtr, "%f %f %f %f \n", &(waveCal[0][i]), &(waveCal[1][i]), 
                        &(waveCal[2][i]), &(waveCal[3][i])) == EOF)
                    {
                        *error = 1;
                        sprintf (midiMessage, "Read %d rows of wavelength values from %s. Expected %d \n", 
                            i, fileNames->wclNameRead, format->iXWidth);
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                        break;
                    }
                    if (diagnostic)cpl_msg_info(cpl_func,"Channel %3d    %7.4f   %7.4f    %7.4f    %7.4f \n", 
                        i, waveCal[0][i], waveCal[1][i], waveCal[2][i], waveCal[3][i]);
                    fprintf (midiReportPtr, "Channel %3d    %7.4f   %7.4f    %7.4f    %7.4f \n", 
                        i, waveCal[0][i], waveCal[1][i], waveCal[2][i], waveCal[3][i]);
                }
                else
                {
                    if (fscanf (filePtr, "%f %f \n", &(waveCal[0][i]), &(waveCal[1][i])) == EOF)
                    {
                        *error = 1;
                        sprintf (midiMessage, "Read %d rows of wavelength values from %s. Expected %d \n", 
                            i, fileNames->wclNameRead, format->iXWidth);
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                        break;
                    }
                    if (diagnostic)cpl_msg_info(cpl_func,"Channel %3d    %7.4f   %7.4f \n", i,  waveCal[0][i], waveCal[1][i]);
                    fprintf (midiReportPtr, "Channel %3d    %7.4f   %7.4f \n", i,  waveCal[0][i], waveCal[1][i]);
                }
            }
           cpl_msg_info(cpl_func,"\nRead %d rows of wavelength values from %s \n", format->iXWidth, fileNames->wclNameRead);
            fprintf (midiReportPtr, "\nRead %d rows of wavelength values from %s \n", format->iXWidth, fileNames->wclNameRead);
            fclose (filePtr);
            if (strcmp (format->obsCatg, "SCIENCE") == 0) remove (fileNames->wclNameRead);
        }
        free (givenGrismId);
        free (givenBeamCombiner);        
    }
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  calibrateWaveUsingPolynom
* Input/Output: See function arguments to avoid duplication
* Description:    Wavelength calibration. Checks if calibration file exists. If not
*                It creates a file using hardcoded polynomial method
*
* History:
* 03-Feb-05 (csabet) Created
******************************************************************************/
void calibrateWaveUsingPolynom (
    MidiFiles        *fileNames,            // In: Pointer to MIDI file structure
    ImageFormat        *format)            // In: Image format

{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "calibrateWaveUsingPolynom";
    int            error, numOfRecords;
    FILE        *waveCalPtr=NULL;
    char        *fileString;
    float        record1, record2, record3, record4;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    error = 0;

   cpl_msg_info(cpl_func,"\nWavelength calibration for batch %d \n", batchNumber);
   cpl_msg_info(cpl_func,"-------------------------------- \n");
    fprintf (midiReportPtr, "\nWavelength calibration for batch %d \n", batchNumber);
    fprintf (midiReportPtr, "-------------------------------- \n");

    //    Check if correct wavelength calibration file exists in the calibration database
    fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sprintf (fileString , "%s_%s_%s.dat", fileNames->waveCalibName, format->grismId, format->beamCombiner);
    if ((waveCalPtr = fopen (fileString, "r")) != NULL)
    {
        if (diagnostic)cpl_msg_info(cpl_func,"\nTrying to extract Wavelength Calibration from %s \n", fileString);
        fprintf (midiReportPtr, "\nTrying to extract Wavelength Calibration from %s \n", fileString);
        numOfRecords = 0;
        if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
            while (fscanf (waveCalPtr, "%f %f %f %f\n", &record1, &record2, &record3, &record4) != EOF) numOfRecords++;
        else
            while (fscanf (waveCalPtr, "%f %f\n", &record1, &record2) != EOF) numOfRecords++;
        fclose (waveCalPtr);
        if (numOfRecords == format->iXWidth)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"\nFound a wavelength calibration file %s with %d pairs of floats \n", 
                fileString, numOfRecords);
            fprintf (midiReportPtr, "\nFound a wavelength calibration file %s with %d pairs of floats \n", 
                fileString, numOfRecords);
        }
        else 
        {
            error = 1;
            sprintf (midiMessage, "Inconsistent calibration data in %s. Data ignored", fileString);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        }
    }
    else 
    {
        error = 1;
        sprintf (midiMessage, "Cannot find calibration data file ... %s", fileString);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
    free (fileString);

    if (error)
    {
        getWaveCalPoly (fileNames, format, &error);
        if (error) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create wavelength calibration");
    }

    return;
}
/*****************************************************************************/


 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getWaveCalPoly
* Input/Output: See function arguments to avoid duplication
* Description:    Supplies the a priori waveCal[][] from a CALIB sources. 
*                The contents are filled in by a calibration polynomial.
*                The format of the output file is as shown below:
*                Record 1    Grism ID
*                Record 2    Beam Combiner
*                Record 3    Number of records
*                Record 4    Value1 value2
*                .
*                .
*                Record n    value1 value2
*
* History:
* 17-Jan-05     (jmeisner) Created
* 26-Jan-05     (csabet) Cleaned up, created database and integrated
******************************************************************************/
void getWaveCalPoly (
    MidiFiles    *fileNames,    // IO: Pointer to midi files structure
    ImageFormat    *imageFormat,
    int            *error)        // Ou: Error status
{
 
    /*  Local Declarations
    --------------------*/
    const char    routine[] = "getWaveCalPoly";
    float         A[2], B[2],    C[2], D[2], x, **waveCal=NULL;
    int         i, j, regions, useGrism;
    FILE        *waveCalPtr;
    struct stat    buf;
 
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
 
   cpl_msg_info(cpl_func,"\nComputing wavelength calibration using Calibration Polynomial \n");
    fprintf (midiReportPtr, "\nComputing wavelength calibration using Calibration Polynomial \n");
 
    //    Reset status. Check if wavelength calibration file exist. If so delete it
    if (stat (fileNames->wclNameWrite, &buf) == 0) remove (fileNames->wclNameWrite);
    *error = 0;
    regions = 2;
    useGrism = 0;
    
    //    Allocate memory
    waveCal = callocWaveCalibration (regions, imageFormat);
 
    //    Open the wavelength calibration file for writing
    if ((waveCalPtr = fopen (fileNames->wclNameWrite, "w")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open file to store wavelength calibrations");
        freeWaveCalibration (regions, waveCal);
        *error = 1;
        return;
    }
    
    //    Check Grism ID
    if (strcmp (imageFormat->grismId, "GRISM") == 0) useGrism = 1;
   
    //    Now here are some numbers obtained from Jeff, to use for the time being......
    if (useGrism)    // for the grism
    {
        A[0] = 1.9837227e-08;
        A[1] = 3.0043738e-08;
        B[0] = -7.7775886e-06;
        B[1] = -1.2241891e-05;
        C[0] = 0.023746001;
        C[1] = 0.024323120;
        D[0] = 7.6040181;
        D[1] = 7.5892995;
        for (i = 0; i < imageFormat->iXWidth; i++)
        {
            for (j = 0; j < regions; j++)
            {
                x = (float) i;
                waveCal[j][i] = 299.8F/(A[j]*x*x*x + B[j]*x*x + C[j]*x + D[j]); 
            }
        }
    }
 
    if (!useGrism)    // for the Prism:
    {
        A[0] = -6.1780766e-07;
        A[1] = -6.8179155e-07;
        B[0] = -1.6145018e-06;
        B[1] = 2.1291316e-05;
        C[0] = -0.037844515;
        C[1] = -0.040598183;
        D[0] = 14.576070;
        D[1] = 14.652558;
        for (i = 0; i < imageFormat->iXWidth; i++)
        {
            for (j = 0; j < regions; j++)
            {
                x = (float) i;
                waveCal[j][i] = 299.8F/(A[j]*x*x*x + B[j]*x*x + C[j]*x + D[j]); 
            }
        }
    }
 
    //    Convert to wavelength
    for (j = 0; j < regions; j++)
    {
        for (i = 0; i < imageFormat->iXWidth; i++) waveCal[j][i] = SPEED_OF_LIGHT / waveCal[j][i];
    }    
 
    //    Report
    if (diagnostic)cpl_msg_info(cpl_func,"\nWavelength Calibration \n");
    fprintf (midiReportPtr, "\nWavelength Calibration \n");
    for (i = 0; i < imageFormat->iXWidth; i++)
    {
        if (strcmp (imageFormat->beamCombiner, "SCI_PHOT") == 0)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"%3d   %7.4f   %7.4f   %7.4f   %7.4f \n", i, 
                waveCal[0][i], waveCal[0][i], waveCal[1][i], waveCal[0][i]);
            fprintf (midiReportPtr, "%3d   %7.4f   %7.4f   %7.4f   %7.4f \n", i, 
                waveCal[0][i], waveCal[0][i], waveCal[1][i], waveCal[0][i]);
        }
        else
        {
            if (diagnostic)cpl_msg_info(cpl_func,"%3d   %7.4f   %7.4f \n", i, waveCal[0][i], waveCal[1][i]);
            fprintf (midiReportPtr, "%3d   %7.4f   %7.4f \n", i, waveCal[0][i], waveCal[1][i]);
        }
    }
    if (diagnostic)cpl_msg_info(cpl_func,"\n");
    fprintf (midiReportPtr, "\n");
 
    //    Write into the Wavelength Calibration file
    fprintf (waveCalPtr, "%s \n", imageFormat->grismId);        //    Record 1
    fprintf (waveCalPtr, "%s \n", imageFormat->beamCombiner);    //    Record 2
    fprintf (waveCalPtr, "%d \n", imageFormat->iXWidth);        //    Record 3
    if (strcmp (imageFormat->beamCombiner, "SCI_PHOT") == 0)
    {
        for (i = 0; i < imageFormat->iXWidth; i++)
            fprintf (waveCalPtr, "%f   %f   %f   %f \n", waveCal[0][i], waveCal[0][i], 
                waveCal[1][i], waveCal[0][i]);
    }
    else
    {
        for (i = 0; i < imageFormat->iXWidth; i++)
            fprintf (waveCalPtr, "%f   %f \n", waveCal[0][i], waveCal[1][i]);
    }
 
    fclose (waveCalPtr);
    
   cpl_msg_info(cpl_func,"Created the wavelength calibration file ... %s \n", fileNames->wclNameWrite);
    fprintf (midiReportPtr, "Created the wavelength calibration file ... %s \n", fileNames->wclNameWrite);
    
    //    Release memory
    freeWaveCalibration (regions, waveCal);
    
    return;
}
/******************************************************************************/
 
 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:    checkDelayLineJumps
* Input/Output:    See function arguments to avoid duplication
* Description:    Check the Big delay line and the Piezo for illegal DL jumps (within one scan):
*
* History:
* 03-Feb-06        (csabet) Aapted from Jeff Meisner for SCI_PHOT
******************************************************************************/
void checkDelayLineJumps (
    const char            *dataKey,        // In: Whether Interferometry or Photometry etc.
    ImageFormat        *format,
    CompressedData    *compressed)  
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "checkDelayLineJumps";
    int            i, x, numSummed = 0;
    float        lastDL, diff, avg, rms, threshold, lastPiezo,
                sum=0.0, sumsqr=0.0;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nChecking Delay Line Jumps within scans for %s \n", dataKey);
   cpl_msg_info(cpl_func,"-------------------------------------------- \n");
    fprintf (midiReportPtr, "\nChecking Delay Line Jumps within scans for %s \n", dataKey);
    fprintf (midiReportPtr, "-------------------------------------------- \n");

    for (i = 0; i < format->numOfFrames; i++)
    {
        if ((i % format->framesPerScan) == 0)    // If the first sample of a scan
            lastDL = compressed->bigDL[i];
        else if (lastDL != compressed->bigDL[i])
        {
           cpl_msg_info(cpl_func,"<<Warning>>: Delay Line jump at scan %d, from %f to %f \n", 
                i/format->framesPerScan, lastDL, compressed->bigDL[i]);
            fprintf (midiReportPtr, "<<Warning>>: Delay Line jump at scan %d, from %f to %f \n", 
                i, lastDL, compressed->bigDL[i]);
            compressed->badScanList[i/format->framesPerScan] |= BSL_DLJ_ERROR;                // Indicates DL jump within scan
            lastDL = (compressed->bigDL)[i];
        }
    }
    
    //    Now look at the Piezo, and try to set the optical freq cal:
    for (i = 0; i < format->numOfFrames; i++) 
    {
        if ((i % format->framesPerScan) == 0)    // If the first sample of a scan
            lastPiezo  = (compressed->localOPD)[i];
        else
        {    
            //    If this was NOT just after a flyback, good values and good scan
            if (!(isnan (compressed->localOPD[i])) && 
                !(isnan (lastPiezo)) && 
                !(compressed->badScanList[i/format->framesPerScan])) // All regions and channels are affected
            {
                diff = compressed->localOPD[i] - lastPiezo;    //  compressed->localOPD[i-1]
                sum += diff;
                sumsqr += diff * diff;
                numSummed++;
                lastPiezo = compressed->localOPD[i];        // for next pass through loop
            }
        }
    }
        
    avg = sum/((float) numSummed);
    rms =  sqrtp(sumsqr/((float) numSummed) - avg * avg);
    if ( rms < (0.02 * fabs(avg))) rms = 0.02F * fabs(avg);

    //    Now for a second pass, not using the outliers (but marking them!!):
    threshold = 5.F * rms;    // Set at 5 sigma
    for (i = 0; i < format->numOfFrames; i++) 
    {
        if ((i % format->framesPerScan) == 0)    // If the first sample of a scan
            lastPiezo  = compressed->localOPD[i];
        else
        {   // If this was NOT just after a flyback, good values and good scan
            if (!(isnan (compressed->localOPD[i])) && 
                !(isnan (lastPiezo)) && 
                !(compressed->badScanList[i/format->framesPerScan])) // All regions and channels are affected
            {
                diff = compressed->localOPD[i] - lastPiezo;    //  compressed->localOPD[i-1]
                if (fabs(diff-avg) > threshold) 
                {
                   cpl_msg_info(cpl_func,"<<Warning>>: PIEZO jump at scan %d, of %f microns. Expected %f \n",
                        i/format->framesPerScan, 1000000.F*diff, 1000000.F * avg);
                    fprintf (midiReportPtr, "<<Warning>>: PIEZO jump at scan %d, of %f microns. Expected %f \n",
                        i/format->framesPerScan, 1000000.F*diff, 1000000.F * avg);
                    compressed->badScanList[i/format->framesPerScan] |= BSL_DLJ_ERROR;                // Indicates piezo jump within scan.
                    lastPiezo += avg;    // <<<< Fake it!
                }
                else  // Normally:
                {
                    sum += diff;
                    numSummed++;
                    lastPiezo = compressed->localOPD[i]; // for next pass thru loop
                }
            }
        }
    }
    avg = sum/((float) numSummed);
    if (diagnostic)cpl_msg_info(cpl_func,"Average piezo step = %5.3f microns \n", 1000000.F*avg);
    fprintf(midiReportPtr, "Average piezo step = %5.3f microns \n", 1000000.F*avg);

    format->optFreqCal  = 1.F/(3335.F*fabs(avg));    // NOW SET HERE!!!;

    if (diagnostic)cpl_msg_info(cpl_func,"Setting optical wavelength calibration for this observation: %5.3f micron\n", 
        format->optFreqCal);
    fprintf(midiReportPtr, "Setting optical wavelength calibration for this observation: %5.3f micron\n", 
        format->optFreqCal);
    
    //    Transfer badScanList to rejectList for DISPERSED processing
    for (x = 0; x < format->iXWidth; x++)
    {
        for (i = 0; i < format->numOfFrames; i++)
            compressed->rejectList[x][i] |= compressed->badScanList[i/format->framesPerScan];
    }
    
    return;     
}
/******************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  checkDelayLineConsistencies
* Input/Output: See function arguments to avoid duplication
* Description:    Checks delay line to see if it is reasonable
*
* History:
* 03-Feb-06 (csabet) Adapted from Jeff Meisner for SCI_PHOT
******************************************************************************/
void checkDelayLineConsistencies (
    const char            *dataKey1,            // In: Whether Interferometry or Photometry etc.
    const char            *dataKey2,            // In: Whether Interferometry or Photometry etc.
    CompressedData    *compressedInterf,    // In: Compressed Interferometry data
    CompressedData    *compressedPhotom,    // In: Compressed photometry data
    ImageFormat        *formatInterf,        // In: Interferompetry data size
    ImageFormat        *formatPhotom,        // In: Photometry data size
    int                *error)                // Ou: Error status

{

    //    Local Declarations
    //    ------------------
    const char            routine[] = "checkDelayLineConsistencies";
    int                 j, type, numOfFrames, startFrame, toBeCounted;
    CompressedData         *thisData=NULL;
    float                frameTime100[ARB_NUM_OF_FRAMETIME];    // Number of frametime between 100 frames
    float                mean[3], rms;


    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nChecking Delay Line Consistencies between %s and %s \n", dataKey1, dataKey2);
   cpl_msg_info(cpl_func,"------------------------------------------- \n");
    fprintf (midiReportPtr, "\nChecking Delay Line Consistencies between %s and %s \n", dataKey1, dataKey2);
    fprintf (midiReportPtr, "------------------------------------------- \n");

    //    Reset status
    *error = 0;
    
    //    Look at the DL data and the TIME data. Perhaps check to see if reasonable
    for (type = 0; type < 2; type++)
    {
        if (type == 0) thisData = compressedInterf;    // Interferometry
        if (type == 1) thisData = compressedPhotom;    // Photometry
        if (type == 0) numOfFrames = formatInterf->numOfFrames;    // Interferometry
        if (type == 1) numOfFrames = formatPhotom->numOfFrames;    // Photometry
        if (numOfFrames < (ARB_NUM_OF_FRAMETIME + 101)) continue;
        toBeCounted = 0;    // In case of bad value (csabet, 02-Mar-05)
        for (j = 0; j < ARB_NUM_OF_FRAMETIME; j++)
        {
            startFrame = j * ((numOfFrames-101)/ARB_NUM_OF_FRAMETIME);
            if (!isnan (thisData->time[startFrame+100]) && !isnan (thisData->time[startFrame]))
            {
                //    Now in milliseconds
                frameTime100[j] = (thisData->time[startFrame+100] - thisData->time[startFrame]) * 864000.0;
                toBeCounted++;
            }
        }
        makeStats (frameTime100, toBeCounted, mean+type, &rms);
        if (diagnostic)cpl_msg_info(cpl_func,"Found FRAME TIME for dataset %d = %7.2f ms (apparent rms = %f) \n", 
            type, mean[type], rms*10.0);
        fprintf (midiReportPtr, "Found FRAME TIME for dataset %d = %7.2f ms (apparent rms = %f) \n", 
            type, mean[type], rms*10.0);
        if (rms > (0.01 * mean[type])) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Apparently an inconsistent FRAME TIME");
    }
    
    if (fabs(mean[1] - mean[0]) > (0.01 * mean[0]))
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Found inconsistent FRAME TIME between Interferometry & Photometry files");
        *error = 1;
    }

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createMaskFile
* Input/Output: See function arguments to avoid duplication
* Description:
*
* History:
* 21-Jul-03 (csabet) Created
******************************************************************************/
void createMaskFile (
    MidiFiles    *fileNames,    // IO: Pointer to midi files structure
    int            *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char                routine[] = "createMaskFile";
    FILE                    *inFitsBatchPtr = NULL;
    ImageFormat                *param;
    int                        imagingDataExtension;
    char                    *stringTemp, *classification, *maskPath;
    struct stat                buf;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating a mask file for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------ \n");
    fprintf (midiReportPtr, "\nCreating a mask file for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------ \n");

    //    Reset status
    *error = 0;

    //    Allocate memory
    param = callocImageFormat ();
    maskPath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot open input FITS file list\n                 This batch has been ignored");
        freeImageFormat (param);
        free (classification);
        free (stringTemp);
        *error = 1;
        return;
    }

    //    Loop through the file list
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"Processing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "Processing file   %s \n", fileNames->inFitsName);

        //    Get extension number of IMAGING_DATA in input file
        imagingDataExtension  = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format
        if (imagingDataExtension > 0)
        {
            getImageFormat (fileNames->inFitsName, imagingDataExtension, param, error);
            if (*error) break;
        }
        else param->hasData = 0;

        //    Check if the file has data
        if (param->hasData)
        {
            //    If mask file is not available then create it
            sprintf (fileNames->maskFileName, "Mask_ONLINE_%s_%s.fits", param->grismId, param->beamCombiner);
            sprintf (maskPath, "%s%s", outFileDir, fileNames->maskFileName);
            if ((stat (maskPath, &buf)) != 0)
            {
                //    Check if Categ, Tech, Type is suitable for this task
                if (((strcmp (param->obsCatg, "CALIB") == 0) || 
                    (strcmp (param->obsCatg, "SCIENCE") == 0)) &&
                    (strcmp (param->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0) && 
                    (strcmp (param->obsType, "PHOTOMETRY,OBJECT") == 0))
                {
                    preparePhotomAperMask (fileNames->inFitsName, maskPath, error);
                    if (*error) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create mask file from");
                }
                else if ((strcmp (param->beamCombiner, "SCI_PHOT") == 0))
                {
                    //    Use only one file
                    createMaskSP (fileNames->inFitsName, maskPath, error);
                    if (*error) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create mask file from");
                    break;
                }
                else if (((strcmp (param->obsCatg, "CALIB") == 0) || (strcmp (param->obsCatg, "SCIENCE") == 0)) &&
                    (strcmp (param->obsTech, "UNKNOWN_TECH") == 0) && (strcmp (param->obsType, "UNKNOWN_TYPE") == 0))
                    prepareSpectroPhotomAperMask ();
                else if (((strcmp (param->obsCatg, "CALIB") == 0) || (strcmp (param->obsCatg, "SCIENCE") == 0)) &&
                    (strcmp (param->obsTech, "UNKNOWN_TECH") == 0) && (strcmp (param->obsType, "UNKNOWN_TYPE") == 0))
                    prepareMasterFlat ();
                else if (((strcmp (param->obsCatg, "CALIB") == 0) || (strcmp (param->obsCatg, "SCIENCE") == 0)) &&
                    (strcmp (param->obsTech, "UNKNOWN_TECH") == 0) && (strcmp (param->obsType, "UNKNOWN_TYPE") == 0))
                    prepareMasterDark ();
                else if (((strcmp (param->obsCatg, "CALIB") == 0) || (strcmp (param->obsCatg, "SCIENCE") == 0)) &&
                    (strcmp (param->obsTech, "UNKNOWN_TECH") == 0) && (strcmp (param->obsType, "UNKNOWN_TYPE") == 0))
                    prepareDispersionRelation ();
                else if (((strcmp (param->obsCatg, "CALIB") == 0) || (strcmp (param->obsCatg, "SCIENCE") == 0)) &&
                    (strcmp (param->obsTech, "UNKNOWN_TECH") == 0) && (strcmp (param->obsType, "UNKNOWN_TYPE") == 0))
                    prepareZeroPoint ();
                else if (((strcmp (param->obsCatg, "CALIB") == 0) || (strcmp (param->obsCatg, "SCIENCE") == 0)) &&
                    (strcmp (param->obsTech, "UNKNOWN_TECH") == 0) && (strcmp (param->obsType, "UNKNOWN_TYPE") == 0))
                    prepareSpectralSensitivity ();
                else
                    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, "The above file is not suitable for this task");
            }
            else
            {
                if (diagnostic)cpl_msg_info(cpl_func,"Found mask FITS file   %s \n", maskPath);
                fprintf (midiReportPtr, "Found mask FITS file   %s \n", maskPath);
                break;
            }
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    //    Release memory and close files
    fclose (inFitsBatchPtr);
    freeImageFormat (param);
    free (classification);
    free (stringTemp);
    free (maskPath);

    return;
}
/*****************************************************************************/

/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  prepareMasterFlat
* Input/Output: See function arguments to avoid duplication
* Description:
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void prepareMasterFlat (void)
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "prepareMasterFlat";

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire routine to be written");

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  prepareMasterDark
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void prepareMasterDark (void)
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "prepareMasterDark";
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire routine to be written");

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  prepareDispersiveRelation
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void prepareDispersionRelation (void)
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "prepareDispersiveRelation";

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire routine to be written");

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  prepareSpectroPhotomAperMask
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void prepareSpectroPhotomAperMask (void)
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "prepareSpectroPhotomAperMask";

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire routine to be written");

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createMaskSP
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 07-Mar-06     (csabet) Created
******************************************************************************/
void createMaskSP (
    char    *inFitsName,    // In: Name of the input data file
    char    *maskFitsName,    // In: Name of the mask file
    int        *error)            // Ou: Error status
{
 
    //    Local Declarations
    //    ------------------
    const char        routine[] = "createMaskSP";
    ImageFormat        *format;
    int                i, x, y, extNumOfImagingData, subWindowSize, tableWidth, numOfColumns, 
                    edgeLX, edgeRX, edgeFY, edgeBY, numOfRows;
    qfits_header    *primHeadMask, *imagDataHeader;
    char            *dateTime, *tdim1, *tdim2, *tdim3, *tdim4;
    FILE            *maskFitsPtr;
    qfits_table        *maskTable;
    float            **array, *data1, *data2, *data3, *data4;
    qfits_col        *col;
 
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
 
    //    Reset status
    *error = 0;
    numOfRows = 1;
    numOfColumns = 4;
    tableWidth = - 1;
    
   cpl_msg_info(cpl_func,"Creating mask FITS file   %s \n", maskFitsName);
    fprintf (midiReportPtr, "Creating mask FITS file   %s \n", maskFitsName);
 
    //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
    extNumOfImagingData  = findImagingDataExtension (inFitsName, TAB_IMAGING_DATA, error);
 
    //    Get Image Format parameters and allocate memory for the data1 and data2 */
    format = callocImageFormat ();
    getImageFormat (inFitsName, extNumOfImagingData, format, error);
    subWindowSize = format->iXWidth * format->iYWidth;
 
    //    Extract Primary Header from the input file
    if (diagnostic)cpl_msg_info(cpl_func,"Extracting primary header from input FITS file   %s \n", inFitsName);
    if (diagnostic) fprintf(midiReportPtr, "Extracting primary header from input FITS file   %s \n", inFitsName);
    primHeadMask = qfits_header_read (inFitsName);
    if (primHeadMask == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot extract primary header from input FITS file");
       cpl_msg_info(cpl_func,"\nCannot Create mask FITS file for batch  %d \n", batchNumber);
        fprintf (midiReportPtr, "\nCannot Create mask FITS file for batch  %d \n", batchNumber);
        freeImageFormat (format);
        *error = 1;
        return;
    }
 
    //    Update primary header for the Mask file
    dateTime = qfits_get_datetime_iso8601 ();
    qfits_header_mod (primHeadMask, "DATE", dateTime, "Date and time this file was written");
    qfits_header_mod (primHeadMask, "INSTRUME", "MIDI", "MIDI Mask created by DRS pipeline" );
    qfits_header_add (primHeadMask, "EXTNAME", "Primary", "MIDI Mask primary header", "" );
 
    //    Create the mask file and dump the primary header and close the file
    maskFitsPtr = fopen (maskFitsName, "w");
    qfits_header_dump (primHeadMask, maskFitsPtr);
    qfits_header_destroy (primHeadMask);
    fclose (maskFitsPtr);
 
    //    Allocate memory for data
    data1 = (float *) calloc (subWindowSize, sizeof (float));
    data2 = (float *) calloc (subWindowSize, sizeof (float));
    data3 = (float *) calloc (subWindowSize, sizeof (float));
    data4 = (float *) calloc (subWindowSize, sizeof (float));
    tdim1 = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    tdim2 = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    tdim3 = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    tdim4 = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
 
    //    Load data
    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "This is a temporary rectangular mask");
    edgeFY = 0.30 * format->iYWidth;
    edgeBY = 0.50 * format->iYWidth;
    edgeLX = 0.20 * format->iXWidth;
    edgeRX = 0.20 * format->iXWidth;
    for (y = edgeFY; y < format->iYWidth-edgeBY; y++)
    {
        for (x = edgeLX; x < format->iXWidth-edgeRX; x++)
        {
            i = x + y * format->iXWidth;
            data1[i] = 1.0;
            data2[i] = 1.0;
            data3[i] = 1.0;
            data4[i] = 1.0;
        }
    }
 
    //    Open the mask file for writing
    maskFitsPtr = fopen (maskFitsName, "a");
    if (maskFitsPtr == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open mask file");
        *error = 1;
        free (tdim1);
        free (tdim2);
        free (tdim3);
        free (tdim4);
        free (data1);
        free (data2);
        free (data3);
        free (data4);
        freeImageFormat (format);
        return;
    }
 
    //    Create a new table
    maskTable = qfits_table_new (maskFitsName, QFITS_BINTABLE, tableWidth, numOfColumns, numOfRows);
    if (!maskTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create table");
        *error = 1;
        free (tdim1);
        free (tdim2);
        free (tdim3);
        free (tdim4);
        free (data1);
        free (data2);
        free (data3);
        free (data4);
        freeImageFormat (format);
        fclose (maskFitsPtr);
        return;
    }
 
    //    Fill in the column information
    col = maskTable->col;
    qfits_col_fill (col, subWindowSize, 2, sizeof(float), TFITS_BIN_TYPE_E, "DATA1", " ", " ", " ",
        0, 0.0, 0, 1.0, 0);
    col++;
    qfits_col_fill (col, subWindowSize, 2, sizeof(float), TFITS_BIN_TYPE_E, "DATA2", " ", " ", " ",
        0, 0.0, 0, 1.0, 0);
    col++;
    qfits_col_fill (col, subWindowSize, 2, sizeof(float), TFITS_BIN_TYPE_E, "DATA3", " ", " ", " ",
        0, 0.0, 0, 1.0, 0);
    col++;
    qfits_col_fill (col, subWindowSize, 2, sizeof(float), TFITS_BIN_TYPE_E, "DATA4", " ", " ", " ",
        0, 0.0, 0, 1.0, subWindowSize* sizeof(float));
 
    //    Create the imaging data header
    imagDataHeader = qfits_table_ext_header_default(maskTable);
    qfits_header_add (imagDataHeader, "EXTNAME", "IMAGING_DATA", "MIDI Mask Extension name", "" );
 
 
    //    Add dimensionality to the extension header
    sprintf (tdim1, "'(%d,%d)'", format->iXWidth, format->iYWidth);
    sprintf (tdim2, "'(%d,%d)'", format->iXWidth, format->iYWidth);
    sprintf (tdim3, "'(%d,%d)'", format->iXWidth, format->iYWidth);
    sprintf (tdim4, "'(%d,%d)'", format->iXWidth, format->iYWidth);
    qfits_header_add (imagDataHeader, "TDIM1", tdim1, "Column dimensionality", "" );
    qfits_header_add (imagDataHeader, "TDIM2", tdim2, "Column dimensionality", "" );
    qfits_header_add (imagDataHeader, "TDIM3", tdim2, "Column dimensionality", "" );
    qfits_header_add (imagDataHeader, "TDIM4", tdim2, "Column dimensionality", "" );
 
    //    Allocate space for the array of data
    array = (float **) calloc (numOfColumns, sizeof(float *));
    array[0] = data1;
    array[1] = data2;
    array[2] = data3;
    array[3] = data4;
 
    if (qfits_table_append_xtension_hdr (maskFitsPtr, maskTable, (const void **)array, imagDataHeader) == -1)
        *error = 1;
 
    qfits_header_destroy (imagDataHeader);
    qfits_table_close (maskTable);
 
    if (!(*error))
    {
       cpl_msg_info(cpl_func,"Created Mask FITS file: %s \n", maskFitsName);
        fprintf (midiReportPtr, "Created Mask FITS file: %s \n", maskFitsName);
    }
 
    //    Release memory
    free (array);
    fclose (maskFitsPtr);
    free (tdim1);
    free (tdim2);
    free (tdim3);
    free (tdim4);
    free (data1);
    free (data2);
    free (data3);
    free (data4);
    freeImageFormat (format);
 
    return;
}
/*****************************************************************************/
 
 
/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  preparePhotomAperMask
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void preparePhotomAperMask (
    char    *inFileName,    // In: Name of the input data file
    char    *maskFileName,    // In: Name of the mask file
    int        *error)            // Ou: Error status
{

    /*  Local Declarations
    --------------------*/
    const char          routine[] = "preparePhotomAperMask";
    static short int    *aveData_1, *aveData_2;
    short int           *inData_1, *inData_2;
    ImageFormat         *format;
    int                 i, j, extNumOfImagingData, numOfElemets,
                        scalingOffset, indexData1, indexData2, subWindowSize, pixel;
    static int          aprioriNumOfElemets = 0, numOfFiles = 0;
    qfits_table         *imagDataTable;
    char                *tempStr, *dataName;
    int                 foundData1 = 0, foundData2 = 0;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /*  Reset status */
    *error = 0;
    numOfFiles++;

    /*  Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file */
    extNumOfImagingData  = findImagingDataExtension (inFileName, TAB_IMAGING_DATA, error);

    /*  Get Image Format parameters and allocate memory for the data1 and data2 */
    format = callocImageFormat ();
    getImageFormat (inFileName, extNumOfImagingData, format, error);
    subWindowSize = format->iXWidth * format->iYWidth;
    numOfElemets = subWindowSize * format->numOfFrames;

    /*  Allocate memory in the first call */
    if (numOfFiles == 1)
    {
        aprioriNumOfElemets = numOfElemets;
        aveData_1 = (short int *) calloc (numOfElemets, sizeof (short int));
        aveData_2 = (short int *) calloc (numOfElemets, sizeof (short int));
    }

    if (aprioriNumOfElemets != numOfElemets)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent photometry files");
        *error = 1;
        freeImageFormat (format);
        free (aveData_1);
        free (aveData_2);
        return;
    }

    /*  Open IMAGING_DATA */
    imagDataTable = qfits_table_open (inFileName, extNumOfImagingData);
    if (!imagDataTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load IMAGING_DATA");
        *error = 1;
        freeImageFormat (format);
        free (aveData_1);
        free (aveData_2);
        return;
    }

    dataName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    //    Get the scaling offset
    for (i = 14; i < 25; i++)
    {
        sprintf (dataName, "TZERO%d", i);
        tempStr = qfits_query_ext (inFileName, dataName, extNumOfImagingData);
        if (tempStr != NULL)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"Scaling Offset = %s\n", tempStr);
            fprintf( midiReportPtr, "Scaling Offset = %s\n", tempStr);
            sscanf (tempStr, "%d", &scalingOffset);
            break;
        }
    }
    if (tempStr == NULL)
    {
        scalingOffset = 0;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Scaling Offset. It is set to 0");
    }
    free (dataName);
    
    /*  Get data table information */
    indexData1 = 0;
    indexData2 = 0;
    for (i = 0; i < imagDataTable->nc; i++)
    {
        if (strcmp (imagDataTable->col[i].tlabel, "DATA1") == 0)
        {
            foundData1 = 1;
            indexData1 = i;
        }
        if (strcmp (imagDataTable->col[i].tlabel, "DATA2") == 0)
        {
            foundData2 = 1;
            indexData2 = i;
        }
    }
    if (foundData1 == 0 || foundData2 == 0)
        {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find requested columns in date FITS file");
        *error = 1;
        qfits_table_close (imagDataTable);
        free (aveData_1);
        freeImageFormat (format);
        free (aveData_2);
        return;
    }

    /*  Load data */
    inData_1 = (short int *) (qfits_query_column (imagDataTable, indexData1, NULL));
    inData_2 = (short int *) (qfits_query_column (imagDataTable, indexData2, NULL));

    /*  Cumulate data */
    for (j = 0; j < format->numOfFrames; j++)
    {
        for (pixel = 0; pixel < subWindowSize; pixel++)
        {
            i = j * subWindowSize + pixel;
/*
            if (isnan (inData_1[i]) || isnan (inData_2[i]))
            {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Corrupted DATA1 or DATA2");
                qfits_table_close (imagDataTable);
                free (inData_1);
                free (inData_2);
                freeImageFormat (format);
                *error = 1;
                return;
            }
*/
            aveData_1[i] += (inData_1[i] + scalingOffset);
            aveData_2[i] += (inData_2[i] + scalingOffset);
        }
    }
    free (inData_1);
    free (inData_2);

    /*  Create the mask file */
    if (numOfFiles == 2)
    {
        for (i = 0; i < numOfElemets; i++)
        {
            aveData_1[i] *= 0.5;
            aveData_2[i] *= 0.5;
        }
        createMidiMaskFile(format->iXWidth, format->iYWidth, inFileName, (short int *)(aveData_1), (short int *)(aveData_2),
            format->numOfFrames, subWindowSize, maskFileName, error);
        if (*error)
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create mask file");

        /*  Reset counter for other batches */
        numOfFiles = 0;

        /*  Release memory */
        free (aveData_1);
        free (aveData_2);
    }

    /*  Release memory */
    freeImageFormat (format);

    qfits_table_close (imagDataTable);

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  prepareZeroPoint
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void prepareZeroPoint (void)
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "prepareZeroPoint";

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire routine to be written");

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  prepareSpectralSensitivity
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void prepareSpectralSensitivity (void)
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "prepareSpectralSensitivity";

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire routine to be written");

    return;
}
/*****************************************************************************/
