/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  createProdFrg.c
* Description:  Contains routines for product files handling
*
* History:
* 25-Apr-03     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <cpl.h>
#include <time.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "iauWrite.h"
#include "createProdFrg.h"
#include "midiFitsUtility.h"
#include "geometry.h"
#include "memoryHandling.h"
#include "errorHandling.h"
#include "fileHandling.h"
#include "qfits.h"

/**********************************************************
*   Global Variables
**********************************************************/

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Function definitions
**********************************************************/
static void set_fnan(float *f)
{

#ifndef WORDS_BIGENDIAN
    static unsigned char fnan_pat[] = {0, 0, 0xc0, 0x7f};
#else
    static unsigned char fnan_pat[] = {0x7f, 0xc0, 0, 0};
#endif

memcpy(f, fnan_pat, 4);
}


/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createFrgProd
* Input/Output: See function arguments to avoid duplication
* Description:  Creates all product files for the DISPERSED mode of processing
*
*
* History:
* 09-Feb-05     (csabet) Created
******************************************************************************/
IauExchange * createFrgProd (
    MidiFiles        *fileNames,            // In: File names
    ImageFormat        *imageSizeInterf,    // In: Interferometry data parameters
    float           *waveCal,            // In: Wavelength array
    DispersedResult    *dispResult,        // In: Dispersed result
    CalibratorParam    *calibrator,        // In: Calibrator parameters
    int                *error,             // Ou: Error Status
    CorrectedFrames   *corrFrames)

{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "createFrgProd";
    int                numOfTelescopes, extNumber;
    char            *stringQfits, *classification, *stringTemp;
    FILE            *inFitsBatchPtr = NULL;
    IauExchange     *iauData;
    UVW             *uvw;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating Product files for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"-------------------------------- \n");
    fprintf (midiReportPtr, "\nCreating Product files for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "-------------------------------- \n");

    //    Reset status
    *error = 0;
    
    //    Allocate memory
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (stringTemp);
        free (classification);
        return NULL;
    }

    //    Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);
    fclose (inFitsBatchPtr);

    //    Copy keywords from the raw FITS file into the QC log
    createQcLog (fileNames->inFitsName, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot copy keywords to QC log");
        free (stringTemp);
        free (classification);
        return NULL;
    }

    //    Get number of array elements. Number of telescopes
    extNumber = getFitsExtensionNumber (fileNames->inFitsName, "IMAGING_DATA", error);
    stringQfits = qfits_query_ext (fileNames->inFitsName, "MAXTEL", extNumber);
    if (stringQfits == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read MAXTEL from input FITS file");
        free (stringTemp);
        free (classification);
        return NULL;
    }
    else sscanf (stringQfits, "%d", &numOfTelescopes);

    //    Allocate memory for data structures relating to IAU exchange product FITS file
    iauData = callocIauExchange (numOfTelescopes, imageSizeInterf->iXWidth);
    uvw = (UVW *) calloc (1, sizeof (UVW));

    //    Load output data into the IAU exchange structures
    loadFrgOutputData (imageSizeInterf->obsCatg, fileNames->inFitsName, waveCal, dispResult, iauData, uvw, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load data structure");
        freeIauExchange (iauData);
        free (uvw);
        free (stringTemp);
        free (classification);
        return NULL;
    }

    //    Add product info to QC log
    addProdInfoToFrgQcLog (fileNames, imageSizeInterf, dispResult, uvw, waveCal, calibrator, error, corrFrames);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot add product info to QC log");
        freeIauExchange (iauData);
        free (uvw);
        free (stringTemp);
        free (classification);
        return NULL;
    }

    //    Create the primary header extension
    createFrgPrimHead (fileNames, imageSizeInterf, dispResult, uvw, waveCal,
    		calibrator, error, corrFrames);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Primary Header extension");
        freeIauExchange (iauData);
        free (uvw);
        free (stringTemp);
        free (classification);
        return NULL;
    }

    //    At this stage output FITS file has probably been created with appropriate
    //    header. Now we create the tables and write the data in accordance with the IAU exchange
    writeFrgFitsFile (fileNames->outFitsName, iauData, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot write product FITS file");
        freeIauExchange (iauData);
        free (uvw);
        free (stringTemp);
        free (classification);
        return NULL;
    }

    if (diagnostic)cpl_msg_info(cpl_func,"Created Output FITS file: %s \n", fileNames->outFitsName);
    fprintf (midiReportPtr, "Created Output FITS file: %s \n", fileNames->outFitsName);

    //    Release memory

    /* Has to be free'ed by the caller */
    /*freeIauExchange (iauData);*/

    free (uvw);
    free (stringTemp);
    free (classification);

    return iauData;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createFrgPrimHead
* Input/Output: See function arguments to avoid duplication
* Description:  Creates the primary header
*
*
* History:
* 10-Feb-05     (csabet) Created
******************************************************************************/
void createFrgPrimHead (
    MidiFiles        *fileNames,
    ImageFormat        *sizeInterf,
    DispersedResult    *dispResult,
    UVW                *uvw,
    float           *waveCal,
    CalibratorParam    *calibrator,
    int                *error,
	CorrectedFrames   *corrFrames)
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "createFrgPrimHead";
    qfits_header    *outFitsHeader;
    FILE            *inFitsBatchPtr, *outFitsPtr;
    int             X;
    char            *textBuff, *stringQfits, *messageBuffer, *currentTime, *cleanString,
                    *stringTemp, *classification;
/*    unsigned int    stringLength;*/
    time_t          now;
    struct tm       *newTime;
    struct stat     buf;
    float            photA=0.0, photAErr=0.0, photB=0.0, photBErr=0.0, photI=0.0, photIErr=0.0,
                    photNet2=0.0, photNet2Err=0.0;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    *error = 0;

    //    Allocate memory
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Get current time/date
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Copy primary header from the raw file to the output header
    outFitsHeader = qfits_header_read (fileNames->inFitsName);
    if (outFitsHeader == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load header from the input FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    //    Now write all the required product keaywords to the output header
    //    -----------------------------------------------------------------
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TYPE", sizeInterf->obsType, "MIDI pipeline product type", "");

    if ((strcmp (batchTemplate, "SCI_PHOT_CALIB") == 0) ||    (strcmp (batchTemplate, "SCI_PHOT_SCIENCE") == 0))
        qfits_header_add (outFitsHeader, "HIERARCH ESO PRO CATG", "REDUCED_DISPERSED_SCIPHOT", "Pipeline product category", NULL);
    else
        qfits_header_add (outFitsHeader, "HIERARCH ESO PRO CATG", "REDUCED_DISPERSED", "Pipeline product category", NULL);
    
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO ARCFILE", fileNames->archFileName, "Arcfile name of first raw file", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO PIPEDATE", currentTime, "Pipeline run date", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO VERSION", MIDI_PIPE_VERSION, "Pipeline version", "");
    qfits_header_add (outFitsHeader, "PIPEFILE", fileNames->pipeFileName, "Pipeline product file name", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO DID", MIDI_QC_DIC_VERSION, "QC dictionary version", "");

    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    //    Loop through the list of files and write into the primary header
    X = 1;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        //    If classification is not given get it from the raw file
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL)
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d NAME", X);
        qfits_header_add (outFitsHeader, textBuff, messageBuffer, "FITS file name", "");
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d CATG", X++);
        qfits_header_add (outFitsHeader, textBuff, classification, "Observation Categoty", "");
    }
    fclose (inFitsBatchPtr);

/*
 * Not anymore needed as properly inserted  in the SOF and inherited by the cpl
 * saving functions
    //    Write the mask file
    sprintf (messageBuffer, "%s", fileNames->maskFileName);
    removePathName (messageBuffer, midiReportPtr);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO REC2 RAW1 NAME", messageBuffer, "Pipeline Mask File", "");

    //    Write the transfer function file
    if (strcmp (sizeInterf->obsCatg, "CALIB") == 0) sprintf (messageBuffer, "%s", fileNames->trfNameWrite);
    else sprintf (messageBuffer, "%s", fileNames->trfNameRead);
    removePathName (messageBuffer, midiReportPtr);
    stringLength = strlen (messageBuffer);
    sprintf (textBuff, "r.");
    strncat (textBuff, messageBuffer, stringLength-4);
    sprintf (messageBuffer, "%s_tpl_0001.dummy", textBuff);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO REC2 RAW2 NAME", messageBuffer, "Transfer Function File", "");
*/

    //    Scan parameters
    sprintf (messageBuffer, "%d", sizeInterf->numOfChannelsProcessed);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC CH PROCESSED", messageBuffer, "Number of chnnels processed", "");
    sprintf (messageBuffer, "%d", sizeInterf->numOfChannelsRejected);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC CH REJECTED", messageBuffer, "Number of channels rejected", "");

    //    Add Wavelengths
    for (X = 0; X < sizeInterf->iXWidth; X++)
    {
        if (badChannelList[X])
        {
            sprintf (textBuff, "HIERARCH ESO QC LAMBDA%d", X);
            sprintf (messageBuffer, "%s", UNAV);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Calibrated wavelength", "");
            continue;
        }

        sprintf (textBuff, "HIERARCH ESO QC LAMBDA%d", X);
        sprintf (messageBuffer, "%f", waveCal[X]);
        qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Calibrated wavelength", "");
    }

    //    Transfer functions
    if (dispResult->trfExists)
    {
        qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TRF", "AVAILABLE", "Flag for Transfer Function availability", NULL);
        if (strcmp (sizeInterf->obsCatg, "CALIB") == 0)
        {
            sprintf (messageBuffer, "%f", calibrator->calibVisSqrd);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC THEOVIS", messageBuffer, "Theoretical visibility", "");
            sprintf (messageBuffer, "%f", calibrator->calibVisSqrdErr);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC DTHEOVIS", messageBuffer, "(sigma) Error on theoretical visibility", "");
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB POS", "UNKNOWN", "Position of the calibrator entry", "");
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB NAME", calibrator->calibName, "Name of the calibrator entry", "");
            sprintf (messageBuffer, "%f", calibrator->calibRA);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB RA", messageBuffer, "Calibrator Right Ascension", "");
            sprintf (messageBuffer, "%f", calibrator->calibDEC);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DEC", messageBuffer, "Calibrator Declination", "");
            sprintf (messageBuffer, "%f", RAD_TO_ARCSEC * calibrator->calibDistance);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DIST", messageBuffer, "Calibrator Radial Distance in arcsec", "");
            sprintf (messageBuffer, "%f", calibrator->calibDiameter);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DIAM", messageBuffer, "Calibrator Diameter in milliarcsec", "");
            sprintf (messageBuffer, "%f", calibrator->calibDiameterErr);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB DDIAM", messageBuffer, "(sigma) Error on calibrator Diameter in milliarcsec", "");
            sprintf (messageBuffer, "%d", calibrator->calibFlag);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB FLAG", messageBuffer, "Calibrator quality flag", "");
            sprintf (messageBuffer, "%g", calibrator->calibFlux);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC CALIB FLUX", messageBuffer, "Calibrator N-band flux", "");
            sprintf (messageBuffer, "%f", calibrator->calibPblAverage);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL LENGTH", messageBuffer, "Baseline vector length", "");
            sprintf (messageBuffer, "%f", calibrator->calibParangAverage);
            qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL ANGLE", messageBuffer, "Baseline vector angle", "");

            if (strcmp(sizeInterf->grismId, "PRISM") == 0)
            {
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned1);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED1", messageBuffer, 
                                 "Binned transfer function at 8.64 micron (Ch 120-126)", "");
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned2);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED2", messageBuffer, 
                                 "Binned transfer function at 9.00 micron (Ch 113-121)", "");
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned3);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED3", messageBuffer, 
                                 "Binned transfer function at 10.46 micron (Ch 88-98)", "");
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned4);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED4", messageBuffer, 
                                 "Binned transfer function at 11.79 micron (Ch 60-72)", "");
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned5);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED5", messageBuffer, 
                                 "Binned transfer function at 12.80 micron (Ch 36-50)", "");
            }
            else
            {
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned1);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED1", messageBuffer, 
                                 "Binned transfer function at 8.64 micron (Ch 38-50)", "");
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned2);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED2", messageBuffer, 
                                 "Binned transfer function at 9.00 micron (Ch 52-68)", "");
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned3);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED3", messageBuffer, 
                                 "Binned transfer function at 10.46 micron (Ch 116-136)", "");
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned4);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED4", messageBuffer, 
                                 "Binned transfer function at 11.79 micron (Ch 176-200)", "");
               sprintf (messageBuffer, "%0.4f", dispResult->trfBinned5);
               qfits_header_add (outFitsHeader, "HIERARCH ESO QC TRF BINNED5", messageBuffer, 
                                 "Binned transfer function at 12.80 micron (Ch 217-245)", "");
            }
            

            for (X = 0; X < sizeInterf->iXWidth; X++)
            {
                if (badChannelList[X])
                {
                    sprintf (textBuff, "HIERARCH ESO QC T%d", X);
                    sprintf (messageBuffer, "%s", UNAV);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Measured transfer function", "");

                    sprintf (textBuff, "HIERARCH ESO QC DT%d", X);
                    sprintf (messageBuffer, "%s", UNAV);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error on measured transfer function", "");

                    sprintf (textBuff, "HIERARCH ESO QC COT%d", X);
                    sprintf (messageBuffer, "%s", UNAV);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Inverse transfer function", "");

                    sprintf (textBuff, "HIERARCH ESO QC DCOT%d", X);
                    sprintf (messageBuffer, "%s", UNAV);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error on inverse transfer function", "");
                }
                else
                {
                    sprintf (textBuff, "HIERARCH ESO QC T%d", X);
                    sprintf (messageBuffer, "%f", (dispResult->trf)[X]);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Measured transfer function", "");

                    sprintf (textBuff, "HIERARCH ESO QC DT%d", X);
                    sprintf (messageBuffer, "%f", (dispResult->trfErr)[X]);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error on measured transfer function", "");

                    sprintf (textBuff, "HIERARCH ESO QC COT%d", X);
                    sprintf (messageBuffer, "%f", 1.0/(dispResult->trf)[X]);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Inverse transfer function", "");

                    sprintf (textBuff, "HIERARCH ESO QC DCOT%d", X);
                    sprintf (messageBuffer, "%f", 1.0/(dispResult->trfErr)[X]);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error on inverse transfer function", "");
                }
            }
        }
        else
        {
            //    Add Applied Transfer Functions
            for (X = 0; X < sizeInterf->iXWidth; X++)
            {
                if (badChannelList[X])
                {
                    sprintf (textBuff, "HIERARCH ESO QC AT%d", X);
                    sprintf (messageBuffer, "%s", UNAV);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Applied transfer function", "");

                    sprintf (textBuff, "HIERARCH ESO QC DAT%d", X);
                    sprintf (messageBuffer, "%s", UNAV);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error on applied transfer function", "");
                }
                else
                {
                    sprintf (textBuff, "HIERARCH ESO QC AT%d", X);
                    sprintf (messageBuffer, "%f", (dispResult->trf)[X]);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Applied transfer function", "");

                    sprintf (textBuff, "HIERARCH ESO QC DAT%d", X);
                    sprintf (messageBuffer, "%f", (dispResult->trfErr)[X]);
                    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error on applied transfer function", "");
                }
            }
        }
    }
    else
    {
    	qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TRF", UNAV, "Flag for Transfer Function availability", NULL);
    	if (strcmp (sizeInterf->obsCatg, "CALIB") == 0)
    	{
    		sprintf (messageBuffer, "%f", calibrator->calibPblAverage);
    		qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL LENGTH", messageBuffer, "Baseline vector length", "");
    		sprintf (messageBuffer, "%f", calibrator->calibParangAverage);
    		qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL ANGLE", messageBuffer, "Baseline vector angle", "");

    	}
    }

    //    Add uncalibrated Normal Visibilities
    for (X = 0; X < sizeInterf->iXWidth; X++)
    {
        if (badChannelList[X])
        {
            sprintf (textBuff, "HIERARCH ESO QC UNCALV%d", X);
            sprintf (messageBuffer, "%s", UNAV);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Uncalibrated Normal Visibility", "");

            sprintf (textBuff, "HIERARCH ESO QC DUNCALV%d", X);
            sprintf (messageBuffer, "%s", UNAV);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error of uncalibrated Normal Visibility", "");
        }
        else
        {
            sprintf (textBuff, "HIERARCH ESO QC UNCALV%d", X);
            sprintf (messageBuffer, "%f", (dispResult->normVis2)[X]);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Uncalibrated Normal Visibility", "");

            sprintf (textBuff, "HIERARCH ESO QC DUNCALV%d", X);
            sprintf (messageBuffer, "%f", (dispResult->normVis2Err)[X]);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error of uncalibrated Normal Visibility", "");
        }
    }
    

    //    Add Photometry parameters
    for (X = 0; X < sizeInterf->iXWidth; X++)
    {
        if (!(badChannelList[X]))
        {
            photA += (dispResult->photomA)[0][X];
            photAErr += (dispResult->photomAErr)[0][X];
            photB += (dispResult->photomB)[0][X];
            photBErr += (dispResult->photomBErr)[0][X];
            photI += (dispResult->photomI)[0][X];
            photIErr += (dispResult->photomIErr)[0][X];
            photNet2 += (dispResult->photomNet2)[X];
            photNet2Err += (dispResult->photomNet2Err)[X];
        }
    }
    if (sizeInterf->numOfChannelsProcessed)
    {
        photA /= sizeInterf->numOfChannelsProcessed;
        photAErr /= sizeInterf->numOfChannelsProcessed;
        photB /= sizeInterf->numOfChannelsProcessed;
        photBErr /= sizeInterf->numOfChannelsProcessed;
        photI /= sizeInterf->numOfChannelsProcessed;
        photIErr /= sizeInterf->numOfChannelsProcessed;
        photNet2 /= sizeInterf->numOfChannelsProcessed;
        photNet2Err /= sizeInterf->numOfChannelsProcessed;
    }
    sprintf (textBuff, "HIERARCH ESO QC AV PHOTA");
    sprintf (messageBuffer, "%f", photA);
    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Average photometry", "");
    sprintf (textBuff, "HIERARCH ESO QC AV DPHOTA");
    sprintf (messageBuffer, "%f", photAErr);
    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error of Average photometry", "");
    sprintf (textBuff, "HIERARCH ESO QC AV PHOTB");
    sprintf (messageBuffer, "%f", photB);
    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Average photometry", "");
    sprintf (textBuff, "HIERARCH ESO QC AV DPHOTB");
    sprintf (messageBuffer, "%f", photBErr);
    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error of Average photometry", "");
    sprintf (textBuff, "HIERARCH ESO QC AV PHOTI");
    sprintf (messageBuffer, "%f", photI);
    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Average photometry", "");
    sprintf (textBuff, "HIERARCH ESO QC AV DPHOTI");
    sprintf (messageBuffer, "%f", photIErr);
    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error of Average photometry", "");
    sprintf (textBuff, "HIERARCH ESO QC AV PHOTN");
    sprintf (messageBuffer, "%f", photNet2);
    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Average photometry", "");
    sprintf (textBuff, "HIERARCH ESO QC AV DPHOTN");
    sprintf (messageBuffer, "%f", photNet2Err);
    qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error of Average photometry", "");


    //    Add Calibrated Visibilities
    if (dispResult->calibVisExists)
    {
        for (X = 0; X < sizeInterf->iXWidth; X++)
        {
            if (badChannelList[X])
            {
                sprintf (textBuff, "HIERARCH ESO QC CALV%d", X);
                sprintf (messageBuffer, "%s", UNAV);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Calibrated Visibility", "");

                sprintf (textBuff, "HIERARCH ESO QC DCALV%d", X);
                sprintf (messageBuffer, "%s", UNAV);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error of calibrated Visibility", "");
            }
            else
            {
                sprintf (textBuff, "HIERARCH ESO QC CALV%d", X);
                sprintf (messageBuffer, "%f", (dispResult->calibVis2)[X]);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Calibrated Visibility", "");

                sprintf (textBuff, "HIERARCH ESO QC DCALV%d", X);
                sprintf (messageBuffer, "%f", (dispResult->calibVis2Err)[X]);
                qfits_header_add (outFitsHeader, textBuff, messageBuffer, "(sigma) Error of calibrated Visibility", "");
            }
        }
    }


    //    Add Target keywords
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC TARG DIAM", "UNKNOWN", "Estimated diameter of the target (mas, Un.Disk model)", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC TARG DDIAM", "UNKNOWN", "(sigma) Error on estimated target diameter in milliarcsec", "");

    //    Add UVW. The values of uvw will be added during pipeline execution
    sprintf (messageBuffer, "%f", uvw->uCoord);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL UVW1", messageBuffer, "u component of UVW vector", "");
    sprintf (messageBuffer, "%f", uvw->vCoord);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL UVW2", messageBuffer, "v component of UVW vector", "");
    sprintf (messageBuffer, "%f", uvw->wCoord);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC BL UVW3", messageBuffer, "w component of UVW vector", "");

    //    Add binned photometry measurements
    if ((strcmp(sizeInterf->grismId, "PRISM") == 0) && (strcmp(sizeInterf->beamCombiner, "HIGH_SENS") == 0))
    {
        sprintf (messageBuffer, "%0.2f", dispResult->photomATarg1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TARG1", messageBuffer, "PhotA Targ1 Ch 113 - 121", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTarg1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TARG1", messageBuffer, "PhotB Targ1 Ch 113 - 121", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATarg2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TARG2", messageBuffer, "PhotA Targ2 Ch 88 - 98", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTarg2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TARG2", messageBuffer, "PhotB Targ2 Ch 88 - 98", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATarg3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TARG3", messageBuffer, "PhotA Targ3 Ch 36 - 50", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTarg3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TARG3", messageBuffer, "PhotB Targ3 Ch 36 - 50", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATotal1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TOTAL1", messageBuffer, "PhotA Total1 Ch 113 - 121", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTotal1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TOTAL1", messageBuffer, "PhotB Total1 Ch 113 - 121", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATotal2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TOTAL2", messageBuffer, "PhotA Total2 Ch 88 - 98", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTotal2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TOTAL2", messageBuffer, "PhotB Total2 Ch 88 - 98", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATotal3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TOTAL3", messageBuffer, "PhotA Total3 Ch 36 - 50", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTotal3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TOTAL3", messageBuffer, "PhotB Total3 Ch 36 - 50", "");
    }
    else
    {
        sprintf (messageBuffer, "%0.2f", dispResult->photomATarg1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TARG1", messageBuffer, "PhotA Targ1 Ch 52 - 68", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTarg1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TARG1", messageBuffer, "PhotB Targ1 Ch 52 - 68", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATarg2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TARG2", messageBuffer, "PhotA Targ2 Ch 116 - 136", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTarg2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TARG2", messageBuffer, "PhotB Targ2 Ch 116 - 136", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATarg3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TARG3", messageBuffer, "PhotA Targ3 Ch 217 - 244", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTarg3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TARG3", messageBuffer, "PhotB Targ3 Ch 217 - 244", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATotal1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TOTAL1", messageBuffer, "PhotA Total1 Ch 52 - 68", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTotal1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TOTAL1", messageBuffer, "PhotB Total1 Ch 52 - 68", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATotal2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TOTAL2", messageBuffer, "PhotA Total2 Ch 116 - 136", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTotal2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TOTAL2", messageBuffer, "PhotB Total2 Ch 116 - 136", "");

        sprintf (messageBuffer, "%0.2f", dispResult->photomATotal3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTA TOTAL3", messageBuffer, "PhotA Total3 Ch 217 - 244", "");
        sprintf (messageBuffer, "%0.2f", dispResult->photomBTotal3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC PHOTB TOTAL3", messageBuffer, "PhotB Total3 Ch 217 - 244", "");
    }

    //    Add binned uncalibrated visibilities
    if (strcmp(sizeInterf->grismId, "PRISM") == 0)
    {
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED1", messageBuffer, 
            "Binned uncal vis at 8.64 micron (Ch 120-126)", "");
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED2", messageBuffer, 
            "Binned uncal vis at 9.00 micron (Ch 113-121)", "");
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED3", messageBuffer, 
            "Binned uncal vis at 10.46 micron (Ch 88-98)", "");
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned4);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED4", messageBuffer, 
            "Binned uncal vis at 11.79 micron (Ch 60-72)", "");
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned5);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED5", messageBuffer, 
            "Binned uncal vis at 12.80 micron (Ch 36-50)", "");
    }
    else
    {
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED1", messageBuffer, 
            "Binned uncal vis at 8.64 micron (Ch 38-50)", "");
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED2", messageBuffer, 
            "Binned uncal vis at 9.00 micron (Ch 52-68)", "");
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED3", messageBuffer, 
            "Binned uncal vis at 10.46 micron (Ch 116-136)", "");
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned4);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED4", messageBuffer, 
            "Binned uncal vis at 11.79 micron (Ch 176-200)", "");
        sprintf (messageBuffer, "%0.4f", dispResult->uncalVisBinned5);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC UNCAL BINNED5", messageBuffer, 
            "Binned uncal vis at 12.80 micron (Ch 217-245)", "");
    }

    sprintf (messageBuffer, "%ld", corrFrames->CorrInterf);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC TARTYPE INTERF CHANGED", messageBuffer,
        "Number of type-modified frames", "");

    sprintf (messageBuffer, "%ld", corrFrames->CorrPhotomA);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC TARTYPE PHOTOMA CHANGED", messageBuffer,
        "Number of type-modified frames", "");

    sprintf (messageBuffer, "%ld", corrFrames->CorrPhotomB);
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC TARTYPE PHOTOMB CHANGED", messageBuffer,
        "Number of type-modified frames", "");

    //    Place Holders
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH1", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH2", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH3", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH4", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH5", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH6", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH7", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH8", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH9", "TBD", "TBD", "");

    //    Create the output fits file
    if (stat (fileNames->outFitsName, &buf) == 0)   // If the file exist delete it
        remove (fileNames->outFitsName);
    outFitsPtr = fopen (fileNames->outFitsName, "w");
    if (!outFitsPtr)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create output FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    //    Write header into the output file
    qfits_header_sort (&outFitsHeader);
    qfits_header_dump (outFitsHeader, outFitsPtr);
    qfits_header_destroy (outFitsHeader);
    fclose (outFitsPtr);

    //    release memory
    free (messageBuffer);
    free (currentTime);
    free (cleanString);
    free (stringTemp);
    free (classification);
    free (textBuff);

    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  addProdInfoToFrgQcLog
* Input/Output: See function arguments to avoid duplication
* Description:  Adds product information to QC log
*
*
* History:
* 09-Feb-05     (csabet) Created
******************************************************************************/
void addProdInfoToFrgQcLog (
    MidiFiles        *fileNames,        // In: File names
    ImageFormat        *sizeInterf,    // In: Interferometry data parameters
    DispersedResult    *dispResult,    // In: Dispersed result
    UVW                *uvw,            // In: UVW coordinates
    float           *freqCal,        // In: Frequency array
    CalibratorParam    *calibrator,    // In: Calibrator parameters
    int                *error,            // Ou: Error status
	CorrectedFrames   *corrFrames)
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "addProdInfoToFrgQcLog";
    int             X;
    FILE            *inFitsBatchPtr;
    char            *stringQfits, *messageBuffer, *currentTime, *textBuff, *cleanString,
                    *stringTemp, *classification;
    unsigned int    stringLength;
    time_t          now;
    struct tm       *newTime;
    float            photA=0.0, photAErr=0.0, photB=0.0, photBErr=0.0, photI=0.0, photIErr=0.0,
                    photNet2=0.0, photNet2Err=0.0;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset parameters
    *error = 0;

    //    Allocate memory
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Get current time/date
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Add keyword to QC log file
    fprintf (midiQcLogPtr, "PRO.TYPE            \"%s\" \n", sizeInterf->obsType);

    if ((strcmp (batchTemplate, "SCI_PHOT_CALIB") == 0) ||    (strcmp (batchTemplate, "SCI_PHOT_SCIENCE") == 0))
        fprintf (midiQcLogPtr, "PRO.CATG            \"REDUCED_DISPERSED_SCIPHOT\" \n");
    else
        fprintf (midiQcLogPtr, "PRO.CATG            \"REDUCED_DISPERSED\" \n");

    fprintf (midiQcLogPtr, "PRO.ARCFILE         \"%s\" \n", fileNames->archFileName);
    fprintf (midiQcLogPtr, "PRO.PIPEDATE        \"%s\" \n", currentTime);
    fprintf (midiQcLogPtr, "PRO.VERSION         \"%s\" \n", MIDI_PIPE_VERSION);
    fprintf (midiQcLogPtr, "PRO.PIPEFILE        \"%s\" \n", fileNames->pipeFileName);
    fprintf (midiQcLogPtr, "PRO.DID             \"%s\" \n", MIDI_QC_DIC_VERSION);

    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    //    Loop through the list of files and write into the QC log
    X = 1;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        //    If classification is not given get it from the raw file
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL)
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.NAME  \"%s\" \n", X, messageBuffer);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.CATG  \"%s\" \n", X++, classification);
    }
    fclose (inFitsBatchPtr);

    //    Add name of Mask file to QC log file
    sprintf (messageBuffer, "%s", fileNames->maskFileName);
    removePathName (messageBuffer, midiReportPtr);
    fprintf (midiQcLogPtr, "PRO.REC2.RAW1.NAME  \"%s\" \n", messageBuffer);

    //    Add name of Transfer Function file to QC log file
    if (strcmp (sizeInterf->obsCatg, "CALIB") == 0) sprintf (messageBuffer, "%s", fileNames->trfNameWrite);
    else sprintf (messageBuffer, "%s", fileNames->trfNameRead);
    removePathName (messageBuffer, midiReportPtr);
    stringLength = strlen (messageBuffer);
    sprintf (textBuff, "r.");
    strncat (textBuff, messageBuffer, stringLength-4);
    sprintf (messageBuffer, "%s_tpl_0001.dummy", textBuff);
    fprintf (midiQcLogPtr, "PRO.REC2.RAW2.NAME  \"%s\" \n", messageBuffer);

    //    Scan parameters
    fprintf (midiQcLogPtr, "QC.CH.PROCESSED  %d \n", sizeInterf->numOfChannelsProcessed);
    fprintf (midiQcLogPtr, "QC.CH.REJECTED   %d \n", sizeInterf->numOfChannelsRejected);

    //    Add Wavelengths
    for (X = 0; X < sizeInterf->iXWidth; X++)
    {
        if (badChannelList[X])
        {
            fprintf (midiQcLogPtr, "QC.LAMBDA%d            %s \n", X, UNAV);
            continue;
        }
            
        fprintf (midiQcLogPtr, "QC.LAMBDA%d            %f \n", X, freqCal[X]);
    }
    
    //    Transfer functions
    if (dispResult->trfExists)
    {
        if (strcmp (sizeInterf->obsCatg, "CALIB") == 0)
        {
            //    Add Calibrator keywords
            fprintf (midiQcLogPtr, "QC.THEOVIS          %f \n", calibrator->calibVisSqrd);
            fprintf (midiQcLogPtr, "QC.DTHEOVIS         %f \n", calibrator->calibVisSqrdErr);
            fprintf (midiQcLogPtr, "QC.CALIB.POS        \"UNKNOWN\" \n");
            fprintf (midiQcLogPtr, "QC.CALIB.NAME       \"%s\" \n", calibrator->calibName);
            fprintf (midiQcLogPtr, "QC.CALIB.RA         %f \n", calibrator->calibRA);
            fprintf (midiQcLogPtr, "QC.CALIB.DEC        %f \n", calibrator->calibDEC);
            fprintf (midiQcLogPtr, "QC.CALIB.DIST       %f \n", RAD_TO_ARCSEC * calibrator->calibDistance);
            fprintf (midiQcLogPtr, "QC.CALIB.DIAM       %f \n", calibrator->calibDiameter);
            fprintf (midiQcLogPtr, "QC.CALIB.DDIAM      %f \n", calibrator->calibDiameterErr);
            fprintf (midiQcLogPtr, "QC.CALIB.FLAG       %d \n", calibrator->calibFlag);
            fprintf (midiQcLogPtr, "QC.CALIB.FLUX       %g \n", calibrator->calibFlux);
            fprintf (midiQcLogPtr, "QC.BL.LENGTH  %f \n", calibrator->calibPblAverage);
            fprintf (midiQcLogPtr, "QC.BL.ANGLE   %f \n", calibrator->calibParangAverage);

            //    Add binned Transfer Function
            fprintf (midiQcLogPtr, "QC.TRF.BINNED1  %0.4f \n", dispResult->trfBinned1);
            fprintf (midiQcLogPtr, "QC.TRF.BINNED2  %0.4f \n", dispResult->trfBinned2);
            fprintf (midiQcLogPtr, "QC.TRF.BINNED3  %0.4f \n", dispResult->trfBinned3);
            fprintf (midiQcLogPtr, "QC.TRF.BINNED4  %0.4f \n", dispResult->trfBinned4);
            fprintf (midiQcLogPtr, "QC.TRF.BINNED5  %0.4f \n", dispResult->trfBinned5);

        
            for (X = 0; X < sizeInterf->iXWidth; X++)
            {
                if (badChannelList[X])
                {
                    fprintf (midiQcLogPtr, "QC.T%d              %s \n", X, UNAV);
                    fprintf (midiQcLogPtr, "QC.DT%d             %s \n", X, UNAV);
                    fprintf (midiQcLogPtr, "QC.COT%d            %s \n", X, UNAV);
                    fprintf (midiQcLogPtr, "QC.DCOT%d           %s \n", X, UNAV);
                    continue;
                }
            
                fprintf (midiQcLogPtr, "QC.T%d              %f \n", X, (dispResult->trf)[X]);
                fprintf (midiQcLogPtr, "QC.DT%d             %f \n", X, (dispResult->trfErr)[X]);
                fprintf (midiQcLogPtr, "QC.COT%d            %f \n", X, 1.0/(dispResult->trf)[X]);
                fprintf (midiQcLogPtr, "QC.DCOT%d           %f \n", X, 1.0/(dispResult->trfErr)[X]);
            }
        }
        else
        {
            //    Add Applied Transfer Functions
            for (X = 0; X < sizeInterf->iXWidth; X++)
            {
                if (badChannelList[X])
                {
                    fprintf (midiQcLogPtr, "QC.AT%d             %s \n", X, UNAV);
                    fprintf (midiQcLogPtr, "QC.DAT%d            %s \n", X, UNAV);
                }
                else
                {
                    fprintf (midiQcLogPtr, "QC.AT%d             %f \n", X, (dispResult->trf)[X]);
                    fprintf (midiQcLogPtr, "QC.DAT%d            %f \n", X, (dispResult->trfErr)[X]);
                }
            }
        }
    }

    //    Add uncalibrated Normal Visibilities
    for (X = 0; X < sizeInterf->iXWidth; X++)
    {
        if (badChannelList[X])
        {
            fprintf (midiQcLogPtr, "QC.UNCALV%d         %s \n", X, UNAV);
            fprintf (midiQcLogPtr, "QC.DUNCALV%d        %s \n", X, UNAV);
        }
        else
        {
            fprintf (midiQcLogPtr, "QC.UNCALV%d         %f \n", X, (dispResult->normVis2)[X]);
            fprintf (midiQcLogPtr, "QC.DUNCALV%d        %f \n", X, (dispResult->normVis2Err)[X]);
        }
    }

    //    Add Photometry parameters
    for (X = 0; X < sizeInterf->iXWidth; X++)
    {
        if (!(badChannelList[X]))
        {
            photA += (dispResult->photomA)[0][X];
            photAErr += (dispResult->photomAErr)[0][X];
            photB += (dispResult->photomB)[0][X];
            photBErr += (dispResult->photomBErr)[0][X];
            photI += (dispResult->photomI)[0][X];
            photIErr += (dispResult->photomIErr)[0][X];
            photNet2 += (dispResult->photomNet2)[X];
            photNet2Err += (dispResult->photomNet2Err)[X];
        }
    }
    if (sizeInterf->numOfChannelsProcessed)
    {
        photA /= sizeInterf->numOfChannelsProcessed;
        photAErr /= sizeInterf->numOfChannelsProcessed;
        photB /= sizeInterf->numOfChannelsProcessed;
        photBErr /= sizeInterf->numOfChannelsProcessed;
        photI /= sizeInterf->numOfChannelsProcessed;
        photIErr /= sizeInterf->numOfChannelsProcessed;
        photNet2 /= sizeInterf->numOfChannelsProcessed;
        photNet2Err /= sizeInterf->numOfChannelsProcessed;
    }
    fprintf (midiQcLogPtr, "QC.AV.PHOTA         %f \n", photA);
    fprintf (midiQcLogPtr, "QC.AV.DPHOTA        %f \n", photAErr);
    fprintf (midiQcLogPtr, "QC.AV.PHOTB         %f \n", photB);
    fprintf (midiQcLogPtr, "QC.AV.DPHOTB        %f \n", photBErr);
    fprintf (midiQcLogPtr, "QC.AV.PHOTI         %f \n", photI);
    fprintf (midiQcLogPtr, "QC.AV.DPHOTI        %f \n", photIErr);
    fprintf (midiQcLogPtr, "QC.AV.PHOTN         %f \n", photNet2);
    fprintf (midiQcLogPtr, "QC.AV.DPHOTN        %f \n", photNet2Err);

    //    Add Calibrated Visibilities
    if (dispResult->calibVisExists)
    {
        for (X = 0; X < sizeInterf->iXWidth; X++)
        {
            if (badChannelList[X])
            {
                fprintf (midiQcLogPtr, "QC.CALV%d           %s \n", X, UNAV);
                fprintf (midiQcLogPtr, "QC.DCALV%d          %s \n", X, UNAV);
            }
            else
            {
                fprintf (midiQcLogPtr, "QC.CALV%d           %f \n", X, (dispResult->calibVis2)[X]);
                fprintf (midiQcLogPtr, "QC.DCALV%d          %f \n", X, (dispResult->calibVis2Err)[X]);
            }
        }
    }

    //    Add Target keywords
    fprintf (midiQcLogPtr, "QC.TARG.DIAM        \"UNKNOWN\" \n");
    fprintf (midiQcLogPtr, "QC.TARG.DDIAM       \"UNKNOWN\" \n");

    //    Add UVW. The values of uvw will be added during pipeline execution
    fprintf (midiQcLogPtr, "QC.BL.UVW1    %f \n", uvw->uCoord);
    fprintf (midiQcLogPtr, "QC.BL.UVW2    %f \n", uvw->vCoord);
    fprintf (midiQcLogPtr, "QC.BL.UVW3    %f \n", uvw->wCoord);

    //    Add binned photometry measurements
    if (strcmp(sizeInterf->beamCombiner, "HIGH_SENS") == 0)
    {
        fprintf (midiQcLogPtr, "QC.PHOTA.TARG1  %0.2f \n", dispResult->photomATarg1);
        fprintf (midiQcLogPtr, "QC.PHOTB.TARG1  %0.2f \n", dispResult->photomBTarg1);

        fprintf (midiQcLogPtr, "QC.PHOTA.TARG2  %0.2f \n", dispResult->photomATarg2);
        fprintf (midiQcLogPtr, "QC.PHOTB.TARG2  %0.2f \n", dispResult->photomBTarg2);

        fprintf (midiQcLogPtr, "QC.PHOTA.TARG3  %0.2f \n", dispResult->photomATarg3);
        fprintf (midiQcLogPtr, "QC.PHOTB.TARG3  %0.2f \n", dispResult->photomBTarg3);

        fprintf (midiQcLogPtr, "QC.PHOTA.TOTAL1  %0.2f \n", dispResult->photomATotal1);
        fprintf (midiQcLogPtr, "QC.PHOTB.TOTAL1  %0.2f \n", dispResult->photomBTotal1);

        fprintf (midiQcLogPtr, "QC.PHOTA.TOTAL2  %0.2f \n", dispResult->photomATotal2);
        fprintf (midiQcLogPtr, "QC.PHOTB.TOTAL2  %0.2f \n", dispResult->photomBTotal2);

        fprintf (midiQcLogPtr, "QC.PHOTA.TOTAL3  %0.2f \n", dispResult->photomATotal3);
        fprintf (midiQcLogPtr, "QC.PHOTB.TOTAL3  %0.2f \n", dispResult->photomBTotal3);
    }

    //    Add binned uncalibrated visibilities
    fprintf (midiQcLogPtr, "QC.UNCAL.BINNED1  %0.4f \n", dispResult->uncalVisBinned1);
    fprintf (midiQcLogPtr, "QC.UNCAL.BINNED2  %0.4f \n", dispResult->uncalVisBinned2);
    fprintf (midiQcLogPtr, "QC.UNCAL.BINNED3  %0.4f \n", dispResult->uncalVisBinned3);
    fprintf (midiQcLogPtr, "QC.UNCAL.BINNED4  %0.4f \n", dispResult->uncalVisBinned4);
    fprintf (midiQcLogPtr, "QC.UNCAL.BINNED5  %0.4f \n", dispResult->uncalVisBinned5);

    //    Add number of changed target types
    fprintf (midiQcLogPtr, "QC.TARTYPE.INTERF.CHANGED   %ld \n", corrFrames->CorrInterf );
    fprintf (midiQcLogPtr, "QC.TARTYPE.PHOTOMA.CHANGED  %ld \n", corrFrames->CorrPhotomA );
    fprintf (midiQcLogPtr, "QC.TARTYPE.PHOTOMB.CHANGED  %ld \n", corrFrames->CorrPhotomB );

    
    //    Release memory
    free (messageBuffer);
    free (currentTime);
    free (textBuff);
    free (cleanString);
    free (stringTemp);
    free (classification);

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  loadFrgOutputData
* Input/Output: See function arguments to avoid duplication
* Description:  Loads MIDI product data into appropriate data structures
*
*
* History:
* 11-Feb-05     (csabet) Created
******************************************************************************/
void loadFrgOutputData (
    char            *obsCategory,    // In: Observation category
    char            *fileName,        // In: Input file name
    float           *waveCal,        // In: array of frequencies
    DispersedResult    *dispResult,    // In: Dispersed result
    IauExchange        *iauData,        // Ou: IAU Exchange data structure
    UVW                *uvw,            // Ou: Coordinates of the uvw
    int                *error)

{
    //    Local Declarations
    //    ------------------
    const char        routine[] = "loadFrgOutputData";
    int                i, irec, iwave, itarg, mjdObs, *selection, extNumber;
    float            equinox;
    char            *stringQfits, *emptyString, *buffer, *observationData, *arrayName,
                    *instName, *cleanString;
    double            raEp0, decEp0, integrationTime;
    qfits_table        *arrayGeometry;
/*     char               *qfitsRow=NULL; */
    short            *qfitsRow_int, targetId = 1;
    float            *qfitsRow_flt, nullFloat;
    double            *qfitsRow_dbl, utcTime;

    cpl_table      * table    =NULL;       
    char          ** tel_name =NULL;
    char          ** sta_name =NULL;



    //    Algorithm
    //    ---------
    *error = 0;
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    emptyString = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    buffer = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    observationData = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    arrayName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    instName = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Initialize values
    strcpy (emptyString, "NULL");
    set_fnan (&nullFloat);

    //    Get general parameters from the Header of the input FITS file
    //    -------------------------------------------------------------
    stringQfits = qfits_query_hdr (fileName, "DATE-OBS");
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", observationData);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", observationData);
    }

    stringQfits = qfits_query_hdr (fileName, "EXPTIME");
    if (stringQfits == NULL) integrationTime = nullFloat;
    else sscanf (stringQfits, "%lf", &integrationTime);

    extNumber = getFitsExtensionNumber (fileName, "ARRAY_GEOMETRY", error);
    stringQfits = qfits_query_ext (fileName, "ARRNAME", extNumber);
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", arrayName);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", arrayName);
    }

    stringQfits = qfits_query_hdr (fileName, "INSTRUME");
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", instName);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", instName);
    }

    stringQfits = qfits_query_hdr (fileName, "RA");
    if (stringQfits == NULL) raEp0 = nullFloat;
    else sscanf (stringQfits, "%lf", &raEp0);

    stringQfits = qfits_query_hdr (fileName, "DEC");
    if (stringQfits == NULL) decEp0 = nullFloat;
    else sscanf (stringQfits, "%lf", &decEp0);

    stringQfits = qfits_query_hdr (fileName, "EQUINOX");
    if (stringQfits == NULL) equinox = nullFloat;
    else sscanf (stringQfits, "%f", &equinox);

    stringQfits = qfits_query_hdr (fileName, "MJD-OBS");
    if (stringQfits == NULL) mjdObs = nullFloat;
    else sscanf (stringQfits, "%d", &mjdObs);

    stringQfits = qfits_query_hdr (fileName, "UTC");
    if (stringQfits == NULL) utcTime = nullFloat;
    else sscanf (stringQfits, "%lf", &utcTime);

    //    Load info for OI_ARRAY table
    //    ----------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading OI_ARRAY data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading OI_ARRAY data ... \n");

    //    Get the required parameters from the ARRAY_GEOMETRY extension of the input FITS file
    sscanf (arrayName, "%s", iauData->array->arrname);

    //    Get extension number
    extNumber = getFitsExtensionNumber (fileName, "ARRAY_GEOMETRY", error);
    
    stringQfits = qfits_query_ext (fileName, "FRAME", extNumber);
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", iauData->array->frame);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", iauData->array->frame);
    }
    
    stringQfits = qfits_query_ext (fileName, "ARRAYX", extNumber);
    if (stringQfits == NULL) iauData->array->arrayx = nullFloat;
    else sscanf (stringQfits, "%lf", &(iauData->array->arrayx));

    stringQfits = qfits_query_ext (fileName, "ARRAYY", extNumber);
    if (stringQfits == NULL) iauData->array->arrayy = nullFloat;
    else sscanf (stringQfits, "%lf", &(iauData->array->arrayy));

    stringQfits = qfits_query_ext (fileName, "ARRAYZ", extNumber);
    if (stringQfits == NULL) iauData->array->arrayz = nullFloat;
    else sscanf (stringQfits, "%lf", &(iauData->array->arrayz));

    //    Open the table
    arrayGeometry = qfits_table_open (fileName, extNumber);
    
    
    table = cpl_table_load(fileName, extNumber, 1);
    
    if (cpl_table_has_column(table, "TEL_NAME")) tel_name=cpl_table_get_data_string(table, "TEL_NAME");
    if (cpl_table_has_column(table, "STA_NAME")) sta_name=cpl_table_get_data_string(table, "STA_NAME");
    
    
/*     for (i=0; i<cpl_table_get_nrow(table);i++){ */
/*        cpl_msg_info(cpl_func, "tel_name: %s  sta_name: %s",tel_name[i], sta_name[i]); */
/*     } */
    

    


    //    Load info for OI_ARRAY table
    //    ----------------------------
    selection = (int *) calloc (iauData->array->nelement, sizeof (int));
    for (i = 0; i < iauData->array->nelement; i++)
    {
        selection[i] = 1;
        sprintf ((iauData->array->elem)[i].tel_name, "%s", tel_name[i]);        
        sprintf ((iauData->array->elem)[i].sta_name, "%s", sta_name[i]);        
        if (diagnostic > 2){
           cpl_msg_info(cpl_func,"iauData->array->elem)[i].tel_name: %s tel_name[i] %s", (iauData->array->elem)[i].tel_name,tel_name[i]); 
           cpl_msg_info(cpl_func,"iauData->array->elem)[i].sta_name: %s sta_name[i] %s", (iauData->array->elem)[i].sta_name,sta_name[i]);
        }
 

/*        qfitsRow = (char *) (qfits_query_column (arrayGeometry, 0, selection)); */
/*         if (diagnostic > 2)cpl_msg_info(cpl_func,"qfitsRow    = %s \n", qfitsRow); */
/*         removeNewLine (qfitsRow, cleanString); */
/*         if (diagnostic > 2)cpl_msg_info(cpl_func,"cleanString = %s \n", cleanString); */
/*         sprintf ((iauData->array->elem)[i].tel_name, "%s", cleanString); */
/*         free (qfitsRow); */

        
/*         qfitsRow = (char *) (qfits_query_column (arrayGeometry, 1, selection)); */
/*         removeNewLine (qfitsRow, cleanString); */
/*         sprintf ((iauData->array->elem)[i].sta_name, "%s", cleanString); */
/*         free (qfitsRow); */


        qfitsRow_int = (short *) (qfits_query_column (arrayGeometry, 2, selection));
        (iauData->array->elem)[i].sta_index = qfitsRow_int[0];
        free (qfitsRow_int);

        qfitsRow_flt = (float *) (qfits_query_column (arrayGeometry, 3, selection));
        (iauData->array->elem)[i].diameter = qfitsRow_flt[0];
        free (qfitsRow_flt);

        qfitsRow_dbl = (double *) (qfits_query_column (arrayGeometry, 4, selection));
        (iauData->array->elem)[i].staxyz[0] = qfitsRow_dbl[0];
        (iauData->array->elem)[i].staxyz[1] = qfitsRow_dbl[1];
        (iauData->array->elem)[i].staxyz[2] = qfitsRow_dbl[2];
        free (qfitsRow_dbl);

        selection[i] = 0;
    }
/*     Free table memory */
    cpl_table_delete(table);    

    sprintf (iauData->array->revision, "%s", IAUEXCHANGE_VERSION);
    free (selection);
    qfits_table_close (arrayGeometry);

    //    Load info for OI_TARGET table
    //    -----------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading OI_TARGET data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading OI_TARGET data ... \n");

    stringQfits = qfits_query_hdr (fileName, "HIERARCH ESO OBS TARG NAME");
    if (stringQfits == NULL)
        sscanf (emptyString, "%s", buffer);
    else
    {
        cleanUpString (stringQfits, cleanString);
        sscanf (cleanString, "%s", buffer);
    }

    for (itarg = 0; itarg < iauData->targets->ntarget; itarg++)
    {
        (iauData->targets->targ)[itarg].target_id = targetId;
        sscanf (buffer, "%s", (iauData->targets->targ)[itarg].target);
        (iauData->targets->targ)[itarg].raep0 = raEp0;
        (iauData->targets->targ)[itarg].decep0 = decEp0;
        (iauData->targets->targ)[itarg].equinox = equinox;
        (iauData->targets->targ)[itarg].ra_err = nullFloat;
        (iauData->targets->targ)[itarg].dec_err = nullFloat;
        (iauData->targets->targ)[itarg].sysvel = nullFloat;
        sscanf (emptyString, "%s", (iauData->targets->targ)[itarg].veltyp);
        sscanf (emptyString, "%s", (iauData->targets->targ)[itarg].veldef);
        (iauData->targets->targ)[itarg].pmra = nullFloat;
        (iauData->targets->targ)[itarg].pmdec = nullFloat;
        (iauData->targets->targ)[itarg].pmra_err = nullFloat;
        (iauData->targets->targ)[itarg].pmdec_err = nullFloat;
        (iauData->targets->targ)[itarg].parallax = nullFloat;
        (iauData->targets->targ)[itarg].para_err = nullFloat;
        sscanf (emptyString, "%s", (iauData->targets->targ)[itarg].spectyp);
    }
    sprintf (iauData->targets->revision, "%s", IAUEXCHANGE_VERSION);


    //    Load info for OI_WAVELENGTH table
    //    ---------------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading OI_WAVELENGTH data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading OI_WAVELENGTH data ... \n");

    //    Compute the parameters
    sscanf (instName, "%s", iauData->wavelength->insname);
    for (i = 0; i < (iauData->wavelength->nwave-1); i++)
    {
        (iauData->wavelength->eff_wave)[i] = 1.e-6 * waveCal[i];
        (iauData->wavelength->eff_band)[i] =  1.e-6 * fabs ((waveCal[1+i] - waveCal[i]));
    }
    //    Final row
    (iauData->wavelength->eff_wave)[iauData->wavelength->nwave-1] = 1.e-6 * 
        waveCal[iauData->wavelength->nwave-1];
    (iauData->wavelength->eff_band)[iauData->wavelength->nwave-1] =
        (iauData->wavelength->eff_band)[iauData->wavelength->nwave-2];
    sprintf (iauData->wavelength->revision, "%s", IAUEXCHANGE_VERSION);

    //    Get the uvw parameters
    computeUVW ((iauData->array->elem)[0].staxyz[0], (iauData->array->elem)[0].staxyz[1], (iauData->array->elem)[0].staxyz[2], 
        (iauData->array->elem)[1].staxyz[0], (iauData->array->elem)[1].staxyz[1], (iauData->array->elem)[1].staxyz[2],
        mjdObs, raEp0, decEp0, uvw);

    //    Load info for OI_VIS table
    //    --------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading OI_VIS data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading OI_VIS data ... \n");

    //    Get the required parameters from the FITS file
    sscanf (observationData, "%s", iauData->vis->date_obs);
    sscanf (arrayName, "%s", iauData->vis->arrname);
    sscanf (instName, "%s", iauData->vis->insname);
    for (irec = 0; irec < iauData->vis->numrec; irec++)
    {
        (iauData->vis->record)[irec].target_id = targetId;
        (iauData->vis->record)[irec].time = utcTime;
        (iauData->vis->record)[irec].mjd = mjdObs;
        (iauData->vis->record)[irec].int_time = integrationTime;
        (iauData->vis->record)[irec].ucoord = uvw->uCoord;
        (iauData->vis->record)[irec].vcoord = uvw->vCoord;
        (iauData->vis->record)[irec].sta_index[0] = (iauData->array->elem)[0].sta_index;
        (iauData->vis->record)[irec].sta_index[1] = (iauData->array->elem)[1].sta_index;

        if ((strcmp (obsCategory, "SCIENCE") == 0) && dispResult->calibVisExists)
        {
            for (iwave = 0; iwave < iauData->wavelength->nwave; iwave++)
            {
                if (badChannelList[iwave])
                {
                    (iauData->vis->record)[irec].visamp[iwave] = nullFloat;
                    (iauData->vis->record)[irec].visamperr[iwave] = nullFloat;
                }
                else
                {
                    (iauData->vis->record)[irec].visamp[iwave] = sqrt ((dispResult->calibVis2)[iwave]);
                    (iauData->vis->record)[irec].visamperr[iwave] = sqrt ((dispResult->calibVis2Err)[iwave]);
                }                
                (iauData->vis->record)[irec].visphi[iwave] = nullFloat;
                (iauData->vis->record)[irec].visphierr[iwave] = nullFloat;
            }
        }
        else
        {
            for (iwave = 0; iwave < iauData->wavelength->nwave; iwave++)
            {
                if (badChannelList[iwave])
                {
                    (iauData->vis->record)[irec].visamp[iwave] = nullFloat;
                    (iauData->vis->record)[irec].visamperr[iwave] = nullFloat;
                }
                else
                {
                    (iauData->vis->record)[irec].visamp[iwave] = sqrt ((dispResult->normVis2)[iwave]);
                    (iauData->vis->record)[irec].visamperr[iwave] = sqrt ((dispResult->normVis2Err)[iwave]);
                }                
                (iauData->vis->record)[irec].visphi[iwave] = nullFloat;
                (iauData->vis->record)[irec].visphierr[iwave] = nullFloat;
            }
        }
        sprintf ((iauData->vis->record)[irec].flag, "%s", "FALSE");
    }
    sprintf (iauData->vis->revision, "%s", IAUEXCHANGE_VERSION);
    iauData->vis->nwave = iauData->wavelength->nwave;


    //    Load info for OI_VIS2 table
    //    ---------------------------
    if (diagnostic)cpl_msg_info(cpl_func,"Loading OI_VIS2 data ... \n");
    if (diagnostic) fprintf (midiReportPtr, "Loading OI_VIS2 data ... \n");

    //    Get the required parameters from the FITS file
    sscanf (observationData, "%s", iauData->vis2->date_obs);
    sscanf (arrayName, "%s", iauData->vis2->arrname);
    sscanf (instName, "%s", iauData->vis2->insname);
    for(irec = 0; irec < iauData->vis2->numrec; irec++)
    {
        (iauData->vis2->record)[irec].target_id = targetId;
        (iauData->vis2->record)[irec].time = utcTime;
        (iauData->vis2->record)[irec].mjd = mjdObs;
        (iauData->vis2->record)[irec].int_time = integrationTime;
        (iauData->vis2->record)[irec].ucoord = uvw->uCoord;
        (iauData->vis2->record)[irec].vcoord = uvw->vCoord;
        (iauData->vis2->record)[irec].sta_index[0] = (iauData->array->elem)[0].sta_index;;
        (iauData->vis2->record)[irec].sta_index[1] = (iauData->array->elem)[1].sta_index;;

        if ((strcmp (obsCategory, "SCIENCE") == 0) && dispResult->calibVisExists)
        {
            for (iwave = 0; iwave < iauData->wavelength->nwave; iwave++)
            {
                if (badChannelList[iwave])
                {
                    (iauData->vis2->record)[irec].vis2data[iwave] = nullFloat;
                    (iauData->vis2->record)[irec].vis2err[iwave] = nullFloat;
                }
                else
                {
                    (iauData->vis2->record)[irec].vis2data[iwave] = (dispResult->calibVis2)[iwave];
                    (iauData->vis2->record)[irec].vis2err[iwave] = (dispResult->calibVis2Err)[iwave];
                }                
            }
        }
        else
        {
            for (iwave = 0; iwave < iauData->wavelength->nwave; iwave++)
            {
                if (badChannelList[iwave])
                {
                    (iauData->vis2->record)[irec].vis2data[iwave] = nullFloat;
                    (iauData->vis2->record)[irec].vis2err[iwave] = nullFloat;
                }
                else
                {
                    (iauData->vis2->record)[irec].vis2data[iwave] = (dispResult->normVis2)[iwave];
                    (iauData->vis2->record)[irec].vis2err[iwave] = (dispResult->normVis2Err)[iwave];
                }
            }
        }
        sprintf ((iauData->vis2->record)[irec].flag, "%s", "FALSE");
    }
    sprintf (iauData->vis2->revision, "%s", IAUEXCHANGE_VERSION);
    iauData->vis2->nwave = iauData->wavelength->nwave;

    //    Release memory
    free (instName);
    free (arrayName);
    free (observationData);
    free (emptyString);
    free (buffer);
    free (cleanString);

    return;

}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  writeFrgFitsFile
* Input/Output: See function arguments to avoid duplication
* Description:  Writes and creates appropriate tables into a new fits file in
*               accordance with the IAU format
*
* History:
* 11-Feb-05     (csabet) Created
******************************************************************************/
void writeFrgFitsFile (
    char        *outFitsName,
    IauExchange    *iauData,
    int            *error)
{
    //    Local Declarations
    //    ------------------
    const char  routine[] = "writeFrgFitsFile";

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    if (diagnostic)cpl_msg_info(cpl_func,"Writing data into the output FITS file \n\n");
    if (diagnostic) fprintf (midiReportPtr, "Writing data into the output FITS file \n\n");

    //    Reset parameters
    *error = 0;

    //    Write OI_ARRAY table
    writeOiArray (outFitsName, iauData->array, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot write OI_ARRAY in the output FITS file");
        return;
    }

    //    Write OI_TARGET table
    writeOiTarget (outFitsName, iauData->targets, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot write OI_TARGET in the output FITS file");
        return;
    }

    //    Write OI_WAVELENGTH table
    writeOiWavelength (outFitsName, iauData->wavelength, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot write OI_WAVELENGTH in the output FITS file");
        return;
    }

    //    Write OI_VIS table
    writeOiVis (outFitsName, iauData->vis, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot write OI_VIS in the output FITS file");
        return;
    }

    //    Write OI_VIS2 table
    writeOiVis2 (outFitsName, iauData->vis2, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot write OI_VIS2 in the output FITS file");
        return;
    }

    return;

}
/*****************************************************************************/

