
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  fitsAnalysisFrgHS.c
* Description:  Contains all routines for FITS format analysis
*
* History:      
* 14-Jun-02     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "fft.h"
#include "errorHandling.h"
#include "memoryHandling.h"
#include "midiFitsUtility.h"
#include "fitsAnalysisFrgHS.h"

/**********************************************************
*   Global Variables 
**********************************************************/



/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  analyseFitsFrgHS
* Input/Output: See function arguments to avoid duplication
* Description:  Analyses all the FITS files in the given list. This routine
*                determines if the FITS list contains useful data files. If
*                not it will discard the list. This routine is used in GIGH_SENS
*                mode
*
* History:      
* 22-Jul-03     (csabet) Created
******************************************************************************/
void analyseFitsFrgHS (
    MidiFiles    *fileNames,            // IO: Pointer to midi files structure
    ImageFormat    *formatInterf,        // Ou: Size for Interferometry file
    ImageFormat    *formatPhotomA,        // Ou: Size for Photometry A file
    ImageFormat    *formatPhotomB,        // Ou: Size for Photometry B file
    int            *error)                // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char            routine[] = "analyseFitsFrgHS";
    char                *fileTemp, *classification;
    FILE                *inFitsBatchPtr=NULL;
    int                    extNumOfImagingDataFile, aprioriDataIsDisp=0;
    ImageFormat            *localFormat=NULL;
    char                *cleanString=NULL, *aprioriCatg=NULL;
    unsigned int        numOfInterf, numOfPhotomA, numOfPhotomB, numOfAcq;
    ImageFormat            *formatAcq;
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nAnalysing batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------- \n");
    fprintf (midiReportPtr, "\nAnalysing batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "--------------- \n");

    //    Reset status
    *error = 0;
    
    //    Allocate memory for a temporary file name buffer
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    aprioriCatg = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localFormat = callocImageFormat ();
    formatAcq = callocImageFormat ();
    
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No analysis has been carried out for this batch");
        free (fileTemp);
        free (cleanString);
        freeImageFormat (localFormat);
        freeImageFormat (formatAcq);
        free (classification);
        *error = 1;
        return;
    }

    //    Reset image format
    numOfInterf = 0;
    midiInitFormat (formatInterf);
    numOfPhotomA = 0;
    midiInitFormat (formatPhotomA);
    numOfPhotomB = 0;
    midiInitFormat (formatPhotomB);
    numOfAcq = 0;
    midiInitFormat (formatAcq);

    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\nAnalysing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\nAnalysing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, localFormat, error);
            if (*error) break;
        }
        else localFormat->hasData = 0;

        //    Check if the file has data
        if (localFormat->hasData)
        {
            //    If it is an aquisition save parameters and exit
            if (((strcmp (localFormat->obsCatg, "CALIB") == 0) || 
                (strcmp (localFormat->obsCatg, "SCIENCE") == 0)) &&
                ((strcmp (localFormat->obsTech, "IMAGE,WINDOW") == 0) || 
                (strcmp (localFormat->obsType, "COARSE,OBJECT") == 0)))
            {
                //    Save local parameters
                formatAcq->hasData = 1;
                formatAcq->chopped = 0;
                midiCopyFormat (localFormat, formatAcq);
            
                //    Get cummulative results
                formatAcq->numOfFrames += localFormat->numOfFrames;

                //    Increment file counter
                numOfAcq++;
                break;
            }

            //    Save initial Beam Combiner and Category
            if (numOfPhotomA == 0 && numOfPhotomA == 0 && numOfInterf == 0)
            {
                aprioriDataIsDisp = localFormat->isDisp;
                sprintf (aprioriCatg, "%s", localFormat->obsCatg);
            }
            
            //    Check Beam Combiner consistency
            if (localFormat->isDisp != aprioriDataIsDisp)
            {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent Beam Combiner");
                *error = 1;
                break;
            }
            else
                aprioriDataIsDisp = localFormat->isDisp;

            //    Check Category consistency
            if (strcmp (localFormat->obsCatg, aprioriCatg) != 0)
            {
                sprintf (midiMessage, "Inconsistent Observation Category. Expected: %s", aprioriCatg);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                *error = 1;
                break;
            }
            else
                sprintf (aprioriCatg, "%s", localFormat->obsCatg);

            //    Check Categ, Tech and Type. If INTERFEROMETRY then compute the image size
            if (strcmp (localFormat->obsTech, "INTERFEROMETRY") == 0)
            {
                //    Save local parameters
                if (numOfInterf == 0)
                {
                    formatInterf->hasData = 1;
                    formatInterf->chopped = 0;
                    midiCopyFormat (localFormat, formatInterf);
                }

                //    Get cummulative results
                formatInterf->numOfFrames += localFormat->numOfFrames;

                //    Increment file counter
                numOfInterf++;
            }
            //    If it is a Photometry file, it could either be HIGH_SENS or SCI_PHOT_KAPPA
            else if ((strcmp (localFormat->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0) &&
                (strcmp(localFormat->shutterId,"AOPEN") == 0))
            {
                //    Save local parameters
                if (numOfPhotomA == 0)
                {
                    formatPhotomA->hasData = 1;
                    formatPhotomA->chopped = 1;
                    midiCopyFormat (localFormat, formatPhotomA);
                }
                
                //    Get cummulative results
                formatPhotomA->numOfFrames += localFormat->numOfFrames;

                //    Increment file counter
                numOfPhotomA++;
            }
            else if ((strcmp (localFormat->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0) &&
                (strcmp(localFormat->shutterId,"BOPEN") == 0))
            {
                //    Save local parameters
                if (numOfPhotomB == 0)
                {
                    formatPhotomB->hasData = 1;
                    formatPhotomB->chopped = 1;
                    midiCopyFormat (localFormat, formatPhotomB);
                }
                
                //    Get cummulative results
                formatPhotomB->numOfFrames += localFormat->numOfFrames;

                //    Increment file counter
                numOfPhotomB++;
            }
            else
            {
                sprintf (midiMessage, "Unknown Category, Tech or Type in ... %s. Not processed", fileNames->inFitsName);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in ... %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    //    Close the file list
    if (fclose (inFitsBatchPtr))
        midiReportError (midiReportPtr, routine, __FILE__, __LINE__, "Cannot close batch file");
    
    //    Check if the FITS files contained useful data
    if (!(*error) && (numOfPhotomA > 0) && 
        (numOfPhotomB > 0) && (numOfInterf > 0) && 
        (numOfPhotomA == numOfPhotomB))
    {
        //    Compute number of scans
        formatInterf->numOfScans = formatInterf->numOfFrames / formatInterf->framesPerScan;

        //    Compute default FFT size
        formatInterf->fftsize = determineFFTsize (formatInterf->framesPerScan);

        //    Compute length of allSpectrum
        formatInterf->allSpectrumLength = REGIONS_UNDISPERSED * formatInterf->numOfScans * (formatInterf->fftsize/2);

        //    Log results
        midiReportFormat ("INTERFEROMETRY", aprioriDataIsDisp, formatInterf);
        midiReportFormat ("PHOTOMETRY A", aprioriDataIsDisp, formatPhotomA);
        midiReportFormat ("PHOTOMETRY B", aprioriDataIsDisp, formatPhotomB);
        
        sprintf (midiMessage, "%d useful Photometry and %d useful Interferometry files detected", 
            numOfPhotomA+numOfPhotomB, numOfInterf);
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
    else if (!(*error) && (numOfAcq > 0))
    {
        //    Compute number of scans
        formatAcq->numOfScans = formatAcq->numOfFrames / formatAcq->framesPerScan;

        //    Log results
        midiReportAcqFormat (formatAcq);
    }
    else
    {
        sprintf (fileTemp, "%d", batchNumber);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent batch files");
        *error = 1;
    }
 
    //    Check if requested processing is valid. For now just report warning.
    if (!(*error))
    {
        if (!numOfAcq)
        { 
            if (!aprioriDataIsDisp)
            {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                    "Data is UNDISPERSED\n                 Algorithm for processing this type of data is not ready");
                *error = 1;
            }
        }
    }

    //    Release memory
    if (fileTemp) free (fileTemp);
    if (localFormat) freeImageFormat (localFormat);
    if (formatAcq) freeImageFormat (formatAcq);
    if (cleanString) free (cleanString);
    if (classification) free (classification);
    if (aprioriCatg) free (aprioriCatg);
     
    return;
}
/******************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiReportAcqFormat
* Input/Output: See function arguments to avoid duplication
* Description:  Reports image format
*
* History:      
* 10-Jul-05     (csabet) created
******************************************************************************/
void midiReportAcqFormat (
    ImageFormat *formatAcq)    // In: Pointer to the image format structure
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "midiReportAcqFormat";
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"ACQUISITION FORMAT \n");
   cpl_msg_info(cpl_func,"------------------ \n");
   cpl_msg_info(cpl_func,"FITS Template Name         = %s \n", formatAcq->tplName);
   cpl_msg_info(cpl_func,"Batch Template Name        = %s \n", batchTemplate);
   cpl_msg_info(cpl_func,"Observation Category       = %s \n", formatAcq->obsCatg);
   cpl_msg_info(cpl_func,"Observation Technique      = %s \n", formatAcq->obsTech);
   cpl_msg_info(cpl_func,"Observation Type           = %s \n", formatAcq->obsType);
   cpl_msg_info(cpl_func,"Camera ID                  = %s \n", formatAcq->cameraId);
   cpl_msg_info(cpl_func,"Beam Combiner              = %s \n", formatAcq->beamCombiner);
   cpl_msg_info(cpl_func,"Filter ID                  = %s \n", formatAcq->filterName);
   cpl_msg_info(cpl_func,"Shutter ID                 = %s \n", formatAcq->shutterId);
   cpl_msg_info(cpl_func,"Grism ID                   = %s \n", formatAcq->grismId);
   cpl_msg_info(cpl_func,"Slit Name                  = %s \n", formatAcq->slitName);
   cpl_msg_info(cpl_func,"Target Name                = %s \n", formatAcq->targetName);
   cpl_msg_info(cpl_func,"TPL Start                  = %s \n", formatAcq->tplStart);
   cpl_msg_info(cpl_func,"Number of Frames           = %d \n", formatAcq->numOfFrames);
   cpl_msg_info(cpl_func,"Frames per scan            = %d \n", formatAcq->framesPerScan);
   cpl_msg_info(cpl_func,"Number of Scans            = %d \n", formatAcq->numOfScans);
   cpl_msg_info(cpl_func,"X, Column dimension        = %d \n", formatAcq->iXWidth);
   cpl_msg_info(cpl_func,"Y, Column dimension        = %d \n", formatAcq->iYWidth);
   cpl_msg_info(cpl_func,"Sub-window size            = %d \n", formatAcq->subWindowSize);
   cpl_msg_info(cpl_func,"Number of Pinholes         = %d \n", formatAcq->numOfPinholes);
   cpl_msg_info(cpl_func,"Number of Detector Regions = %d \n", formatAcq->numOfDetectorRegions);
   cpl_msg_info(cpl_func,"\n");

    fprintf (midiReportPtr, "ACQUISITION FORMAT \n");
    fprintf (midiReportPtr, "------------------ \n");
    fprintf (midiReportPtr, "FITS Template Name         = %s \n", formatAcq->tplName);
    fprintf (midiReportPtr, "Batch Template Name        = %s \n", batchTemplate);
    fprintf (midiReportPtr, "Observation Category       = %s \n", formatAcq->obsCatg);
    fprintf (midiReportPtr, "Observation Technique      = %s \n", formatAcq->obsTech);
    fprintf (midiReportPtr, "Observation Type           = %s \n", formatAcq->obsType);
    fprintf (midiReportPtr, "Camera ID                  = %s \n", formatAcq->cameraId);
    fprintf (midiReportPtr, "Beam Combiner              = %s \n", formatAcq->beamCombiner);
    fprintf (midiReportPtr, "Filter ID                  = %s \n", formatAcq->filterName);
    fprintf (midiReportPtr, "Shutter ID                 = %s \n", formatAcq->shutterId);
    fprintf (midiReportPtr, "Grism ID                   = %s \n", formatAcq->grismId);
    fprintf (midiReportPtr, "Slit Name                  = %s \n", formatAcq->slitName);
    fprintf (midiReportPtr, "Target Name                = %s \n", formatAcq->targetName);
    fprintf (midiReportPtr, "TPL Start                  = %s \n", formatAcq->tplStart);
    fprintf (midiReportPtr, "Number of Frames           = %d \n", formatAcq->numOfFrames);
    fprintf (midiReportPtr, "Frames per scan            = %d \n", formatAcq->framesPerScan);
    fprintf (midiReportPtr, "Number of Scans            = %d \n", formatAcq->numOfScans);
    fprintf (midiReportPtr, "X, Column dimension        = %d \n", formatAcq->iXWidth);
    fprintf (midiReportPtr, "Y, Column dimension        = %d \n", formatAcq->iYWidth);
    fprintf (midiReportPtr, "Sub-window size            = %d \n", formatAcq->subWindowSize);
    fprintf (midiReportPtr, "Number of Pinholes         = %d \n", formatAcq->numOfPinholes);
    fprintf (midiReportPtr, "Number of Detector Regions = %d \n", formatAcq->numOfDetectorRegions);
    fprintf (midiReportPtr, "\n");
     
    return;
}
/******************************************************************************/


