
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  fitsAnalysisTec.c
* Description:  Contains all routines for FITS format analysis
*
* History:      
* 14-Jun-02     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "fft.h"
#include "errorHandling.h"
#include "memoryHandling.h"
#include "midiFitsUtility.h"
#include "fitsAnalysisTec.h"

/**********************************************************
*   Global Variables 
**********************************************************/



/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  analyseFitsDetLin
* Input/Output: See function arguments to avoid duplication
* Description:  Analyses all the FITS files in the given list. This routine
*                determines if the FITS list contains useful data files. If
*                not it will discard the list.
*
* History:      
* 14-Jun-04     (csabet) created
******************************************************************************/
void analyseFitsDetLin (
    MidiFiles   *fileNames,        /*  IO: Pointer to midi files structure */
    ImageFormat *format,        /*  Ou: Pointer to the image format structure */
    int            *numOfFiles,    /*    Ou: Number of files with data */
    int            *error)            /*    Ou:    Error status */
{

    //    Local Declarations
    //    ------------------
    const char              routine[] = "analyseFitsDetLin";
    char                        *fileTemp=NULL, *classification=NULL;
    FILE                    *inFitsBatchPtr=NULL;
    ImageFormat             *localFormat=NULL;
    int                     extNumOfImagingDataFile;
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nAnalysing batch  %d  for Detector Linearity \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------- \n");
    fprintf (midiReportPtr, "\nAnalysing batch  %d  for Detector Linearity \n", batchNumber);
    fprintf (midiReportPtr, "--------------- \n");

    //    Reset status
    *error = 0;
    *numOfFiles = 0;
    
    //    Allocate memory for a temporary file name buffer
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localFormat = callocImageFormat ();
    
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No analysis has been carried out for this batch");
        free (fileTemp);
        freeImageFormat (localFormat);
        free (classification);
        *error = 1;
        return;
    }

    //    Reset image format
    format->numOfScans = 0;
    format->numOfFrames = 0;
    format->subWindowSize = 0;
    format->framesPerScan = 0;
    format->iXWidth = 0;
    format->iYWidth = 0;
    
    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Analysing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Analysing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, localFormat, error);
            if (*error) break;
        }
        else localFormat->hasData = 0;

        //    Check if the file has data
        if (localFormat->hasData)
        {
            if ((strcmp (localFormat->obsCatg, "CALIB") == 0) &&
                ((strcmp (localFormat->obsTech, "IMAGE") == 0) || 
                (strcmp (localFormat->obsTech, "SPECTRUM") == 0)) &&
                (strcmp (localFormat->obsType, "FLAT") == 0))
            {
                //    Increment file counter
                (*numOfFiles)++;

                //    Log results
               cpl_msg_info(cpl_func,"File number           = %d \n", *numOfFiles);
               cpl_msg_info(cpl_func,"Observation Category  = %s \n", localFormat->obsCatg);
               cpl_msg_info(cpl_func,"Observation Technique = %s \n", localFormat->obsTech);
               cpl_msg_info(cpl_func,"Observation Type      = %s \n", localFormat->obsType);
               cpl_msg_info(cpl_func,"Beam Combiner         = %s \n", localFormat->beamCombiner);
               cpl_msg_info(cpl_func,"Shutter ID            = %s \n", localFormat->shutterId);
               cpl_msg_info(cpl_func,"Filter Name           = %s \n", localFormat->filterName);
               cpl_msg_info(cpl_func,"Grism ID              = %s \n", localFormat->grismId);
               cpl_msg_info(cpl_func,"Target Name           = %s \n", localFormat->targetName);
               cpl_msg_info(cpl_func,"Number of Frames      = %d \n", localFormat->numOfFrames);
               cpl_msg_info(cpl_func,"X, Column dimension   = %d \n", localFormat->iXWidth);
               cpl_msg_info(cpl_func,"Y, Column dimension   = %d \n", localFormat->iYWidth);
               cpl_msg_info(cpl_func,"Sub-window size       = %d \n", localFormat->subWindowSize);
               cpl_msg_info(cpl_func,"\n");

                fprintf (midiReportPtr, "File number           = %d \n", *numOfFiles);
                fprintf (midiReportPtr, "Observation Category  = %s \n", localFormat->obsCatg);
                fprintf (midiReportPtr, "Observation Technique = %s \n", localFormat->obsTech);
                fprintf (midiReportPtr, "Observation Type      = %s \n", localFormat->obsType);
                fprintf (midiReportPtr, "Beam Combiner         = %s \n", localFormat->beamCombiner);
                fprintf (midiReportPtr, "Shutter ID            = %s \n", localFormat->shutterId);
                fprintf (midiReportPtr, "Filter Name           = %s \n", localFormat->filterName);
                fprintf (midiReportPtr, "Grism ID              = %s \n", localFormat->grismId);
                fprintf (midiReportPtr, "Target Name           = %s \n", localFormat->targetName);
                fprintf (midiReportPtr, "Number of Frames      = %d \n", localFormat->numOfFrames);
                fprintf (midiReportPtr, "X, Column dimension   = %d \n", localFormat->iXWidth);
                fprintf (midiReportPtr, "Y, Column dimension   = %d \n", localFormat->iYWidth);
                fprintf (midiReportPtr, "Sub-window size       = %d \n", localFormat->subWindowSize);
                fprintf (midiReportPtr, "\n");
                 
                //    Compute maximum image format.  Save local parameters
                format->hasData = 1;
                if (*numOfFiles == 1)
                {
                    sprintf (format->obsCatg, "%s", localFormat->obsCatg);
                    sprintf (format->obsType, "%s", localFormat->obsType);
                    sprintf (format->obsTech, "%s", localFormat->obsTech);
                    sprintf (format->beamCombiner, "%s", localFormat->beamCombiner);
                    sprintf (format->shutterId, "%s", localFormat->shutterId);
                    sprintf (format->grismId, "%s", localFormat->grismId);
                    sprintf (format->targetName, "%s", localFormat->targetName);
                    sprintf (format->tplStart, "%s", localFormat->tplStart);
                    sprintf (format->cameraId, "%s", localFormat->cameraId);
                    sprintf (format->filterName, "%s", localFormat->filterName);
                }

                //    Save and cummulate size
                format->numOfFrames = localFormat->numOfFrames;
                format->subWindowSize = localFormat->subWindowSize;
                format->framesPerScan = localFormat->framesPerScan;
                format->iXWidth = localFormat->iXWidth;
                format->iYWidth = localFormat->iYWidth;
            }
            else
            {
                sprintf (midiMessage, "Unknown Category, Tech or Type %s Not processed", fileNames->inFitsName);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }


    //    Close the file list
    fclose (inFitsBatchPtr);
    
    //    Check if the FITS files contained useful data
    if ((*numOfFiles > 0) && !(*error))
    {
       cpl_msg_info(cpl_func,"   Maximum image size \n");
       cpl_msg_info(cpl_func,"   ------------------ \n");
       cpl_msg_info(cpl_func,"   Number of Frames      = %d \n", format->numOfFrames);
       cpl_msg_info(cpl_func,"   X, column dimension   = %d \n", format->iXWidth);
       cpl_msg_info(cpl_func,"   Y, column dimension   = %d \n", format->iYWidth);
       cpl_msg_info(cpl_func,"   Sub-window size       = %d \n", format->subWindowSize);
       cpl_msg_info(cpl_func,"   Number of Data Files  = %d \n", *numOfFiles);
       cpl_msg_info(cpl_func,"\n");
                
        fprintf (midiReportPtr, "   Maximum image size \n");
        fprintf (midiReportPtr, "   ------------------ \n");
        fprintf (midiReportPtr, "   Number of Frames      = %d \n", format->numOfFrames);
        fprintf (midiReportPtr, "   X, column dimension   = %d \n", format->iXWidth);
        fprintf (midiReportPtr, "   Y, column dimension   = %d \n", format->iYWidth);
        fprintf (midiReportPtr, "   Sub-window size       = %d \n", format->subWindowSize);
        fprintf (midiReportPtr, "   Number of Data Files  = %d \n", *numOfFiles);
        fprintf (midiReportPtr, "\n");
    }
    else
    {
        *error = 1;
        sprintf (fileTemp, "%d", batchNumber);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent batch files");
    }

    //    Release memory
    free (fileTemp);
    freeImageFormat (localFormat);
    free (classification);
     
    return;
}
/******************************************************************************/


 

/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  analyseFitDetRon
* Input/Output: See function arguments to avoid duplication
* Description:  Analyses all the FITS files in the given list. This routine
*                determines if the FITS list contains useful data files. If
*                not it will discard the list.
*
* History:      
* 14-Jun-04     (csabet) created
******************************************************************************/
void analyseFitsDetRon (
    MidiFiles   *fileNames,        /*  IO: Pointer to midi files structure */
    ImageFormat *format,        /*  Ou: Pointer to the image format structure */
    int            *numOfFiles,    /*    Ou: Number of files with data */
    int            *error)            /*    Ou:    Error status */
{

    //    Local Declarations
    //    ------------------
    const char              routine[] = "analyseFitDetRon";
    char                        *fileTemp=NULL, *classification=NULL;
    FILE                    *inFitsBatchPtr=NULL;
    ImageFormat             *localFormat=NULL;
    int                     extNumOfImagingDataFile;
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nAnalysing batch  %d  for Detector Readout Noise \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------- \n");
    fprintf (midiReportPtr, "\nAnalysing batch  %d  for Detector Readout Noise \n", batchNumber);
    fprintf (midiReportPtr, "--------------- \n");

    //    Reset status
    *error = 0;
    *numOfFiles = 0;
    
    //    Allocate memory for a temporary file name buffer
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localFormat = callocImageFormat ();
    
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No analysis has been carried out for this batch");
        free (fileTemp);
        freeImageFormat (localFormat);
        free (classification);
        *error = 1;
        return;
    }

    //    Reset image format
    format->numOfScans = 0;
    format->numOfFrames = 0;
    format->subWindowSize = 0;
    format->framesPerScan = 0;
    format->iXWidth = 0;
    format->iYWidth = 0;
    
    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Analysing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Analysing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, localFormat, error);
            if (*error) break;
        }
        else localFormat->hasData = 0;

        //    Check if the file has data
        if (localFormat->hasData)
        {
            if ((strcmp (localFormat->obsCatg, "CALIB") == 0) &&
                (strcmp (localFormat->obsTech, "IMAGE") == 0) &&
                (strcmp (localFormat->obsType, "BIAS") == 0))
            {
                //    Increment file counter
                (*numOfFiles)++;

                //    Log results
               cpl_msg_info(cpl_func,"File number           = %d \n", *numOfFiles);
               cpl_msg_info(cpl_func,"Observation Category  = %s \n", localFormat->obsCatg);
               cpl_msg_info(cpl_func,"Observation Technique = %s \n", localFormat->obsTech);
               cpl_msg_info(cpl_func,"Observation Type      = %s \n", localFormat->obsType);
               cpl_msg_info(cpl_func,"Beam Combiner         = %s \n", localFormat->beamCombiner);
               cpl_msg_info(cpl_func,"Shutter ID            = %s \n", localFormat->shutterId);
               cpl_msg_info(cpl_func,"Grism ID              = %s \n", localFormat->grismId);
               cpl_msg_info(cpl_func,"Filter Name           = %s \n", localFormat->filterName);
               cpl_msg_info(cpl_func,"Target Name           = %s \n", localFormat->targetName);
               cpl_msg_info(cpl_func,"Number of Frames      = %d \n", localFormat->numOfFrames);
               cpl_msg_info(cpl_func,"Frames per scan       = %d \n", localFormat->framesPerScan);
               cpl_msg_info(cpl_func,"Number of Scans       = %d \n", localFormat->numOfScans);
               cpl_msg_info(cpl_func,"X, Column dimension   = %d \n", localFormat->iXWidth);
               cpl_msg_info(cpl_func,"Y, Column dimension   = %d \n", localFormat->iYWidth);
               cpl_msg_info(cpl_func,"Sub-window size       = %d \n", localFormat->subWindowSize);
               cpl_msg_info(cpl_func,"\n");

                fprintf (midiReportPtr, "File number           = %d \n", *numOfFiles);
                fprintf (midiReportPtr, "Observation Category  = %s \n", localFormat->obsCatg);
                fprintf (midiReportPtr, "Observation Technique = %s \n", localFormat->obsTech);
                fprintf (midiReportPtr, "Observation Type      = %s \n", localFormat->obsType);
                fprintf (midiReportPtr, "Beam Combiner         = %s \n", localFormat->beamCombiner);
                fprintf (midiReportPtr, "Shutter ID            = %s \n", localFormat->shutterId);
                fprintf (midiReportPtr, "Grism ID              = %s \n", localFormat->grismId);
                fprintf (midiReportPtr, "Filter Name           = %s \n", localFormat->filterName);
                fprintf (midiReportPtr, "Target Name           = %s \n", localFormat->targetName);
                fprintf (midiReportPtr, "Number of Frames      = %d \n", localFormat->numOfFrames);
                fprintf (midiReportPtr, "Frames per scan       = %d \n", localFormat->framesPerScan);
                fprintf (midiReportPtr, "Number of Scans       = %d \n", localFormat->numOfScans);
                fprintf (midiReportPtr, "X, Column dimension   = %d \n", localFormat->iXWidth);
                fprintf (midiReportPtr, "Y, Column dimension   = %d \n", localFormat->iYWidth);
                fprintf (midiReportPtr, "Sub-window size       = %d \n", localFormat->subWindowSize);
                fprintf (midiReportPtr, "\n");
                 
                //    Compute maximum image format. Save local parameters
                format->hasData = 1;
                if (*numOfFiles == 1)
                {
                    sprintf (format->obsCatg, "%s", localFormat->obsCatg);
                    sprintf (format->obsType, "%s", localFormat->obsType);
                    sprintf (format->obsTech, "%s", localFormat->obsTech);
                    sprintf (format->beamCombiner, "%s", localFormat->beamCombiner);
                    sprintf (format->shutterId, "%s", localFormat->shutterId);
                    sprintf (format->grismId, "%s", localFormat->grismId);
                    sprintf (format->targetName, "%s", localFormat->targetName);
                    sprintf (format->tplStart, "%s", localFormat->tplStart);
                    sprintf (format->cameraId, "%s", localFormat->cameraId);
                    sprintf (format->filterName, "%s", localFormat->filterName);
                }

                //    Save and cummulate size
                format->numOfFrames += localFormat->numOfFrames;
                format->subWindowSize = localFormat->subWindowSize;
                format->iXWidth = localFormat->iXWidth;
                format->iYWidth = localFormat->iYWidth;
            }
            else
            {
                sprintf (midiMessage, "Unknown Category, Tech or Type %s Not processed", fileNames->inFitsName);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }
    
    //    Check if the FITS files contained useful data
    if ((*numOfFiles > 0) && !(*error))
    {
       cpl_msg_info(cpl_func,"   Maximum image size \n");
       cpl_msg_info(cpl_func,"   ------------------ \n");
       cpl_msg_info(cpl_func,"   Number of Frames      = %d \n", format->numOfFrames);
       cpl_msg_info(cpl_func,"   X, column dimension   = %d \n", format->iXWidth);
       cpl_msg_info(cpl_func,"   Y, column dimension   = %d \n", format->iYWidth);
       cpl_msg_info(cpl_func,"   Sub-window size       = %d \n", format->subWindowSize);
       cpl_msg_info(cpl_func,"   Number of Data Files  = %d \n", *numOfFiles);
       cpl_msg_info(cpl_func,"\n");
                
        fprintf (midiReportPtr, "   Maximum image size \n");
        fprintf (midiReportPtr, "   ------------------ \n");
        fprintf (midiReportPtr, "   Number of Frames      = %d \n", format->numOfFrames);
        fprintf (midiReportPtr, "   X, column dimension   = %d \n", format->iXWidth);
        fprintf (midiReportPtr, "   Y, column dimension   = %d \n", format->iYWidth);
        fprintf (midiReportPtr, "   Sub-window size       = %d \n", format->subWindowSize);
        fprintf (midiReportPtr, "   Number of Data Files  = %d \n", *numOfFiles);
        fprintf (midiReportPtr, "\n");
    }
    else
    {
        *error = 1;
        sprintf (fileTemp, "%d", batchNumber);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent batch files");
    }

    //    Release memory and close files
    fclose (inFitsBatchPtr);
    free (fileTemp);
    freeImageFormat (localFormat);
    free (classification);
     
    return;
}
/******************************************************************************/




/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  analyseFitsDspTrn
* Input/Output: See function arguments to avoid duplication
* Description:  Analyses all the FITS files in the given list. This routine
*                determines if the FITS list contains useful data files. If
*                not it will discard the list.
*
* History:      
* 14-Jun-04     (csabet) created
******************************************************************************/
void analyseFitsDspTrn (
    MidiFiles   *fileNames,        /*  IO: Pointer to midi files structure */
    ImageFormat *format,        /*  Ou: Pointer to the image format structure */
    int            *numOfFiles,    /*    Ou: Number of files with data */
    int            *error)            /*    Ou:    Error status */
{

    //    Local Declarations
    //    ------------------
    const char              routine[] = "analyseFitsDspTrn";
    char                        *fileTemp=NULL, *classification=NULL;
    FILE                    *inFitsBatchPtr=NULL;
    ImageFormat             *localFormat=NULL;
    int                     extNumOfImagingDataFile;
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nAnalysing batch  %d  for Dispersive Transmission Assessment \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------- \n");
    fprintf (midiReportPtr, "\nAnalysing batch  %d  for Dispersive Transmission Assessment \n", batchNumber);
    fprintf (midiReportPtr, "--------------- \n");

    //    Reset status
    *error = 0;
    *numOfFiles = 0;
    
    //    Allocate memory for a temporary file name buffer
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localFormat = callocImageFormat ();
    
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No analysis has been carried out for this batch");
        free (fileTemp);
        freeImageFormat (localFormat);
        free (classification);
        *error = 1;
        return;
    }

    //    Reset image format
    format->numOfScans = 0;
    format->numOfFrames = 0;
    format->subWindowSize = 0;
    format->framesPerScan = 0;
    format->iXWidth = 0;
    format->iYWidth = 0;
    
    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Analysing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Analysing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, localFormat, error);
            if (*error) break;
        }
        else localFormat->hasData = 0;

        //    Check if the file has data
        if (localFormat->hasData)
        {
            if ((strcmp (localFormat->obsCatg, "CALIB") == 0) &&
                (strcmp (localFormat->obsTech, "SPECTRUM") == 0) &&
                (strcmp (localFormat->obsType, "WAVE") == 0))
            {
                //    Increment file counter
                (*numOfFiles)++;

                //    Log results
               cpl_msg_info(cpl_func,"File number           = %d \n", *numOfFiles);
               cpl_msg_info(cpl_func,"Observation Category  = %s \n", localFormat->obsCatg);
               cpl_msg_info(cpl_func,"Observation Technique = %s \n", localFormat->obsTech);
               cpl_msg_info(cpl_func,"Observation Type      = %s \n", localFormat->obsType);
               cpl_msg_info(cpl_func,"Beam Combiner         = %s \n", localFormat->beamCombiner);
               cpl_msg_info(cpl_func,"Shutter ID            = %s \n", localFormat->shutterId);
               cpl_msg_info(cpl_func,"Grism ID              = %s \n", localFormat->grismId);
               cpl_msg_info(cpl_func,"Filter Name           = %s \n", localFormat->filterName);
               cpl_msg_info(cpl_func,"Target Name           = %s \n", localFormat->targetName);
               cpl_msg_info(cpl_func,"Number of Frames      = %d \n", localFormat->numOfFrames);
               cpl_msg_info(cpl_func,"Frames per scan       = %d \n", localFormat->framesPerScan);
               cpl_msg_info(cpl_func,"Number of Scans       = %d \n", localFormat->numOfScans);
               cpl_msg_info(cpl_func,"X, Column dimension   = %d \n", localFormat->iXWidth);
               cpl_msg_info(cpl_func,"Y, Column dimension   = %d \n", localFormat->iYWidth);
               cpl_msg_info(cpl_func,"Sub-window size       = %d \n", localFormat->subWindowSize);
               cpl_msg_info(cpl_func,"\n");

                fprintf (midiReportPtr, "File number           = %d \n", *numOfFiles);
                fprintf (midiReportPtr, "Observation Category  = %s \n", localFormat->obsCatg);
                fprintf (midiReportPtr, "Observation Technique = %s \n", localFormat->obsTech);
                fprintf (midiReportPtr, "Observation Type      = %s \n", localFormat->obsType);
                fprintf (midiReportPtr, "Beam Combiner         = %s \n", localFormat->beamCombiner);
                fprintf (midiReportPtr, "Shutter ID            = %s \n", localFormat->shutterId);
                fprintf (midiReportPtr, "Grism ID              = %s \n", localFormat->grismId);
                fprintf (midiReportPtr, "Filter Name           = %s \n", localFormat->filterName);
                fprintf (midiReportPtr, "Target Name           = %s \n", localFormat->targetName);
                fprintf (midiReportPtr, "Number of Frames      = %d \n", localFormat->numOfFrames);
                fprintf (midiReportPtr, "Frames per scan       = %d \n", localFormat->framesPerScan);
                fprintf (midiReportPtr, "Number of Scans       = %d \n", localFormat->numOfScans);
                fprintf (midiReportPtr, "X, Column dimension   = %d \n", localFormat->iXWidth);
                fprintf (midiReportPtr, "Y, Column dimension   = %d \n", localFormat->iYWidth);
                fprintf (midiReportPtr, "Sub-window size       = %d \n", localFormat->subWindowSize);
                fprintf (midiReportPtr, "\n");
                 
                //    Compute maximum image format. Save local parameters
                format->hasData = 1;
                if (*numOfFiles == 1)
                {
                    sprintf (format->obsCatg, "%s", localFormat->obsCatg);
                    sprintf (format->obsType, "%s", localFormat->obsType);
                    sprintf (format->obsTech, "%s", localFormat->obsTech);
                    sprintf (format->beamCombiner, "%s", localFormat->beamCombiner);
                    sprintf (format->shutterId, "%s", localFormat->shutterId);
                    sprintf (format->grismId, "%s", localFormat->grismId);
                    sprintf (format->targetName, "%s", localFormat->targetName);
                    sprintf (format->tplStart, "%s", localFormat->tplStart);
                    sprintf (format->cameraId, "%s", localFormat->cameraId);
                    sprintf (format->filterName, "%s", localFormat->filterName);
                }

                if (localFormat->numOfScans > format->numOfScans) 
                    format->numOfScans = localFormat->numOfScans;
                    
                if (localFormat->numOfFrames > format->numOfFrames) 
                    format->numOfFrames = localFormat->numOfFrames;
                    
                if (localFormat->subWindowSize > format->subWindowSize) 
                    format->subWindowSize = localFormat->subWindowSize;
                    
                if (localFormat->framesPerScan > format->framesPerScan) 
                    format->framesPerScan = localFormat->framesPerScan;
                    
                if (localFormat->iXWidth > format->iXWidth) 
                    format->iXWidth = localFormat->iXWidth;
                    
                if (localFormat->iYWidth > format->iYWidth) 
                    format->iYWidth = localFormat->iYWidth;
            }
            else
            {
                sprintf (midiMessage, "Unknown Category, Tech or Type %s Not processed", fileNames->inFitsName);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }


    //    Close the file list
    fclose (inFitsBatchPtr);
    
    //    Check if the FITS files contained useful data
    if ((*numOfFiles > 0) && !(*error))
    {
       cpl_msg_info(cpl_func,"   Maximum image size \n");
       cpl_msg_info(cpl_func,"   ------------------ \n");
       cpl_msg_info(cpl_func,"   Number of Scans       = %d \n", format->numOfScans);
       cpl_msg_info(cpl_func,"   Number of Frames      = %d \n", format->numOfFrames);
       cpl_msg_info(cpl_func,"   Frames per scan       = %d \n", format->framesPerScan);
       cpl_msg_info(cpl_func,"   X, column dimension   = %d \n", format->iXWidth);
       cpl_msg_info(cpl_func,"   Y, column dimension   = %d \n", format->iYWidth);
       cpl_msg_info(cpl_func,"   Sub-window size       = %d \n", format->subWindowSize);
       cpl_msg_info(cpl_func,"   Number of Data Files  = %d \n", *numOfFiles);
       cpl_msg_info(cpl_func,"\n");
                
        fprintf (midiReportPtr, "   Maximum image size \n");
        fprintf (midiReportPtr, "   ------------------ \n");
        fprintf (midiReportPtr, "   Number of Scans       = %d \n", format->numOfScans);
        fprintf (midiReportPtr, "   Number of Frames      = %d \n", format->numOfFrames);
        fprintf (midiReportPtr, "   Frames per scan       = %d \n", format->framesPerScan);
        fprintf (midiReportPtr, "   X, column dimension   = %d \n", format->iXWidth);
        fprintf (midiReportPtr, "   Y, column dimension   = %d \n", format->iYWidth);
        fprintf (midiReportPtr, "   Sub-window size       = %d \n", format->subWindowSize);
        fprintf (midiReportPtr, "   Number of Data Files  = %d \n", *numOfFiles);
        fprintf (midiReportPtr, "\n");
    }
    else
    {
        *error = 1;
        sprintf (fileTemp, "%d", batchNumber);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent batch files");
    }

    //    Release memory
    free (fileTemp);
    freeImageFormat (localFormat);
    free (classification);
     
    return;
}
/******************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  analyseFitsRefPix
* Input/Output: See function arguments to avoid duplication
* Description:  Analyses all the FITS files in the given list. This routine
*                determines if the FITS list contains useful data files. If
*                not it will discard the list.
*
* History:      
* 14-Jun-04     (csabet) created
******************************************************************************/
void analyseFitsRefPix (
    MidiFiles   *fileNames,        //  IO: Pointer to midi files structure */
    ImageFormat *format,        //  Ou: Pointer to the image format structure */
    int            *numOfFiles,    //    Ou: Number of files with data */
    int            *numOfBeams,    //    Ou:    Maximum number of beams
    int            *error)            //    Ou:    Error status */
{

    //    Local Declarations
    //    ------------------
    const char              routine[] = "analyseFitsRefPix";
    char                        *fileTemp=NULL, *classification=NULL;
    FILE                    *inFitsBatchPtr=NULL;
    ImageFormat             *localFormat=NULL;
    int                     extNumOfImagingDataFile;
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nAnalysing batch  %d  for Reference Pixel Alignment \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------- \n");
    fprintf (midiReportPtr, "\nAnalysing batch  %d  for Reference Pixel Alignment \n", batchNumber);
    fprintf (midiReportPtr, "--------------- \n");

    //    Reset status
    *error = 0;
    *numOfFiles = 0;
    *numOfBeams = 2;
    
    //    Allocate memory for a temporary file name buffer
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localFormat = callocImageFormat ();
    
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No analysis has been carried out for this batch");
        free (fileTemp);
        freeImageFormat (localFormat);
        free (classification);
        *error = 1;
        return;
    }

    //    Reset image format
    format->numOfScans = 0;
    format->numOfFrames = 0;
    format->subWindowSize = 0;
    format->framesPerScan = 0;
    format->iXWidth = 0;
    format->iYWidth = 0;
    
    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Analysing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Analysing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, localFormat, error);
            if (*error) break;
        }
        else localFormat->hasData = 0;

        //    Check if the file has data
        if (localFormat->hasData)
        {
            if ((strcmp (localFormat->obsCatg, "CALIB") == 0) &&
            (strcmp (localFormat->obsTech, "IMAGE") == 0) &&
            (strcmp (localFormat->obsType, "FMTCHCK") == 0))
            {
                //    Increment file counter
                (*numOfFiles)++;

                //    Compute maximum number of beams per exposure
                if (strcmp (localFormat->beamCombiner, "SCI_PHOT") == 0) *numOfBeams = 3;

                //    Log results
               cpl_msg_info(cpl_func,"File number           = %d \n", *numOfFiles);
               cpl_msg_info(cpl_func,"Observation Category  = %s \n", localFormat->obsCatg);
               cpl_msg_info(cpl_func,"Observation Technique = %s \n", localFormat->obsTech);
               cpl_msg_info(cpl_func,"Observation Type      = %s \n", localFormat->obsType);
               cpl_msg_info(cpl_func,"Beam Combiner         = %s \n", localFormat->beamCombiner);
               cpl_msg_info(cpl_func,"Shutter ID            = %s \n", localFormat->shutterId);
               cpl_msg_info(cpl_func,"Grism ID              = %s \n", localFormat->grismId);
               cpl_msg_info(cpl_func,"Filter Name           = %s \n", localFormat->filterName);
               cpl_msg_info(cpl_func,"Target Name           = %s \n", localFormat->targetName);
               cpl_msg_info(cpl_func,"Number of Frames      = %d \n", localFormat->numOfFrames);
               cpl_msg_info(cpl_func,"Frames per scan       = %d \n", localFormat->framesPerScan);
               cpl_msg_info(cpl_func,"Number of Scans       = %d \n", localFormat->numOfScans);
               cpl_msg_info(cpl_func,"X, Column dimension   = %d \n", localFormat->iXWidth);
               cpl_msg_info(cpl_func,"Y, Column dimension   = %d \n", localFormat->iYWidth);
               cpl_msg_info(cpl_func,"Sub-window size       = %d \n", localFormat->subWindowSize);
               cpl_msg_info(cpl_func,"\n");

                fprintf (midiReportPtr, "File number           = %d \n", *numOfFiles);
                fprintf (midiReportPtr, "Observation Category  = %s \n", localFormat->obsCatg);
                fprintf (midiReportPtr, "Observation Technique = %s \n", localFormat->obsTech);
                fprintf (midiReportPtr, "Observation Type      = %s \n", localFormat->obsType);
                fprintf (midiReportPtr, "Beam Combiner         = %s \n", localFormat->beamCombiner);
                fprintf (midiReportPtr, "Shutter ID            = %s \n", localFormat->shutterId);
                fprintf (midiReportPtr, "Grism ID              = %s \n", localFormat->grismId);
                fprintf (midiReportPtr, "Filter Name           = %s \n", localFormat->filterName);
                fprintf (midiReportPtr, "Target Name           = %s \n", localFormat->targetName);
                fprintf (midiReportPtr, "Number of Frames      = %d \n", localFormat->numOfFrames);
                fprintf (midiReportPtr, "Frames per scan       = %d \n", localFormat->framesPerScan);
                fprintf (midiReportPtr, "Number of Scans       = %d \n", localFormat->numOfScans);
                fprintf (midiReportPtr, "X, Column dimension   = %d \n", localFormat->iXWidth);
                fprintf (midiReportPtr, "Y, Column dimension   = %d \n", localFormat->iYWidth);
                fprintf (midiReportPtr, "Sub-window size       = %d \n", localFormat->subWindowSize);
                fprintf (midiReportPtr, "\n");
                 
                //    Compute maximum image format. Save local parameters
                format->hasData = 1;
                if (*numOfFiles == 1)
                {
                    sprintf (format->obsCatg, "%s", localFormat->obsCatg);
                    sprintf (format->obsType, "%s", localFormat->obsType);
                    sprintf (format->obsTech, "%s", localFormat->obsTech);
                    sprintf (format->beamCombiner, "%s", localFormat->beamCombiner);
                    sprintf (format->shutterId, "%s", localFormat->shutterId);
                    sprintf (format->grismId, "%s", localFormat->grismId);
                    sprintf (format->targetName, "%s", localFormat->targetName);
                    sprintf (format->tplStart, "%s", localFormat->tplStart);
                    sprintf (format->cameraId, "%s", localFormat->cameraId);
                    sprintf (format->filterName, "%s", localFormat->filterName);
                }

                if (localFormat->numOfScans > format->numOfScans) 
                    format->numOfScans = localFormat->numOfScans;
                    
                if (localFormat->numOfFrames > format->numOfFrames) 
                    format->numOfFrames = localFormat->numOfFrames;
                    
                if (localFormat->subWindowSize > format->subWindowSize) 
                    format->subWindowSize = localFormat->subWindowSize;
                    
                if (localFormat->framesPerScan > format->framesPerScan) 
                    format->framesPerScan = localFormat->framesPerScan;
                    
                if (localFormat->iXWidth > format->iXWidth) 
                    format->iXWidth = localFormat->iXWidth;
                    
                if (localFormat->iYWidth > format->iYWidth) 
                    format->iYWidth = localFormat->iYWidth;
            }
            else
            {
                sprintf (midiMessage, "Unknown Category, Tech or Type %s Not processed", fileNames->inFitsName);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }


    //    Close the file list
    fclose (inFitsBatchPtr);
    
    //    Check if the FITS files contained useful data
    if ((*numOfFiles > 0) && !(*error))
    {
       cpl_msg_info(cpl_func,"   Maximum image size \n");
       cpl_msg_info(cpl_func,"   ------------------ \n");
       cpl_msg_info(cpl_func,"   Number of Scans       = %d \n", format->numOfScans);
       cpl_msg_info(cpl_func,"   Number of Frames      = %d \n", format->numOfFrames);
       cpl_msg_info(cpl_func,"   Frames per scan       = %d \n", format->framesPerScan);
       cpl_msg_info(cpl_func,"   Number of Beams       = %d \n", *numOfBeams);
       cpl_msg_info(cpl_func,"   X, column dimension   = %d \n", format->iXWidth);
       cpl_msg_info(cpl_func,"   Y, column dimension   = %d \n", format->iYWidth);
       cpl_msg_info(cpl_func,"   Sub-window size       = %d \n", format->subWindowSize);
       cpl_msg_info(cpl_func,"   Number of Data Files  = %d \n", *numOfFiles);
       cpl_msg_info(cpl_func,"\n");
                
        fprintf (midiReportPtr, "   Maximum image size \n");
        fprintf (midiReportPtr, "   ------------------ \n");
        fprintf (midiReportPtr, "   Number of Scans       = %d \n", format->numOfScans);
        fprintf (midiReportPtr, "   Number of Frames      = %d \n", format->numOfFrames);
        fprintf (midiReportPtr, "   Frames per scan       = %d \n", format->framesPerScan);
        fprintf (midiReportPtr, "   Number of Beams       = %d \n", *numOfBeams);
        fprintf (midiReportPtr, "   X, column dimension   = %d \n", format->iXWidth);
        fprintf (midiReportPtr, "   Y, column dimension   = %d \n", format->iYWidth);
        fprintf (midiReportPtr, "   Sub-window size       = %d \n", format->subWindowSize);
        fprintf (midiReportPtr, "   Number of Data Files  = %d \n", *numOfFiles);
        fprintf (midiReportPtr, "\n");
    }
    else
    {
        *error = 1;
        sprintf (fileTemp, "%d", batchNumber);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent batch files");
    }

    //    Release memory
    free (fileTemp);
    freeImageFormat (localFormat);
    free (classification);
     
    return;
}
/******************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  analyseFitsWaveCal
* Input/Output: See function arguments to avoid duplication
* Description:  Analyses all the FITS files in the given list. This routine
*                determines if the FITS list contains useful data files. If
*                not it will discard the list.
*
* History:      
* 10-Jun-05     (csabet) created
******************************************************************************/
void analyseFitsWaveCal (
    MidiFiles   *fileNames,        // IO: Pointer to midi files structure
    ImageFormat *format,        // Ou: Pointer to the image format structure
    int            *numOfFiles,    // Ou: Number of files with data
    int            *error)            // Ou:    Error status
{

    //    Local Declarations
    //    ------------------
    const char              routine[] = "analyseFitsWaveCal";
    char                        *fileTemp=NULL, *classification=NULL;
    FILE                    *inFitsBatchPtr=NULL;
    ImageFormat             *localFormat=NULL;
    int                     extNumOfImagingDataFile;
        
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nAnalysing batch  %d  for Wavelength Calibration \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------- \n");
    fprintf (midiReportPtr, "\nAnalysing batch  %d  for Wavelength Calibration \n", batchNumber);
    fprintf (midiReportPtr, "--------------- \n");

    //    Reset status
    *error = 0;
    *numOfFiles = 0;
    
    //    Allocate memory for a temporary file name buffer
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localFormat = callocImageFormat ();
    
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No analysis has been carried out for this batch");
        free (fileTemp);
        freeImageFormat (localFormat);
        free (classification);
        *error = 1;
        return;
    }

    //    Reset image format
    format->hasData = 0;
    format->numOfDetectorRegions = 0;
    format->numOfRegionsToProcess = 0;
    format->numOfPinholes = 0;
    format->numOfFrames = 0;
    format->framesPerScan = 0;
    format->numOfScans = 0;
    format->subWindowSize = 0;
    format->iXWidth = 0;
    format->iYWidth = 0;

    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Analysing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Analysing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, localFormat, error);
            if (*error) break;
        }
        else localFormat->hasData = 0;

        //    Check if the file has data
        if (localFormat->hasData)
        {
            if ((strcmp (localFormat->obsCatg, "CALIB") == 0) &&
                (strcmp (localFormat->obsTech, "SPECTRUM") == 0) &&
                (strcmp (localFormat->obsType, "WAVE,SPECTEMPL") == 0))
            {
                //    Increment file counter
                (*numOfFiles)++;

                //    Log results
               cpl_msg_info(cpl_func,"File number           = %d \n", *numOfFiles);
               cpl_msg_info(cpl_func,"Observation Category  = %s \n", localFormat->obsCatg);
               cpl_msg_info(cpl_func,"Observation Technique = %s \n", localFormat->obsTech);
               cpl_msg_info(cpl_func,"Observation Type      = %s \n", localFormat->obsType);
               cpl_msg_info(cpl_func,"Beam Combiner         = %s \n", localFormat->beamCombiner);
               cpl_msg_info(cpl_func,"Shutter ID            = %s \n", localFormat->shutterId);
               cpl_msg_info(cpl_func,"Filter Name           = %s \n", localFormat->filterName);
               cpl_msg_info(cpl_func,"Grism ID              = %s \n", localFormat->grismId);
               cpl_msg_info(cpl_func,"Target Name           = %s \n", localFormat->targetName);
               cpl_msg_info(cpl_func,"Number of Frames      = %d \n", localFormat->numOfFrames);
               cpl_msg_info(cpl_func,"Frames per scan       = %d \n", localFormat->framesPerScan);
               cpl_msg_info(cpl_func,"Number of Scans       = %d \n", localFormat->numOfScans);
               cpl_msg_info(cpl_func,"X, Column dimension   = %d \n", localFormat->iXWidth);
               cpl_msg_info(cpl_func,"Y, Column dimension   = %d \n", localFormat->iYWidth);
               cpl_msg_info(cpl_func,"Sub-window size       = %d \n", localFormat->subWindowSize);
               cpl_msg_info(cpl_func,"\n");

                fprintf (midiReportPtr, "File number           = %d \n", *numOfFiles);
                fprintf (midiReportPtr, "Observation Category  = %s \n", localFormat->obsCatg);
                fprintf (midiReportPtr, "Observation Technique = %s \n", localFormat->obsTech);
                fprintf (midiReportPtr, "Observation Type      = %s \n", localFormat->obsType);
                fprintf (midiReportPtr, "Beam Combiner         = %s \n", localFormat->beamCombiner);
                fprintf (midiReportPtr, "Shutter ID            = %s \n", localFormat->shutterId);
                fprintf (midiReportPtr, "Filter Name           = %s \n", localFormat->filterName);
                fprintf (midiReportPtr, "Grism ID              = %s \n", localFormat->grismId);
                fprintf (midiReportPtr, "Target Name           = %s \n", localFormat->targetName);
                fprintf (midiReportPtr, "Number of Frames      = %d \n", localFormat->numOfFrames);
                fprintf (midiReportPtr, "Frames per scan       = %d \n", localFormat->framesPerScan);
                fprintf (midiReportPtr, "Number of Scans       = %d \n", localFormat->numOfScans);
                fprintf (midiReportPtr, "X, Column dimension   = %d \n", localFormat->iXWidth);
                fprintf (midiReportPtr, "Y, Column dimension   = %d \n", localFormat->iYWidth);
                fprintf (midiReportPtr, "Sub-window size       = %d \n", localFormat->subWindowSize);
                fprintf (midiReportPtr, "\n");
                 
                //    Save local parameters
                if (*numOfFiles == 1)
                {
                    format->hasData = 1;
                    sprintf (format->obsCatg, "%s", localFormat->obsCatg);
                    sprintf (format->obsTech, "%s", localFormat->obsTech);
                    sprintf (format->obsType, "%s", localFormat->obsType);
                    sprintf (format->cameraId, "%s", localFormat->cameraId);
                    sprintf (format->beamCombiner, "%s", localFormat->beamCombiner);
                    sprintf (format->filterName, "%s", localFormat->filterName);
                    sprintf (format->shutterId, "%s", localFormat->shutterId);
                    sprintf (format->grismId, "%s", localFormat->grismId);
                    sprintf (format->targetName, "%s", localFormat->targetName);
                    sprintf (format->slitName, "%s", localFormat->slitName);
                    sprintf (format->tplStart, "%s", localFormat->tplStart);
                    sprintf (format->tplName, "%s", localFormat->tplName);
                    format->numOfDetectorRegions = localFormat->numOfDetectorRegions;
                    format->numOfRegionsToProcess = localFormat->numOfRegionsToProcess;
                    format->numOfPinholes = localFormat->numOfPinholes;
                    format->framesPerScan = localFormat->framesPerScan;
                    format->subWindowSize = localFormat->subWindowSize;
                    format->iXWidth = localFormat->iXWidth;
                    format->iYWidth = localFormat->iYWidth;
                    format->numOfFrames = localFormat->numOfFrames;
                    format->subWindowSize = localFormat->subWindowSize;
                }
                else // Check consistency
                {
                    if (strcmp (format->obsCatg, localFormat->obsCatg) != 0) *error = 1;
                    if (strcmp (format->obsTech, localFormat->obsTech) != 0) *error = 1;
                    if (strcmp (format->obsType, localFormat->obsType) != 0) *error = 1;
                    if (strcmp (format->cameraId, localFormat->cameraId) != 0) *error = 1;
                    if (strcmp (format->beamCombiner, localFormat->beamCombiner) != 0) *error = 1;
                    if (strcmp (format->grismId, localFormat->grismId) != 0) *error = 1;
                    if (strcmp (format->targetName, localFormat->targetName) != 0) *error = 1;
                    if (strcmp (format->slitName, localFormat->slitName) != 0) *error = 1;
                    if (strcmp (format->tplStart, localFormat->tplStart) != 0) *error = 1;
                    if (strcmp (format->tplName, localFormat->tplName) != 0) *error = 1;
                    if (format->numOfDetectorRegions != localFormat->numOfDetectorRegions) *error = 1;
                    if (format->numOfRegionsToProcess != localFormat->numOfRegionsToProcess) *error = 1;
                    if (format->numOfPinholes != localFormat->numOfPinholes) *error = 1;
                    if (format->framesPerScan != localFormat->framesPerScan) *error = 1;
                    if (format->subWindowSize != localFormat->subWindowSize) *error = 1;
                    if (format->iXWidth != localFormat->iXWidth) *error = 1;
                    if (format->iYWidth != localFormat->iYWidth) *error = 1;
                    if (format->numOfFrames != localFormat->numOfFrames) *error = 1;
                    if (format->subWindowSize != localFormat->subWindowSize) *error = 1;
                }
                if (*error)
                {
                    sprintf (midiMessage, "Inconsistent FITS files. %s", fileNames->inFitsName);
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                }
            }
            else
            {
                sprintf (midiMessage, "Unknown Category, Tech or Type %s Not processed", fileNames->inFitsName);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }


    //    Close the file list
    fclose (inFitsBatchPtr);
    
    //    Check if the FITS files contained useful data
    if ((*numOfFiles > 0) && !(*error))
    {
       cpl_msg_info(cpl_func,"   Maximum image size \n");
       cpl_msg_info(cpl_func,"   ------------------ \n");
       cpl_msg_info(cpl_func,"   Number of Scans       = %d \n", format->numOfScans);
       cpl_msg_info(cpl_func,"   Number of Frames      = %d \n", format->numOfFrames);
       cpl_msg_info(cpl_func,"   Frames per scan       = %d \n", format->framesPerScan);
       cpl_msg_info(cpl_func,"   X, column dimension   = %d \n", format->iXWidth);
       cpl_msg_info(cpl_func,"   Y, column dimension   = %d \n", format->iYWidth);
       cpl_msg_info(cpl_func,"   Number of Regions     = %d \n", format->numOfDetectorRegions);
       cpl_msg_info(cpl_func,"   Sub-window size       = %d \n", format->subWindowSize);
       cpl_msg_info(cpl_func,"   Number of Data Files  = %d \n", *numOfFiles);
       cpl_msg_info(cpl_func,"\n");
                
        fprintf (midiReportPtr, "   Maximum image size \n");
        fprintf (midiReportPtr, "   ------------------ \n");
        fprintf (midiReportPtr, "   Number of Scans       = %d \n", format->numOfScans);
        fprintf (midiReportPtr, "   Number of Frames      = %d \n", format->numOfFrames);
        fprintf (midiReportPtr, "   Frames per scan       = %d \n", format->framesPerScan);
        fprintf (midiReportPtr, "   X, column dimension   = %d \n", format->iXWidth);
        fprintf (midiReportPtr, "   Y, column dimension   = %d \n", format->iYWidth);
        fprintf (midiReportPtr, "   Number of Regions     = %d \n", format->numOfDetectorRegions);
        fprintf (midiReportPtr, "   Sub-window size       = %d \n", format->subWindowSize);
        fprintf (midiReportPtr, "   Number of Data Files  = %d \n", *numOfFiles);
        fprintf (midiReportPtr, "\n");
    }
    else
    {
        *error = 1;
        sprintf (fileTemp, "%d", batchNumber);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent batch files");
    }

    //    Release memory
    free (fileTemp);
    freeImageFormat (localFormat);
    free (classification);
     
    return;
}
/******************************************************************************/

