
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  imageProcessing.c
* Description:  Contains routines for image processing
*
* History:      
* 14-Jun-04     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/stat.h>
#include <math.h>
#include <stdio.h>
#include "qfits.h"
#include "midiGlobal.h"
#include "midiLib.h"
#include "imageProcessing.h"
#include "statistics.h"
#include "errorHandling.h"
#include "diagnostics.h"
#include "cpl_image_io.h"
#include "cpl_image_basic.h"
#include "complex_midi.h"
#include "fft.h"
#include <cpl.h>

/**********************************************************
*   Constant definitions
**********************************************************/
#define CIRCLE_TO_SQUARE        (0.350)

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getUndispersedPowerSpectrum
* Input/Output: See function arguments to avoid duplication
* Description:  Creates spectrum for all regions together with the dc levels.
*
* History:
* 17-Sep-03             (JM) Initial code
* 15-Feb-06     (csabet) Completely restructured and also adapted for SCI_PHOT
******************************************************************************/
void getUndispersedPowerSpectrum (
    ImageFormat         *imageFormat,   // In: Pointer to image format
    CompressedData      *compressed)    // In: Pointer to the compressed data
{

        //      Local Declarations
        //      ------------------
        const char              routine[] = "getUndispersedPowerSpectrum";
    double                      *inputPtr;
    struct Complex      *input2ft;
    int                         i, s, R;
    float                       accum, *allSpectrumPtr, *dcLevelsPtr;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

        //      Preset FFT
    SetFFTsize (imageFormat->fftsize);

        //      Allocate the input array for the FFT:
    FFTarray = (struct Complex *) malloc (MAXFFTSIZE * (sizeof(struct Complex)));
    input2ft = (struct Complex *) malloc (imageFormat->fftsize * (sizeof(struct Complex)));
                 
        //      Save pointer etc.
        allSpectrumPtr = compressed->allSpectrum;
        dcLevelsPtr = compressed->dcLevels;
        
    for (R = 0; R < REGIONS_UNDISPERSED; R++)
    {
                if (R == 0)
                        inputPtr = compressed->iFringe;
                else if (R == 1)
                        inputPtr = compressed->iFringe1;
                else
                        inputPtr = compressed->iFringe2;

                //      Process each scan in turn:
                for (s = 0; s < imageFormat->numOfScans; s++)
                {
                        //      Process good scans only
                        if (!(compressed->badScanList[s]))
                        {
                                //      Reset
                                memset (input2ft, 0, (sizeof(struct Complex)) * imageFormat->fftsize);
                                accum = 0.F;
                                for (i = 0; i < imageFormat->framesPerScan; i++)
                                {                               
                                        //      Accumulate to the DC level here. Process good frames and Ignore Sky if any
                                        accum += *inputPtr;

                                        //      Copy to fft input array
                                        input2ft[i].r = *inputPtr;
                                        inputPtr++;
                                }
                        
                                //      DC level of this scan
                                *dcLevelsPtr = accum / ((float) imageFormat->framesPerScan);

                                //      Now subtract the dc level from the values we just inputted to the fft input array:
                                for (i = 0; i < imageFormat->framesPerScan; i++)
                                        input2ft[i].r -= *dcLevelsPtr;
                                
                                //      Increment the pointer
                                dcLevelsPtr++;

                                //  Now do the FFT. The output will be in FFTarray
                                FFT (input2ft, FFTlevel);
                                
                                //      Now copy the (sqrd mag) of the fft result to the power spectrum output
                                for (i = 0; i < imageFormat->fftsize/2; i++)
                                {
                                        *allSpectrumPtr = Cmag2(FFTarray[i]);
                                        allSpectrumPtr++;
                                }
                        }
                        else
                        {
                                //      Increment pointers
                                dcLevelsPtr++;
                                for (i = 0; i < imageFormat->framesPerScan; i++)
                                        inputPtr++;
                                for (i = 0; i < imageFormat->fftsize/2; i++)
                                        allSpectrumPtr++;
                        }
                }
        }

    //  Create plot data file. For SCI_PHOT it will not be used yet. Not necessary
    if (plotFile)
    {
                if (strcmp (imageFormat->beamCombiner, "SCI_PHOT") == 0)        
                        midiCreatePlotFile2D ("SpectrumCombinedDATA2DATA3", "DATA2-DATA3, DATA2 and DATA3 Spectrum", 
                                "FFT bins for each Scan", "Power Spectrum", 0, compressed->allSpectrum, 0, 
                                imageFormat->allSpectrumLength, 1, 0);
                else
                        midiCreatePlotFile2D ("SpectrumCombinedDATA1DATA2", "DATA1-DATA2, DATA1 and DATA2 Spectrum", 
                                "FFT bins for each Scan", "Power Spectrum", 0, compressed->allSpectrum, 0, 
                                imageFormat->allSpectrumLength, 1, 0);
        }
        
        free (input2ft);
        free (FFTarray);
        
        return;
}
//******************************************************************************


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software 
*
* Module name:  normalizeSignal
* Input/Output: See function arguments to avoid duplication
* Description:  normalizes the content of an array. For instance eachspectrum, 
*                               to remove large numbers. FV has problems handling large numbers.
*
* History:      
* 11-Nov-03     (csabet) Created
******************************************************************************/
void normalizeSignal (
        int             arraySize,      /*      In:     Length of the array */
        float   *signal)        /*      IO:     Pointer to an array */
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "normalizeSignal";
        int                     i;
        float           maxSignal;
        
    /*  Algorithm
    -----------*/
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);
        
        /*      Find maximum. Ignore first two elements */
        maxSignal = signal[0];
        for (i = 0; i < arraySize; i++)
                if (signal[i] > maxSignal) maxSignal = signal[i];
                
        /*      Normalize signal */
        for (i = 0; i < arraySize; i++)
                signal[i] /= maxSignal;

    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  computeFrameFlux
* Input/Output: See function arguments to avoid duplication
* Description:  Computes the total flux in the given frame
*
* History:      
* 26-Aug-04     (csabet) Created
******************************************************************************/
void computeFrameFlux (
        short int       *inData,                //      In:     Pointer to the input data
        int                     frame,                  //      In:     Frame number
        int                     scalingOffset,  //      In:     Scaling factor
    ImageFormat *format,                //      In:     Pointer to the image format
        MidiCoords      *target,                //      In:     Region to cover
        float           *flux,                  //      Ou:     Computed flux
        int                     *pixelCount,    //      Ou      Pixel count
    int                 *error)                 //      Ou:     Error status

{
  
    //  Local Declarations
    //  ------------------
        const char  routine[] = "computeFrameFlux";
    int                 i, yWin, xWin, fileSpan, frameSpan, subWindowSize;
    
        //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //  Reset status
        *error = 0;
        *flux = 0.0;
        *pixelCount = 0;

        //      Compute flux in the given region
        subWindowSize = format->iXWidth * format->iYWidth;
        fileSpan = frame * subWindowSize;
        for (yWin = 0; yWin < format->iYWidth; yWin++)
        {
                frameSpan = yWin * format->iXWidth;
                for (xWin = 0; xWin < format->iXWidth; xWin++)
                {
                        //      Compute total flux
                        i = fileSpan + frameSpan + xWin;
/*
                        if (isnan (inData[i]))
                        {
                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Corrupted data");
                                *error = 1;
                                return;
                        }
*/
                        if (xWin >= (int) (target->xCoord) && xWin <= (int) (target->dxCoord) &&
                                yWin >= (int) (target->yCoord) && yWin <= (int) (target->dyCoord))
                        {
                                *flux += (float) (inData[i] + scalingOffset);
                                (*pixelCount)++;
                        }
                }
        }

        if (diagnostic)
        {
                cpl_msg_info(cpl_func,"\nCoordinates and flux results for frame %d: \n", frame);
                cpl_msg_info(cpl_func,"====================================== \n");
                cpl_msg_info(cpl_func,"   Image coordinates = %f %f %f %f\n", target->xCoord, target->yCoord, target->dxCoord, target->dyCoord);
                cpl_msg_info(cpl_func,"   Flux              = %f\n", *flux);
                cpl_msg_info(cpl_func,"   Pixel count       = %d\n", *pixelCount);
                fprintf (midiReportPtr, "\nCoordinates and flux results for frame %d: \n", frame);
                fprintf (midiReportPtr, "====================================== \n");
                fprintf (midiReportPtr, "   Image coordinates = %f %f %f %f\n", target->xCoord, target->yCoord, target->dxCoord, target->dyCoord);
                fprintf (midiReportPtr, "   Flux              = %f\n", *flux);
                fprintf (midiReportPtr, "   Pixel count       = %d\n", *pixelCount);
        }
        
    return; 
}
//******************************************************************************

        


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  computeImageFlux
* Input/Output: See function arguments to avoid duplication
* Description:  Computes the total flux in the given image
*
* History:      
* 26-Aug-04     (csabet) Created
******************************************************************************/
void computeImageFlux (
        float           *image,                 //      In:     Pointer to the input data
    ImageFormat *format,                //      In:     Pointer to the image format
        MidiCoords      *target,                //      In:     Region to cover
        float           *flux,                  //      Ou:     Computed flux
        int                     *pixelCount,    //      Ou      Pixel count
    int                 *error)                 //      Ou:     Error status

{
  
    //  Local Declarations
    //  ------------------
        const char  routine[] = "computeImageFlux";
    int                 i, yWin, xWin, frameSpan;
    
        //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //  Reset status
        *error = 0;
        *flux = 0.0;
        *pixelCount = 0;

        //      Compute flux in the given region
        for (yWin = 0; yWin < format->iYWidth; yWin++)
        {
                frameSpan = yWin * format->iXWidth;
                for (xWin = 0; xWin < format->iXWidth; xWin++)
                {
                        //      Compute total flux
                        i = frameSpan + xWin;
                        if (xWin >= (int) (target->xCoord) && xWin <= (int) (target->dxCoord) &&
                                yWin >= (int) (target->yCoord) && yWin <= (int) (target->dyCoord))
                        {
                                *flux += (float) (image[i]);
                                (*pixelCount)++;
                        }
                }
        }

    return; 
}
//******************************************************************************


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createAveragedImage
* Input/Output: See function arguments to avoid duplication
* Description:  Creates an averaged image from the FITS file
*
* History:      
* 18-Aug-04     (csabet) Created
******************************************************************************/
void createAveragedImage (
        short int       *inData,                //      In:     Pointer to the input data
        float           scalingOffset,  //      In:     Scaling Offset
    ImageFormat *format,                //      In:     Pointer to the image format
        float           *image)                 //      Ou:     Output image

{
    //  Local Declarations
    //  ------------------
    const char  routine[] = "createAveragedImage";
    int                 i, count, pixel, frame, subWindowSize;
    
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //  Reset status
        subWindowSize = format->iXWidth * format->iYWidth;

        for (pixel = 0; pixel < subWindowSize; pixel++)
        {
                //      Along the time axis (frame)
                image[pixel] = 0;
                count = 0;
                for (frame = 0; frame < format->numOfFrames; frame++)
            {
                        //      Load each pixel data along the time axis
                        i = frame * subWindowSize + pixel;
/*
                        if (isnan (inData[i]))
                        {
                                sprintf (midiMessage, "Found bad pixel on frame %d", frame);
                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                        }
                        else
*/
                        {
                                count++;
                                image[pixel] += (float) (inData[i] + scalingOffset);
                        }
                }

                //      Compute average image
                image[pixel] /= count;
        }
        
    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createFitsImage
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a fits file with the following format:
*                               outFileDir, outRootName, batchNumber, fileNumber
*                               e.g. MIDI_detRon_b1.ai1.fits
*                               Where "ai1" means "Average Image 1"
*
* History:      
* 31-Aug-04     (csabet) Created
******************************************************************************/
void createFitsImage (
        const char    *regionOrFile,  // In: Indicating region or file
        const char    *name,                  // In: Name of the image file
    char        *inFitsName,    // In: Name of FITS file
    int         xLength,                // In: X length
    int         yLength,                // In: Y length
    float   *image)             // In: Image of the file
{
  
    //  Local Declarations
    //  ------------------
    const char  routine[] = "createFitsImage";
    qfits_header    *pHeaderOut;
    FILE            *imagePtr;
    qfitsdumper     qdImage;
    char            *sWidthX, *sWidthY, *stringOut;
    struct stat         buf;
   
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

        cpl_msg_info(cpl_func,"\nCreating an image from %s for batch  %d \n", regionOrFile, batchNumber);
        cpl_msg_info(cpl_func,"----------------------------------- \n");
        fprintf (midiReportPtr, "\nCreating an image from %s for batch  %d \n", regionOrFile, batchNumber);
        fprintf (midiReportPtr, "----------------------------------- \n");
        
    //  Allocate memory
    stringOut = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sWidthY = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthX = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //  Write all NAXIS values as character strings
    sprintf (sWidthY, "%d", yLength);
    sprintf (sWidthX, "%d", xLength);

    //  Add primary Header
    if (diagnostic)cpl_msg_info(cpl_func,"   Extracting primary header from input FITS file   %s \n", inFitsName);
    fprintf(midiReportPtr, "   Extracting primary header from input FITS file   %s \n", inFitsName);
    pHeaderOut = qfits_header_read (inFitsName);
    if (pHeaderOut == NULL)
    {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot extract primary header from input FITS file");
            free (sWidthY);
            free (sWidthX);
                free (stringOut);
                return;
        }

    //  Updates header for the DetRon output file
        qfits_header_mod (pHeaderOut, "BITPIX","-32", "number of bits per pixel"); 
    qfits_header_mod (pHeaderOut, "NAXIS", "2", "number of data axes"); 
    qfits_header_add (pHeaderOut, "NAXIS1", sWidthX, "", "");
    qfits_header_add (pHeaderOut, "NAXIS2", sWidthY, "", "");
    
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET WIN1 NX", sWidthX, "# of pixels along x");
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET WIN1 NY", sWidthY, "# of pixels along y");
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET WIN1 STRX", "1", "Lower left pixel in x");
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET WIN1 STRY", "1", "Lower left pixel in y");
    
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET WIN2 NX", sWidthX, "# of pixels along x");
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET WIN2 NY", sWidthY, "# of pixels along y");
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET WIN2 STRX", "1", "Lower left pixel in x");
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET WIN2 STRY", "1", "Lower left pixel in y");
    
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET CHIP NX", sWidthX, "# of pixels along x");
//      qfits_header_mod (pHeaderOut, "HIERARCH ESO DET CHIP NY", sWidthY, "# of pixels along y");
    
    qfits_header_mod (pHeaderOut, "INSTRUME", "MIDI", "MIDI Raw Data Display FITS created by DRS pipeline" );  

    //  Create file and dump FITS Header for DetRon output file
        sprintf (stringOut, "%s%s.%s.fits", outFileDir, outRootName, name);
        if (stat (stringOut, &buf) == 0) remove (stringOut);    //      If the file exist delete it     
    if ((imagePtr = fopen (stringOut, "w")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Image of the data file");
            free (sWidthY);
            free (sWidthX);
                free (stringOut);
            qfits_header_destroy (pHeaderOut);
        return;
        }

        cpl_msg_info(cpl_func,"   Created an Image file: %s \n", stringOut);
        fprintf (midiReportPtr, "   Created an Image file: %s \n", stringOut);

    qfits_header_dump (pHeaderOut, imagePtr); 
    fclose (imagePtr);          

    //  Done writing header, close structure
    qfits_header_destroy (pHeaderOut);

    //  Dumps Fringe into the Detector Readout Noise file
        qdImage.filename  = stringOut;
        qdImage.npix      = xLength * yLength;
        qdImage.ptype     = PTYPE_FLOAT;
        qdImage.fbuf      = image;
        qdImage.out_ptype = BPP_IEEE_FLOAT;

    qfits_pixdump (&qdImage);

    /* Zero-pad the FITS file */
    qfits_zeropad((char *)stringOut);

    //  Release memory
    free (sWidthY);
    free (sWidthX);
    free (stringOut);
 
    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiGaussianFit
* Input/Output: See function arguments to avoid duplication
* Description:  This is a wrapper for the CPL library function
*
* History:      
* 18-Aug-04     (csabet) Created
******************************************************************************/
void midiGaussianFit (
        int             fileNumber,     // In: File number
        int             dimension,      // In: Dimensionality
        float   *image,         // In: Pointer to an image
        int             xImage,         // In: X size of image
        int             yImage,         // In: Y size of image
        int             xP,                     // In: X coordinate of the central pinhole
        int             yP,                     // In: Y coordinate of the central pinhole
        int             sizeP,          // In: Search size for the target
        double  *xT,            // Ou: X coordinate of the target centroid
        double  *yT,            // Ou: Y coordinate of the target centroid
        double  *sizeXT,        // Ou: Width of the target
        double  *sizeYT,        // Ou: Y-Width of the target
    int         *error)         // Ou: Error status

{
  
    //  Local Declarations
    //  ------------------
    const char  routine[] = "midiGaussianFit";
    cpl_image   *cplImage;
    double              sigX, sigY, norm;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        *xT = 0.0;
        *yT = 0.0;
        *sizeXT = 0.0;
        *sizeYT = 0.0;
        
        if (dimension == 1)
        {
                midiGetFWHM (fileNumber, image, xImage, yImage, sizeP, xT, yT, sizeXT, sizeYT, error);
                if (*error) midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find centroid");
        }
        else if (dimension == 2)
        {
                //      Using a 2D Gaussian fit find the coordinates of the targets. First build a cpl_image
                cplImage = cpl_image_wrap_float (xImage, yImage, image);
                
                //      There is a memory leak inside "cpl_image_gaussian_fit"
                if (cpl_image_fit_gaussian (cplImage, xP, yP, sizeP, &norm, xT, yT, &sigX, &sigY, NULL, NULL)!=CPL_ERROR_NONE){
                   *error = 1;
                   midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find the centroid");
                   cpl_image_unwrap (cplImage);
                   return;
                }
                
                *sizeXT = 0.5 * (sigX + sigY);
                *sizeYT = 0.5 * (sigX + sigY);

                cpl_image_unwrap (cplImage);
        }
        else
        {
                *error = 1;
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Invalid dimensionality");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find centroid");
                return;
        }

        //      If all is well print the result
        if (!(*error))
        {
                cpl_msg_info(cpl_func,"   Target centroid     = (x, y, X-Width, Y-Width) %f %f %f %f\n", 
                        *xT, *yT, *sizeXT, *sizeYT);
                fprintf (midiReportPtr, "   Target centroid     = (x, y, X-Width, Y-Width) %f %f %f %f\n", 
                        *xT, *yT, *sizeXT, *sizeYT);
        }

    return; 
}
//*****************************************************************************


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiGaussian_1d_fit
* Input/Output: See function arguments to avoid duplication
* Description:  This function performs a 1d Gaussian fit
*
* History:      
* 23-Sep-04     (csabet) Created
******************************************************************************/
void midiGaussian_1d_fit (
        float   *array,                 //      In:     Pointer to an array of data
        int             length,                 //      In:     length of the array
        int             sizeS,                  //      In: Search size for the target
        float   *centre,                //      Ou:     Position of the target centroid
        float   *sizeT,                 //      Ou:     Size of the target
        float   *fluxErr2Min,   //      Ou:     Flux in the search region
    int         *error)                 //      Ou:     Error status

{
  
    //  Local Declarations
    //  ------------------
    const char  routine[] = "midiGaussian_1d_fit";
    int                 i, j, l, searchLimit, halfGauss;
    float               *gaussian, sigma, exponentConst, fluxErr2;
    char                *fileString, *title;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        *fluxErr2Min = 0.0;
        *centre = 0.0;
        *sizeT = 0.0;
        sigma = 0.0;

        //      Exception handling. Make sure sizeS is odd
        if (diagnostic > 2) cpl_msg_info(cpl_func,"search span before = %d\n", sizeS);
        if (!(sizeS & 1)) sizeS += 1;
        if (diagnostic > 2) cpl_msg_info(cpl_func,"search span after = %d\n", sizeS);
        searchLimit = length - sizeS;
        if (searchLimit <= sizeS)
        {
                *error = 1;
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Search window too large");
                return;
        }

        //      Allocate memory
        gaussian = (float *) calloc (sizeS, sizeof (float));

        //      Determine the symetrical search span
        halfGauss = (sizeS >> 1);

        //      Plot the input array
        if (diagnostic > 2)
                midiCreatePlotFile2D ("InputArray", "Input Array", "X", "Y", 0, array, 0, length, 1, 0);

        //      Search with different values of Gaussian sigma
        for (l = 0; l < GAUSSIAN_SIGMA_SPAN; l++)
        {
                //      Define the Normalized Gaussian with a=0
                sigma += 0.25;
                exponentConst = (-0.5) / (sigma * sigma);
                for (i = -halfGauss; i <= halfGauss; i++)
                {
                        j = halfGauss+i;
                        gaussian[j] = exp (exponentConst * i * i);
                        if (plotFile && diagnostic > 2) 
                        {
                                fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                                title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                                sprintf (fileString , "GaussianWithSigma%f", sigma);
                                sprintf (title , "Gaussian with %f Sigma", sigma);
                                midiCreatePlotFile2D (fileString, title, "Span", "Sigma", 0, gaussian, 0, sizeS, 1, 0);
                                free (fileString);
                                free (title);
                                if (diagnostic > 2)
                                {
                                        cpl_msg_info(cpl_func,"   Gaussian %d = %8.8f\n", i, gaussian[j]);
                                        fprintf (midiReportPtr, "   Gaussian %d = %8.8f\n", i, gaussian[j]);
                                }
                        }
                }
                        
                //      Find the section where squared error is minimum
                //      -----------------------------------------------
                for (i = 0; i < searchLimit; i++)
                {
                        //      Correlate flux
                        fluxErr2 = 0.0;
                        for (j = 0; j < sizeS; j++)     
                                fluxErr2 += ( (gaussian[j] - array[i+j]) * (gaussian[j] - array[i+j]) );
                        
                        //      Initialise the mimimum  
                        if (l == 0 && i == 0) *fluxErr2Min = fluxErr2;
                        else
                        {
                                if (fluxErr2 < *fluxErr2Min)
                                {
                                        *fluxErr2Min = fluxErr2;
                                        *centre = (float) (i + 0.5 * sizeS);
                                        *sizeT = (float) (3.0 * sigma);
                                }
                        }
                }
        }
        
        //      Release memory
        free (gaussian);
        
    return; 
}
//*****************************************************************************


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiGaussianSmooth
* Input/Output: See function arguments to avoid duplication
* Description:  Using a Gaussian fit computes the refined position of peak
*                               of a pinhole spectra
*
* History:      
* 31-Aug-05     (csabet) Created
******************************************************************************/
float midiGaussianSmooth (      //      Ou: Refined peak
        float   *array,                 //      In:     Pointer to an array of normalised data
        int             length,                 //      In:     length of the array
        int             peakIn,                 //      In: Approximate peak
        int             searchSpan,             //      In:     Search span
    int         *error)                 //      Ou:     Error status

{
  
    //  Local Declarations
    //  ------------------
    const char  routine[] = "midiGaussianSmooth";
    int                 i, j, l, start, end, halfGauss;
    float               *gaussian, sigma, exponentConst, fluxErr2, xCPH, fluxErr2Min;
        char            *fileString, *title;
                
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
        fluxErr2Min = 0.0;
        sigma = 0.0;
        xCPH = 0.0;

        //      Exception handling. Make sure searchSpan is odd
        if (diagnostic > 2) cpl_msg_info(cpl_func,"search span before = %d\n", searchSpan);
        if (!(searchSpan & 1)) searchSpan += 1;
        if (diagnostic > 2) cpl_msg_info(cpl_func,"search span after = %d\n", searchSpan);
        halfGauss = (searchSpan >> 1);
        start = peakIn - searchSpan;
        end = peakIn + searchSpan;
        if ((end > length) || (start < 0))
        {
                *error = 1;
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Search window too large");
                return (-1);
        }

        //      Allocate memory
        gaussian = (float *) calloc (searchSpan, sizeof (float));

        //      Search with different values of Gaussian sigma
        for (l = 0; l < GAUSSIAN_SIGMA_SPAN; l++)
        {
                //      Define the Normalized Gaussian with a=0
                sigma += 0.25;
                exponentConst = (-0.5) / (sigma * sigma);
                for (i = -halfGauss; i <= halfGauss; i++)
                {
                        j = halfGauss+i;
                        gaussian[j] = exp (exponentConst * i * i);
                        if (plotFile && diagnostic > 2) 
                        {
                                fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                                title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                                sprintf (fileString , "GaussianWithSigma%f", sigma);
                                sprintf (title , "Gaussian with %f Sigma", sigma);
                                midiCreatePlotFile2D (fileString, title, "Span", "Sigma", 0, gaussian, 0, searchSpan, 1, 0);
                                cpl_msg_info(cpl_func,"   Gaussian %d = %8.8f\n", i, gaussian[j]);
                                fprintf (midiReportPtr, "   Gaussian %d = %8.8f\n", i, gaussian[j]);
                        }
                }
        
                //      Find the section where squared error is minimum
                //      -----------------------------------------------
                for (i = start; i < end; i++)
                {
                        //      Correlate flux
                        fluxErr2 = 0.0;
                        for (j = 0; j < searchSpan; j++)        
                                fluxErr2 += ( (gaussian[j] - array[i+j]) * (gaussian[j] - array[i+j]) );
                        
                        //      Initialise the mimimum  
                        if ((l == 0) && (i == start)) 
                                fluxErr2Min = fluxErr2;
                        else
                        {
                                if (fluxErr2 < fluxErr2Min)
                                {
                                        fluxErr2Min = fluxErr2;
                                        xCPH = (float) (i + 0.5 * searchSpan);
                                }
                        }
                }
        }
        
        //      Release memory
        free (gaussian);
        
    return (xCPH); 
}
//*****************************************************************************


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  getBadScansFromSpectrumUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Rejecting bad scans based on the signal/noise values
*
* History:
* 14-Mar-06     (csabet) Created
******************************************************************************/
void getBadScansFromSpectrumUndisp (
    FilterData          *filterInfo,    // In: Pointer to filter information
    ImageFormat         *format,                // In: Points to the image format
        CompressedData  *compressed,    // IO: Pointer to compressed data
        int                             *error)                 // Ou: Error status
{
    //  Local Declarations
    //--------------------
    const char  routine[] = "getBadScansFromSpectrumUndisp";
        int                     i;
        float           *tempArray;

    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine '%s'  \n", routine);

    cpl_msg_info(cpl_func,"\nRejecting scans based on Signal/Noise \n");
    cpl_msg_info(cpl_func,"------------------------------------- \n");
    fprintf (midiReportPtr, "\nRejecting scans based on Signal/Noise \n");
    fprintf (midiReportPtr, "------------------------------------- \n");

        //      Reset
        *error = 0;
        
        //      Get bad scans due to weak signal/noise
        rejectScansOnWeakSNRUndisp (filterInfo, format, compressed, error);
                
        //      Now compute number of bad scans 
        format->numOfScansRejected = 0;
        format->numOfScansProcessed = 0;
        for (i = 0; i < format->numOfScans; i++)
        {
                if (compressed->badScanList[i])
                        format->numOfScansRejected++;
        }
        format->numOfScansProcessed = format->numOfScans - format->numOfScansRejected;

        //      Create plot file
        if (plotFile && diagnostic) 
        {
                tempArray = (float *) calloc (format->numOfScans, sizeof (float));
                for (i = 0; i < format->numOfScans; i++)
                {
                        if (compressed->badScanList[i] == 0)
                                tempArray[i] = 0.0;
                        else
                                tempArray[i] = compressed->badScanList[i];
                }
                midiCreatePlotFile2D ("BadScanList", "Bad Scan List", "Scan", "Failure ID", 
                        0, tempArray, 0, format->numOfScans, 1, 0);
                free (tempArray);
        }

        //      Report the bad scan list
        if (diagnostic) cpl_msg_info(cpl_func,"\nBad Scan List \n");
        if (diagnostic) cpl_msg_info(cpl_func,"------------- \n");
        fprintf (midiReportPtr, "\nBad Scan List QCLOG \n");
        fprintf (midiReportPtr, "-------------   QCLOG \n");
        for (i = 0; i < format->numOfScans; i++)
        {
                if (diagnostic) cpl_msg_info(cpl_func,"%3d  %3d \n", i, compressed->badScanList[i]);
                fprintf (midiReportPtr, "%3d  %3d     QCLOG \n", i, compressed->badScanList[i]);
        }
        //      Report the bad scan list
        cpl_msg_info(cpl_func,"Number of scans to process = %3d \n", format->numOfScansProcessed);
        cpl_msg_info(cpl_func,"Number of scans rejected   = %3d \n", format->numOfScansRejected);
        fprintf (midiReportPtr, "Number of scans to process = %3d QCLOG \n", format->numOfScansProcessed);
        fprintf (midiReportPtr, "Number of scans rejected   = %3d QCLOG \n", format->numOfScansRejected);
        if (diagnostic) 
        {
                cpl_msg_info(cpl_func,"Failure ID \n");
                cpl_msg_info(cpl_func,"---------- \n");
                cpl_msg_info(cpl_func,"1    = Bit  1, indicates a TIME error \n");
                cpl_msg_info(cpl_func,"2    = Bit  2, indicates an LOCALOPD error \n");
                cpl_msg_info(cpl_func,"4    = Bit  3, indicates an OPD error \n");
                cpl_msg_info(cpl_func,"8    = Bit  4, indicates a Bad DATA \n");
                cpl_msg_info(cpl_func,"16   = Bit  5, indicates a chopping problem (TARTYPE2) \n");
                cpl_msg_info(cpl_func,"32   = Bit  6, indicates a Delay Line Jump \n");
                cpl_msg_info(cpl_func,"64   = Bit  7, indicates an Unwanted Region \n");
                cpl_msg_info(cpl_func,"128  = Bit  8, indicates a Weak Signal/Noise \n");
                cpl_msg_info(cpl_func,"256  = Bit  9, indicates a SKY Background \n");
                cpl_msg_info(cpl_func,"512  = Bit 10, indicates a TARTYPE Cross error. e.g. region 0 different to region 1 \n");
                cpl_msg_info(cpl_func,"\nHence a number such as 168 indicates bits 4, 6 and 8 were set \n\n");        
        }

        fprintf (midiReportPtr, "Failure ID QCLOG \n");
        fprintf (midiReportPtr, "---------- QCLOG \n");
        fprintf (midiReportPtr, "1    = Bit  1, indicates a TIME error \n");
        fprintf (midiReportPtr, "2    = Bit  2, indicates an LOCALOPD error \n");
        fprintf (midiReportPtr, "4    = Bit  3, indicates an OPD error \n");
        fprintf (midiReportPtr, "8    = Bit  4, indicates a Bad DATA \n");
        fprintf (midiReportPtr, "16   = Bit  5, indicates a chopping problem (TARTYPE2) \n");
        fprintf (midiReportPtr, "32   = Bit  6, indicates a Delay Line Jump \n");
        fprintf (midiReportPtr, "64   = Bit  7, indicates an Unwanted Region \n");
        fprintf (midiReportPtr, "128  = Bit  8, indicates a Weak Signal/Noise \n");
        fprintf (midiReportPtr, "256  = Bit  9, indicates a SKY Background \n");
        fprintf (midiReportPtr, "512  = Bit 10, indicates a TARTYPE Cross error. e.g. region 0 different to region 1 \n");
        fprintf (midiReportPtr, "\nHence a number such as 168 indicates bits 4, 6 and 8 were set QCLOG \n\n");
        
        if (format->numOfScansProcessed < 1)
        {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "No Scan to process");
                *error = 1;
        }
        
    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  rejectScansOnWeakSNRUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Uses Power Spectrum to get the bad scans. This routine
*                               is used for HIGH_SENS UNDISPERSED processing
*
* History:
* 15-Mar-06     (csabet) Created
******************************************************************************/
void rejectScansOnWeakSNRUndisp (
    FilterData          *filterInfo,    // In: Pointer to filter information
    ImageFormat         *format,                // In: Points to the image format
        CompressedData  *compressed,    // IO: Pointer to compressed data
        int                             *error)                 // Ou: Error status
{
    //  Local Declarations
    //--------------------
    const char  routine[] = "rejectScansOnWeakSNRUndisp";
    float       *sigPtr, *spectrum, median, peak, ratio, calib;
    int         i, j, fftSizeHalf, peakIndex, loF, hiF;
        char            *fileName, *title;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine '%s'  \n", routine);

    //  Initialization
        *error = 0;
        fftSizeHalf = (format->fftsize >> 1);

        //      Compute indices of the valid frequency range
        calib = format->optFreqCal / ((float) format->fftsize);
        loF = (int) (filterInfo->optFreqLo / calib);
        hiF = (int) (filterInfo->optFreqHi / calib);
        if (hiF > format->fftsize/2) hiF = format->fftsize/2;

    //  Allocate memory
    spectrum = (float *) calloc (fftSizeHalf, sizeof (float));
        
        //      Analyse spectrum for each scan
    for (i = 0; i < format->numOfScans; i++)
    {
            sigPtr = compressed->allSpectrum + i * fftSizeHalf;

                //      Process good scans only
                if (!(compressed->badScanList[i]))
                {
                        //      Load spectrum for this scan
                for (j = 0; j < fftSizeHalf; j++)
                            spectrum[j] = *sigPtr++;

                        //      Analyse the scan spectrum
                        peak = signalPeak (spectrum, 0, fftSizeHalf, &peakIndex);
                        median = signalMedian (spectrum, 0, fftSizeHalf);
                        ratio = peak/median;
                        
                        //      Check validity of the peak power
                        if ((ratio < 10.0) || (peakIndex < loF) || (peakIndex > hiF))
                                compressed->badScanList[i] |= BSL_SNR_ERROR;

                        //      Plot
                        if (plotFile && diagnostic > 2)
                        {
                                if (compressed->badScanList[i]) cpl_msg_info(cpl_func,"\n    <<ERROR>>: Scan %3d rejected \n\n", i);
                                cpl_msg_info(cpl_func,"    Expected low frequency index  = %3d \n", loF);
                                cpl_msg_info(cpl_func,"    Expected high frequency index = %3d \n", hiF);
                                cpl_msg_info(cpl_func,"    Found peak index at           = %3d \n", peakIndex);
                                cpl_msg_info(cpl_func,"    peak/median                   = %f \n", ratio);
                                if (compressed->badScanList[i]) fprintf (midiReportPtr, "\n    <<ERROR>>: Scan %3d rejected \n\n", i);
                                fprintf (midiReportPtr, "    Expected low frequency index  = %3d \n", loF);
                                fprintf (midiReportPtr, "    Expected high frequency index = %3d \n", hiF);
                                fprintf (midiReportPtr, "    Found peak index at           = %3d \n", peakIndex);
                                fprintf (midiReportPtr, "    peak/median                   = %f \n", ratio);

                                fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                                title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                                sprintf (fileName, "SpectrumScan%d", i);
                                sprintf (title, "Power Spectrum. Scan %d", i);
                                midiCreatePlotFile2D (fileName, title, "Frequancy bin", "Power Spectrum", 
                                        1, spectrum, 0, fftSizeHalf, 1, 0);
                                free (fileName);
                                free (title);
                        }
                }
        }

    //  release memory
        free (spectrum);

    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiGetFWHM
* Input/Output: See function arguments to avoid duplication
* Description:  This function computes the centre and size of spectroscopic
*                               image. The computation will return the Full Width at 
*                               Half Maximum (FWHM)     of the image. 
*
* History:      
* 23-Sep-04     (csabet) Created
******************************************************************************/
void midiGetFWHM (
        int             imageCount,     // In: Image number or ID
        float   *image,         // In: Pointer to an image
        int             xImage,         // In: X size of image
        int             yImage,         // In: Y size of image
        int             sizeP,          // In: Search size for the target
        double  *xT,            // Ou: X coordinate of the target centroid
        double  *yT,            // Ou: Y coordinate of the target centroid
        double  *sizeXT,        // Ou: X-Width of the target
        double  *sizeYT,        // Ou: Y-Width of the target
    int         *error)         // Ou: Error status

{
  
    //  Local Declarations
    //  ------------------
    const char  routine[] = "midiGetFWHM";
    int                 i, j, k, l, length, maxLength, start, end;
    float               *arrayData, centre, *centrePositions, *centreFluxErr2Min,
                                fluxErr2Min, *span, siztT, fluxMax, *buffImage, max;
    char                *fileName, *title;
        
    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

        //      Reset status
        *error = 0;
                
        //      Allocate memory
        length = xImage * yImage;
        maxLength = (xImage > yImage) ? xImage : yImage;        // get the largest size
        arrayData = (float *) calloc (maxLength, sizeof (float));
        centrePositions = (float *) calloc (maxLength, sizeof (float));
        centreFluxErr2Min = (float *) calloc (maxLength, sizeof (float));
        span = (float *) calloc (maxLength, sizeof (float));
        buffImage = (float *) calloc (length, sizeof (float));

        //      Copy image
        for (i = 0; i < length; i++)
                buffImage[i] = image[i];
                
        //      Normalize image for a better 2D correlation
        fluxMax = buffImage[0];
        for (i = 0; i < length; i++)
                if (buffImage[i] > fluxMax) fluxMax = buffImage[i];
        for (i = 0; i < length; i++)
                buffImage[i] /= fluxMax;

        //      Determine Y and X-Width
        //      -----------------------
        //      For every row, y = 0, .. yImage
        for (i = 0; i < yImage; i++)
        {
                //      Load row data, x = 0, .. xImage
                for (j = 0; j < xImage; j++)
                {
                        k = j + i * xImage;
                        arrayData[j] = buffImage[k];
                }
                        
                //      For every row compute centre, size and Error Mean Squared of the Gaussian profile
                midiGaussian_1d_fit (arrayData, xImage, sizeP, &centre, &siztT, &fluxErr2Min, error);
                if (*error)
                {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot determine target parameters");
                        free (arrayData);
                        free (centrePositions);
                        free (centreFluxErr2Min);
                        free (span);
                        free (buffImage);
                        return;
                }
                centrePositions[i] = centre;            // Centre positions of the target along Y
                centreFluxErr2Min[i] = fluxErr2Min;     // Graphical presentation of the target centre
                span[i] = siztT;                                        // Variations of Gaussian Span along Y
        }

        //      Find optimum Y
        *sizeXT = 0;
        for (i = 0; i < yImage; i++)
        {
                if (span[i] > *sizeXT)
                {
                        *yT = i; 
                        *sizeXT = span[i];
                }
        }

        //      Find FWHM
        max = 0.0;
        l = *yT * xImage;
        for (j = 0; j < xImage; j++)
        {
                //      Load row data, x = 0, .. xImage
                k = j + l;
                arrayData[j] = buffImage[k];
                if (arrayData[j] > max) max = arrayData[j];
        }
        start = 0;
        end = 0;
        for (j = 0; j < xImage; j++)
        {
                if (arrayData[j] > max/2.0)
                {
                        start = j;
                        break;
                }
        }
        for (j = xImage-1; j >= 0; j--)
        {
                if (arrayData[j] > max/2.0)
                {
                        end = j;
                        break;
                }
        }
        *sizeXT = (end - start) * CIRCLE_TO_SQUARE;     // Hopefully taking the central region of the target only
        
        //      Create plot file
        if (plotFile)
        {
                fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

                sprintf (fileName, "TargetFluxMinVarX%d", imageCount);
                sprintf (title, "Target Flux Minimum Variance for each Y, beam %d", imageCount);
                midiCreatePlotFile2D (fileName, title, "Y", "Flux", 0, centreFluxErr2Min, 0, yImage, 1, 0);

                sprintf (fileName, "TargetPositionX%d", imageCount);
                sprintf (title, "Target Centre along Y, beam %d", imageCount);
                midiCreatePlotFile2D (fileName, title, "Y", "X", 0, centrePositions, 0, yImage, 1, 0);

                sprintf (fileName, "TargetX_WidthAlongY%d", imageCount);
                sprintf (title, "Target X-Width along Y, beam %d", imageCount);
                midiCreatePlotFile2D (fileName, title, "Y", "X-Width", 0, span, 0, yImage, 1, 0);

                sprintf (fileName, "TargetMaximumProfileAlongY%d", imageCount);
                sprintf (title, "Target Maximum Profile at Y = %f, beam %d", *yT, imageCount);
                midiCreatePlotFile2D (fileName, title, "X", "Flux", 0, arrayData, 0, xImage, 1, 0);
        
                //      Release memory
                free (fileName);
                free (title);
        }

        //      Determine X and Y-Width
        //      ---------------------
        //      For every column, x = 0, .. xImage
        for (i = 0; i < xImage; i++)
        {
                //      Load column data y = 0, .. yImage
                for (j = 0; j < yImage; j++)
                {
                        k = i + j * xImage;
                        arrayData[j] = buffImage[k];
                }
                        
                //      For every column compute centre, size and Error Mean Squared of the Gaussian profile
                midiGaussian_1d_fit (arrayData, yImage, sizeP, &centre, &siztT, &fluxErr2Min, error);
                if (*error)
                {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot determine target parameters");
                        free (arrayData);
                        free (centrePositions);
                        free (centreFluxErr2Min);
                        free (buffImage);
                        free (span);
                        return;
                }
                centrePositions[i] = centre;            // Centre positions of the target along X
                centreFluxErr2Min[i] = fluxErr2Min;     // Graphical presentation of the target centre
                span[i] = siztT;                                        // Variations of target Y-Width along X
        }

        *sizeYT = 0;
        for (i = 0; i < xImage; i++)
        {
                if (span[i] > *sizeYT)
                {
                        *xT = i; 
                        *sizeYT = span[i];
                }
        }

        //      Find FWHM
        max = 0.0;
        for (i = 0; i < yImage; i++)
        {
                //      Load column data, y = 0, .. yImage
                k = *xT + i * xImage;
                arrayData[i] = buffImage[k];
                if (arrayData[i] > max) max = arrayData[i];
        }
        start = 0;
        end = 0;
        for (j = 0; j < yImage; j++)
        {
                if (arrayData[j] > max/2.0)
                {
                        start = j;
                        break;
                }
        }
        for (j = yImage-1; j >= 0; j--)
        {
                if (arrayData[j] > max/2.0)
                {
                        end = j;
                        break;
                }
        }
        *sizeYT = (end - start) * CIRCLE_TO_SQUARE;     // Hopefully taking the central region of the target only

        //      Create plot file
        if (plotFile)
        {
                fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

                sprintf (fileName, "TargetFluxMinVarY%d", imageCount);
                sprintf (title, "Target Flux Minimum Variance for each X, beam %d", imageCount);
                midiCreatePlotFile2D (fileName, title, "X", "Flux", 0, centreFluxErr2Min, 0, xImage, 1, 0);

                sprintf (fileName, "TargetPositionY%d", imageCount);
                sprintf (title, "Target Centre along Y, beam %d", imageCount);
                midiCreatePlotFile2D (fileName, title, "X", "Y", 0, centrePositions, 0, xImage, 1, 0);

                sprintf (fileName, "TargetY_WidthAlongY%d", imageCount);
                sprintf (title, "Target Y-Width along Y, beam %d", imageCount);
                midiCreatePlotFile2D (fileName, title, "X", "Y-Width", 0, span, 0, xImage, 1, 0);

                sprintf (fileName, "TargetMaximumProfileAlongX%d", imageCount);
                sprintf (title, "Target Maximum Profile at X = %f, beam %d", *xT, imageCount);
                midiCreatePlotFile2D (fileName, title, "Y", "Flux", 0, arrayData, 0, yImage, 1, 0);
        
                //      Release memory
                free (fileName);
                free (title);
        }

        //      Release memory
        free (arrayData);
        free (centrePositions);
        free (centreFluxErr2Min);
        free (span);
        free (buffImage);
        
    return; 
}
//*****************************************************************************


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  removeSkyBackground
* Input/Output: See function arguments to avoid duplication
* Description:  Removes sky background.
*
* History:
* 08-Aug-05     (csabet) Created
* 13-Feb-06     (csabet) Completely redesigned
******************************************************************************/
void removeSkyBackground (
    const char                                *shutterId,             // In: AOPEN, BOPEN etc.
    enum ProcessingMode processing,             // In: Whether DISP or UNDISP
    ImageFormat                 *format,                // In: Format parameters
    CompressedData              *compressed,    // Ou: Pointer to the compressed data structure
    int                                 *error)                 // Ou: Error status

{
    //  Local Declarations
    //  ------------------
    const char  routine[] = "removeSkyBackground";
        int                     i, j, S, T, R, X, shift, start;
        char            lastType;

    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

        sprintf (midiMessage, "Removing sky background from %s", shutterId);
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);

        //      Initialise
        S = T = 0;
        start = 0;
        lastType = ' ';
        *error = 0;
        
        //      Compute shift
        for (i = 0; i < format->numOfFrames; i++)
        {
                if (compressed->tarType[i] == 'T')
                {
                        if (lastType == 'U' && T && S)
                        {
                                T = 0;
                                S = 0;
                        }
                        T++;
                        if (S)
                        {
                                shift = i-start;
                                for (j = start; j < start+S; j++)
                                {
                                        if (compressed->tarType[j] != 'U')
                                        {
                                                if (diagnostic > 4)
                                                {
                                                        cpl_msg_info(cpl_func,"%4d-%4d   %c-%c \n", j+shift, j, 
                                                                compressed->tarType[j+shift], compressed->tarType[j]);
                                                        fprintf (midiReportPtr, "%4d-%4d   %c-%c \n", j+shift, j, 
                                                                compressed->tarType[j+shift], compressed->tarType[j]);
                                                }
                                                compressed->iFringe[j] = compressed->iFringe[j+shift] - compressed->iFringe[j];
                                                compressed->iFringe1[j] = compressed->iFringe1[j+shift] - compressed->iFringe1[j];
                                                compressed->iFringe2[j] = compressed->iFringe2[j+shift] - compressed->iFringe2[j];
                                                if (processing == DISPERSED)
                                                {
                                                        for (R = 0; R < format->numOfRegionsToProcess; R++)
                                                        {
                                                                for (X = 0; X < format->iXWidth; X++)
                                                                {
                                                                        (((compressed->iDispFringe)[R])[X])[j] = (((compressed->iDispFringe)[R])[X])[j+shift] - 
                                                                                (((compressed->iDispFringe)[R])[X])[j];
                                                                }
                                                        }
                                                }
                                        }
                                }
                                i = j+shift-1;
                        }
                }
                else if (compressed->tarType[i] == 'S')
                {
                        if (lastType == 'U' && T && S)
                        {
                                S = 0;
                                T = 0;
                        }
                        S++;
                        if (T)
                        {
                                shift = i-start;
                                for (j = start; j < start+T; j++)
                                {
                                        if (compressed->tarType[j] != 'U')
                                        {
                                                if (diagnostic > 4)
                                                {
                                                        cpl_msg_info(cpl_func,"%4d-%4d   %c-%c \n", j, j+shift, 
                                                                compressed->tarType[j], compressed->tarType[j+shift]);
                                                        fprintf (midiReportPtr, "%4d-%4d   %c-%c \n", j, j+shift, 
                                                                compressed->tarType[j], compressed->tarType[j+shift]);
                                                }
                                                compressed->iFringe[j] = compressed->iFringe[j] - compressed->iFringe[j+shift];
                                                compressed->iFringe1[j] = compressed->iFringe1[j] - compressed->iFringe1[j+shift];
                                                compressed->iFringe2[j] = compressed->iFringe2[j] - compressed->iFringe2[j+shift];
                                                if (processing == DISPERSED)
                                                {
                                                        for (R = 0; R < format->numOfRegionsToProcess; R++)
                                                        {
                                                                for (X = 0; X < format->iXWidth; X++)
                                                                {
                                                                        (((compressed->iDispFringe)[R])[X])[j] = (((compressed->iDispFringe)[R])[X])[j] - 
                                                                                (((compressed->iDispFringe)[R])[X])[j+shift];
                                                                }
                                                        }
                                                }
                                        }
                                }
                                i = j+shift-1;
                        }
                }
                else
                {
                        if (diagnostic > 4)
                        {
                                fprintf (midiReportPtr, "%4d %c \n", i, compressed->tarType[i]);
                                cpl_msg_info(cpl_func,"%4d %c \n", i, compressed->tarType[i]);
                        }
                        lastType = 'U';
                        if ((S && T) || (!S && !T))
                        {
                                start = i+1;
                        }
                }
        }

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  correctTarType
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a mask from the tarType. Esures number of consecutive 
*                               T and S are equal. Extras should be labled 'U'. New tarType
*                               is then created
*
* History:
* 09-Feb-06     (csabet) Created
******************************************************************************/
long correctTarType (
		const char    *shutterId,
        char          *tarType,
        float         *TimeStamp,
        int            length,
        int            *error)
{

    //  Local Declarations
    //  ------------------
    const char  routine[] = "correctTarType";
    int                     i, j, foundT, foundS, S, T, U, start, end, min, newPair;
    char            *mask, lastType, lastShot;
    long        CounterBad=0;

    //  Algorithm
    //  ---------
    if (diagnostic > 4) cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

        sprintf (midiMessage, "Checking tarType inconsistencies for %s", shutterId);
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);

        //      Initialise
    foundT = foundS = 0;
    T = S = U = 0;
    start = 0;
    lastType = ' ';
    lastShot = ' ';
    newPair = 0;
    *error = 0;
    i=0;
	j=0	;
	CounterBad=0;
        //      Allocate memory
	mask = (char *) calloc (length, sizeof (char));

/*
	Check if the time is valid or not. If not, change tarType to 'U'
	If a SUS or TUT sequence is caused by this change, set all previous frames
	to 'U' until a TUS or a SUT sequence is re-established
*/

	for (i = 0; i < length; i++)
	{
		if(isnan(TimeStamp[i]))
		{
			cpl_msg_warning(cpl_func,"Corrupted time stamp in table! ");
			cpl_msg_warning(cpl_func,"Changing target type of frame %d "
					"from %c to %c", i, tarType[i], 'U');
			tarType[i]='U';
			CounterBad++;
			if(i>0 && i<(length-1) && tarType[i-1]=='S' && tarType[i+1]=='S')
			{
				j=i-1;
				while(tarType[j]!='T' && j>=0)
				{
					cpl_msg_warning(cpl_func,"Changing target type of frame %d "
							"from %c to %c", j, tarType[j], 'U');
					tarType[j]='U';
					CounterBad++;
					j--;
				}
			}
			if(i>0 && i<(length-1) && tarType[i-1]=='T' && tarType[i+1]=='T')
			{
				j=i-1;
				while(tarType[j]!='S' && j>=0)
				{
					cpl_msg_warning(cpl_func,"Changing target type of frame %d "
							"from %c to %c", j, tarType[j], 'U');
					tarType[j]='U';
					CounterBad++;
					j--;
				}

			}

		}
	}

	for (i = 1; i < length-2; i++)
	{
		if(tarType[i-1]=='U' && (tarType[i]=='S' || tarType[i]=='T') &&
				tarType[i+1]=='U')
		{
			cpl_msg_warning(cpl_func,"UTU or USU sequence found, "
					"correcting to UUU");
			tarType[i]='U';
		}
	}


	for (i = 0; i < length-1; i++)
	{
		if(tarType[i]=='T' && tarType[i+1]=='S')
		{
			cpl_msg_warning(cpl_func,"Switching from T to S without U at "
					"frame %d", i);
			cpl_msg_warning(cpl_func,"Changing target type of frame %d and %d"
					" to %c", i, i+1, 'U');
			tarType[i]='U';
			CounterBad++;
			tarType[i+1]='U';
			CounterBad++;
		}

		if(tarType[i]=='S' && tarType[i+1]=='T')
		{
			cpl_msg_warning(cpl_func,"Switching from S to T without U at "
					"frame %d", i);
			cpl_msg_warning(cpl_func,"Changing target type of frame %d and %d"
					" to %c", i, i+1, 'U');
			tarType[i]='U';
			CounterBad++;
			tarType[i+1]='U';
			CounterBad++;
		}

	}


        for (i = 0; i < length; i++)
        {
                //      Count number of Targets
                if (tarType[i] == 'T')
                {
                        if (lastType == 'S')
                        {
                                sprintf (midiMessage, "Switching from S to T without U at frame %d. QCLOG", i);
                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                                *error = 1;
                        }
                        if (lastShot == 'T')
                        {
                                sprintf (midiMessage, "Switching from T to T without S at frame %d. QCLOG", i);
                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                                *error = 1;
                        }
                        T++;
                        mask[i] = 'T';
                        lastType = 'T';
                        newPair = 0;
                }
                //      Count number of Skys
                else if (tarType[i] == 'S')
                {
                        if (lastType == 'T')
                        {
                                sprintf (midiMessage, "Switching from T to S without U at frame %d. QCLOG", i);
                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                                *error = 1;
                        }
                        if (lastShot == 'S')
                        {
                                sprintf (midiMessage, "Switching from S to S without T at frame %d. QCLOG", i);
                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                                *error = 1;
                        }
                        S++;
                        mask[i] = 'S';
                        lastType = 'S';
                        newPair = 0;
                }
                else
                {
                        mask[i] = 'U';
                        if (lastType == 'T')
                        {
                                lastShot = 'T';
                                foundT = 1;
                        }
                        if (lastType == 'S')
                        {
                                lastShot = 'S';
                                foundS = 1;
                        }
                        lastType = 'U';
                        if (newPair) 
                                start++;
                }
                //      Check sizes and apply corrections
                if ((foundS && foundT) || (foundS && T && (i == length-1)) || (foundT && S && (i == length-1)))
                {
                        end = i+1;
                        if (S < T) 
                                min = S;
                        else 
                                min = T;
                                        
                        T = S = 0;
                        for (j = start; j < end; j++)
                        {
                                if (mask[j] == 'T')
                                {
                                        T++;
                                        if (T > min)
                                        {
                                                mask[j] = 'U';
                                                sprintf (midiMessage, "Chopping inconsistency. Found extra 'T' at frame %d. Corrected. QCLOG", j);
                                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                                        }
                                }
                                else if (mask[j] == 'S')
                                {
                                        S++;
                                        if (S > min)
                                        {
                                                mask[j] = 'U';
                                                sprintf (midiMessage, "Chopping inconsistency. Found extra 'S' at frame %d. Corrected. QCLOG", j);
                                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                                        }
                                }
                        }
                        T = S = 0;
                        if (foundS && foundT)
                        {
                                newPair = 1;
                                start = i;
                                --i;
                        }
                        foundS = foundT = 0;
                }
                else if ((!foundS && T && (i == length-1)) || (!foundT && S && (i == length-1)))
                {
                        for (j = start; j <= i; j++)
                        {
                                mask[j] = 'U';
                                sprintf (midiMessage, "Chopping inconsistency. Found extra 'S' or 'T' at frame %d. Corrected. QCLOG", j);
                                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                        }
                }
        }

        //      Update tarType
        for (i = 0; i < length; i++)
                tarType[i] = mask[i];
                
        //      Display new tarType
        if (diagnostic > 4)
        {
                for (i = 0; i < length; i++)
                {
                        cpl_msg_info(cpl_func,"%4d  %c \n", i, tarType[i]);
                        fprintf (midiReportPtr, "%4d  %c \n", i, tarType[i]);
                }
        }

        //      Release memory
        free (mask);
        
        return CounterBad;
}       
/*****************************************************************************/

