
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  procDspTrn.c
* Description:  Contains routines for processing the templates
*
* History:      
* 16-Jun-04     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <math.h>
#include <stdio.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "imageProcessing.h"
#include "memoryHandling.h"
#include "createProdDspTrn.h"
#include "procDspTrn.h"
#include "errorHandling.h"
#include "midiFitsUtility.h"
#include "fitsAnalysisTec.h"
#include "diagnostics.h"
#include "qfits.h"
#include <cpl.h>

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  procDspTrn
* Input/Output: See function arguments to avoid duplication
* Description:  Measures the transmission characteristics of the dispersive 
*                elements (Prism and Grism) in order to monitor the evolution of 
*                their coatings. This is the main routine for this application.
*
* History:      
* 03-May-05     (csabet) Created
******************************************************************************/
void procDspTrn (
    MidiFiles    *fileNames,    // In: Pointer to file names
    int            *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "procDspTrn";
    ImageFormat        *format=NULL;
    DispersiveTrans    *trans=NULL;
    int                numOfFiles;
    FILE            *signaturePtr=NULL;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Write a signature
    signaturePtr = fopen ("MIDI_sig_trn.log", "w");
    fclose (signaturePtr);

    //    Reset status
    *error = 0;
    numOfFiles = 0;
    
    //    Allocate memory
    format = callocImageFormat ();
    
    analyseFitsDspTrn (fileNames, format, &numOfFiles, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot analyse DSPTRN");
        freeImageFormat (format);
        return;
    }
    
    trans = callocDspTrn (numOfFiles);
    computeDspTrn (numOfFiles, fileNames, trans, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot process DSPTRN");
        freeDspTrn (trans);
        freeImageFormat (format);
        return;
    }
    
    createDspTrnProd (fileNames, format, trans, error);
    if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create DSPTRN products");

    //    Release memory
    freeDspTrn (trans);
    freeImageFormat (format);

    return; 
}


/*----------------------------------------------------------------------------*/
/**
  @brief    Apply n x m median filter
  @param    pImage       image containing the beam, must be of
  @param    xwindow      window width, should be an odd number
  @param    ywindow      window height, should be an odd number
  @return   The median filtered image which must be deallocated using 
            cpl_image_delete()

 The function is a workaround for cpl_image_filter_median() 
 supporting kernels no larger than 7x7
 */
/*----------------------------------------------------------------------------*/
static cpl_image *
image_filter_median( const cpl_image *image, int xwindow, int ywindow )
{
    int nx = cpl_image_get_size_x(image);
    int ny = cpl_image_get_size_y(image);
    cpl_image *result = cpl_image_new( nx, ny, CPL_TYPE_FLOAT );
    int x, y;

    if (result == NULL)
    {
        return NULL;
    }

    for (y = 1; y <= ny; y++)
    for (x = 1; x <= nx; x++)
    {
        double median;
        int llx = x - (xwindow - 1) / 2;
        int lly = y - (ywindow - 1) / 2;
        int urx = x + (xwindow - 1) / 2;
        int ury = y + (ywindow - 1) / 2;
        if (llx <  1) llx = 1;
        if (lly <  1) lly = 1;
        if (urx > nx) urx = nx;
        if (ury > ny) ury = ny;
        median = cpl_image_get_median_window( image,
                          llx, lly,
                          urx, ury );

        cpl_image_set( result, x, y, median );
    }

    return result;
}

/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  computeDspTrn
* Input/Output: See function arguments to avoid duplication
* Description:  Measures the transmission characteristics of the dispersive 
*                elements (Prism and Grism) in order to monitor the evolution of 
*                their coatings.
*                   Observations are made by looking at a blackscreen (blackbody). 
*                6 measurements are made with one empty file at the end. 
*                Files 1,2,3 are Photometry (Imaging) with filters and files 4,5,6
*                are Spectroscopy (prism or grism) with filters (ArIII, NeII, SIV)
*                The above sequence could be repeated with different exposure time. 
*                Each observation sequence is analysed independently. Saturation and
*                   noise limits need to be verified to guarantee the validity of the results.
*
*                On the imaging observation, the position of the target (x1,y1,dx1,dy1) 
*                   and the photometry (integral number of counts, P1) is measured for
*                each filter. In addition the exposure time (T1), the maximum count 
*                   level (m1) are being recorded. The background is removed from a 
*                region around the star. 
*
*                  On the spectroscopy observations, the position y1 of the spectrum
*                and extension should be the same as in the imaging mode. The background
*                is removed from a region below and above the spectrum. A 1D fit
*                is performed along each column to subtract the background. 
*                  The signal is integrated over the full spectrum, yielding an integral 
*                  number of counts (P2), In addition the exposure time (t2), the maximum 
*                count level (m2) are being recorded. The photometry data are copied
*                from the corresponding intermediate QC product. 
*
*                Summary:
*                1. Find the coordinates of the target (x1, y1, dx1, dy1) 
*                2. Find the coordinates of the background (xb1, yb1, dxb1, dyb1) 
*                3. collapse each frame of files 1 to 6 containing the target with background removed
*                4. The above gives the "count" of sub-window containing the target
*                5. For each file get the integral count given by:
*                    P(file) = count / exposureTime
*                6. Dispersive Transmissions are given by
*                    alpha1 = P4/P1    alpha2 = P5/P2    alpha3 = P6/P3
*
* History:
* 18-Aug-04     (csabet) Created
******************************************************************************/
void computeDspTrn (
    int                numOfFiles,    // In:    Number of data files
    MidiFiles        *fileNames,    // In: Pointer to the MIDI file structure
    DispersiveTrans    *dspTrn,    // Ou: Pointer to the dispersive transmission data structure
    int                *error)        // Ou:    Error status
{
  
    //    Local Declarations
    //    ------------------
    const char  routine[] = "computeDspTrn";
    char                        *fileTemp, *classification;
    FILE                    *inFitsBatchPtr;
    ImageFormat             *format;
    int                     localError, fileNumber, extNumOfImagingDataFile;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    format = callocImageFormat ();

    //    Reset status
    *error = 0;
    localError = 0;
    fileNumber = 0;
    dspTrn->exists = 0;
    format->hasData = 0;
        
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot open input FITS file list\n                 No data preprocessing has been carried out for this batch");
        freeImageFormat (format);
        free (fileTemp);
        free (classification);
        *error = 1;
        return;
    }

    //    Loop through the files
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Processing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Processing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile  = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, &localError);
        if (localError) 
        {
            *error = 1;
            break;
        }

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, format, &localError);
            if (localError) 
            {
                *error = 1;
                break;
            }
        }
        else format->hasData = 0;

        //    Check if the file has data
        if (format->hasData)
        {
            //    Check Categ, Tech and Type and then compute the image size
            if ((strcmp (format->obsCatg, "CALIB") == 0) &&
                (strcmp (format->obsTech, "SPECTRUM") == 0) &&
                (strcmp (format->obsType, "WAVE") == 0))
            {
                //    Save beam combiner and sutter ID
                sprintf (dspTrn->beamCombiner[fileNumber], "%s", format->beamCombiner);
                sprintf (dspTrn->shutterId[fileNumber], "%s", format->shutterId);
                sprintf (dspTrn->cameraId[fileNumber], "%s", format->cameraId);
                sprintf (dspTrn->grismId[fileNumber], "%s", format->grismId);
                sprintf (dspTrn->filterName[fileNumber], "%s", format->filterName);
                if (diagnostic)cpl_msg_info(cpl_func,"   Beam combiner for file %d      = %s\n", fileNumber+1, dspTrn->beamCombiner[fileNumber]);
                if (diagnostic)cpl_msg_info(cpl_func,"   Shutter ID for file %d         = %s\n", fileNumber+1, dspTrn->shutterId[fileNumber]);
                if (diagnostic)cpl_msg_info(cpl_func,"   Camera ID for file %d          = %s\n", fileNumber+1, dspTrn->cameraId[fileNumber]);
                if (diagnostic)cpl_msg_info(cpl_func,"   Grism ID for file %d           = %s\n", fileNumber+1, dspTrn->grismId[fileNumber]);
                if (diagnostic)cpl_msg_info(cpl_func,"   Filter Name for file %d        = %s\n", fileNumber+1, dspTrn->filterName[fileNumber]);
                fprintf (midiReportPtr, "   Beam combiner for file %d      = %s\n", fileNumber+1, dspTrn->beamCombiner[fileNumber]);
                fprintf (midiReportPtr, "   Shutter ID for file %d         = %s\n", fileNumber+1, dspTrn->shutterId[fileNumber]);
                fprintf (midiReportPtr, "   Camera ID for file %d          = %s\n", fileNumber+1, dspTrn->cameraId[fileNumber]);
                fprintf (midiReportPtr, "   Grism ID for file %d           = %s\n", fileNumber+1, dspTrn->grismId[fileNumber]);
                fprintf (midiReportPtr, "   Filter Name for file %d        = %s\n", fileNumber+1, dspTrn->filterName[fileNumber]);

                //    Compute Integral Flux
                computeIntegralFlux (fileNumber, fileNames->inFitsName, extNumOfImagingDataFile, format, dspTrn, &localError);
                if (localError) *error = 1;
                
                //    Set the flags
                dspTrn->exists = 1;

                //    Increment file number but make sure it is not greater than the allocated
                fileNumber++;
                if (fileNumber > numOfFiles)
                {
                    *error = 1;
                    break;
                }                
            }
            else
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, "The above file is not suitable for this task");
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    //    Check if processing has been carried out
    if (dspTrn->exists && !(*error) && (fileNumber == numOfFiles))
    {
       cpl_msg_info(cpl_func,"\nDispersive Transmission Inventry: \n");
       cpl_msg_info(cpl_func,"================================ \n");
       cpl_msg_info(cpl_func,"   Expected number of data files  = %d\n", numOfFiles);
       cpl_msg_info(cpl_func,"   Number of data files processed = %d\n", fileNumber);
       cpl_msg_info(cpl_func,"\n");

        fprintf (midiReportPtr, "\nDispersive Transmission Inventry: \n");
        fprintf (midiReportPtr, "================================ \n");
        fprintf (midiReportPtr, "   Expected number of data files  = %d\n", numOfFiles);
        fprintf (midiReportPtr, "   Number of data files processed = %d\n", fileNumber);
        fprintf (midiReportPtr, "\n");

        //    Compute Dispersive Transmission
        //    -------------------------------
        computeDispersivity (dspTrn, &localError);
    }
    
    if (*error || localError)
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot access Dispersive Transmission for this batch");

    
    //    Close the file list
    fclose (inFitsBatchPtr);
    
    //    Release memory
    freeImageFormat (format);
    free (fileTemp);
    free (classification);    

    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  computeIntegralFlux
* Input/Output: See function arguments to avoid duplication
* Description:  Integrates number of counts over the full spectrum. That is obtains
*                one number for each frame and then one number for the file and divides
*                by the exposure time.
*
* History:      
* 18-Aug-04     (csabet) Created
******************************************************************************/
void computeIntegralFlux ( 
    int                fileNumber,        //    In:    File number
    char            *fileName,        //  In: Name of file to process
    int                extensionNumber,//    In:    Extension number of the IMAGE_DATA
    ImageFormat        *format,        //    In:    Pointer to the image format
    DispersiveTrans    *dspTrn,        //  Ou: Pointer to the dispersive transmission data structure
    int                *error)            //    Ou:    Error status

{
  
    //  Local Declarations
    //    ------------------
    const char  routine[] = "computeIntegralFlux";
    qfits_table *pTable  = NULL;
    short int   *inData;
    char        *tempStr, *string, *title, *dataName;
    float         numOfSubInteg, subIntegTime, targetRegionFlux;
    int         i, foundData = 0, scalingOffset, indexData, subWindowSize,
                targetPixelCount;
    float        *aveImage;
    MidiCoords    *target;

    cpl_image *pImage = NULL;    
    int nx,ny;
    cpl_image  *background = NULL;
/*     cpl_image  *temp       = NULL; */

    
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;
    subWindowSize = format->iXWidth * format->iYWidth;
        
    //  Open IMAGING_DATA = INDEX 2
    pTable = qfits_table_open (fileName, extensionNumber);
    if (!pTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load IMAGING_DATA");
        *error = 1;
        return;
    }
        
    //    Get data table information
    for (i = 0; i < pTable->nc; i++)
    {
        if (strcmp (pTable->col[i].tlabel, "DATA1") == 0)
        {
            foundData = 1;
            indexData = i;
        }
    }
    if (foundData == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find requested columns in data FITS file");
        qfits_table_close (pTable);
        *error = 1;
        return;
    }

    //  Read column DATA
    inData = (short int*) qfits_query_column (pTable, indexData, NULL); 

    //    Read Scaling Offset
    dataName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    for (i = 14; i < 25; i++)
    {
        sprintf (dataName, "TZERO%d", i);
        tempStr = qfits_query_ext (fileName, dataName, extensionNumber);
        if (tempStr != NULL)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"Scaling Offset = %s\n", tempStr);
            if (diagnostic) fprintf (midiReportPtr, "Scaling Offset = %s\n", tempStr);
            sscanf (tempStr, "%d", &scalingOffset);
            break;
        }
    }
    if (tempStr == NULL)
    {
        scalingOffset = 0;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Scaling Offset. It is set to 0");
    }
    free (dataName);

    //    Create averaged image for the whole file. This also allows a check on existence of a target
    aveImage = (float *) calloc (subWindowSize, sizeof(float));
    createAveragedImage (inData, scalingOffset,    format, aveImage);

        /*##############################################################################*/
        /* Subtract the background column by column */

        pImage = cpl_image_wrap_float(format->iXWidth, format->iYWidth, aveImage);  

        nx =format->iXWidth;
        ny=format->iYWidth;
        
        background  = image_filter_median( pImage, 1, 2*ny );
        cpl_image_subtract ( pImage, background );
        
        cpl_image_unwrap(pImage);
    
        cpl_image_delete(background);
        /*##############################################################################*/




    //    Create an image FITS file
    title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    string = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sprintf (title, "AveImg%d", fileNumber+1);
    sprintf (string, "file %d", fileNumber+1);
    createFitsImage (string, title, fileName, format->iXWidth, format->iYWidth, aveImage);
    free (string);
    free (title);

    //    Create plot file
    if (plotFile)
    {
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        string = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        sprintf (string, "3dAveImg%d", fileNumber+1);
        sprintf (title, "Averaged Image, filter %s", format->filterName);
        midiCreatePlotFile3D (string, title, "X", "Y", "Flux", 
            0, aveImage, format->iXWidth, format->iYWidth, "lines", "3");
        free (title);
        free (string);
    }
    
    //    Get Target coordinates and save the result
    target = (MidiCoords *) calloc (1, sizeof (MidiCoords));
    getDspTrnTargetCoords (fileNumber+1, dspTrn->grismId[fileNumber], aveImage, format, target, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot determine integral flux");
        qfits_table_close (pTable);
        free (inData);
        free (aveImage);
        return;
    }
    dspTrn->target[fileNumber].xCoord = target->xCoord;
    dspTrn->target[fileNumber].yCoord = target->yCoord;
    dspTrn->target[fileNumber].dxCoord = target->dxCoord;
    dspTrn->target[fileNumber].dyCoord = target->dyCoord;

   cpl_msg_info(cpl_func,"\nCoordinates and flux results in file %s: \n", fileName);
   cpl_msg_info(cpl_func,"==================================== \n");
   cpl_msg_info(cpl_func,"   Target coordinates                       = %f %f %f %f\n", target->xCoord, target->yCoord, 
        target->dxCoord, target->dyCoord);
    fprintf (midiReportPtr, "\nCoordinates and flux results in file %s: \n", fileName);
    fprintf (midiReportPtr, "==================================== \n");
    fprintf (midiReportPtr, "   Target coordinates                       = %f %f %f %f\n", target->xCoord, target->yCoord, 
        target->dxCoord, target->dyCoord);

    //    Compute total flux in the target region
    computeImageFlux (aveImage, format, target, &targetRegionFlux, &targetPixelCount, error);
   cpl_msg_info(cpl_func,"   Target flux with background              = %f\n", targetRegionFlux);
   cpl_msg_info(cpl_func,"   Target pixel count                       = %d\n", targetPixelCount);
    fprintf (midiReportPtr, "   Target flux with background              = %f\n", targetRegionFlux);
    fprintf (midiReportPtr, "   Target pixel count                       = %d\n", targetPixelCount);

    //    Compute mean target flux
    dspTrn->meanFlux[fileNumber] = targetRegionFlux / targetPixelCount;
   cpl_msg_info(cpl_func,"   Target mean flux with background         = %f\n", dspTrn->meanFlux[fileNumber]);
    fprintf (midiReportPtr, "   Target mean flux with background         = %f\n", dspTrn->meanFlux[fileNumber]);
    
    //    Get integration time
    tempStr = qfits_query_hdr (fileName, "HIERARCH ESO DET DIT");
    if (tempStr != NULL)
        sscanf(tempStr, "%f", &subIntegTime);
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Integration time");
        qfits_table_close (pTable);
        free (inData);
        *error = 1;
        free (aveImage);
        free (target);
        return;
    }
    tempStr = qfits_query_hdr (fileName, "HIERARCH ESO DET NDIT");
    if (tempStr != NULL)
        sscanf(tempStr, "%f", &numOfSubInteg);
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Number of Integrations");
        qfits_table_close (pTable);
        free (inData);
        *error = 1;
        free (target);
        free (aveImage);
        return;
    }
    //    Compute integral flux
    dspTrn->integTime[fileNumber] = numOfSubInteg * subIntegTime;
    dspTrn->integFlux[fileNumber] = dspTrn->meanFlux[fileNumber] / dspTrn->integTime[fileNumber];

   cpl_msg_info(cpl_func,"   Integration time                         = %f\n", dspTrn->integTime[fileNumber]);
    fprintf (midiReportPtr, "   Integration time                         = %f\n", dspTrn->integTime[fileNumber]);

   cpl_msg_info(cpl_func,"   Total integral flux                      = %f\n", dspTrn->integFlux[fileNumber]);
    fprintf (midiReportPtr, "   Total integral flux                      = %f\n", dspTrn->integFlux[fileNumber]);
    
    //    Release memory
    qfits_table_close (pTable);
    free (inData);
    free (aveImage);
    free (target);

    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  computeDispersivity
* Input/Output: See function arguments to avoid duplication
* Description:  
*
* History:      
* 18-Aug-04     (csabet) Created
******************************************************************************/
void computeDispersivity (
    DispersiveTrans    *dspTrn,    //  Ou: Pointer to the dispersive transmission data structure
    int                *error)        //    Ou:    Error status
{
  
    /*  Local Declarations
    --------------------*/
    const char  routine[] = "computeDispersivity";
    float        *dummy;
    
    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //     Check keywords to make sure that correct parameters are used from files
    if ((strcmp (dspTrn->filterName[0], dspTrn->filterName[3]) == 0) &&
        (strcmp (dspTrn->grismId[0], "OPEN") == 0) && 
        ((strcmp (dspTrn->grismId[3], "PRISM") == 0) || (strcmp (dspTrn->grismId[3], "GRISM") == 0)))
    {
        dspTrn->transmission[0] = dspTrn->integFlux[3] / dspTrn->integFlux[0];
       cpl_msg_info(cpl_func,"   Dispersive Transmission for files 4 / 1 = %f\n", dspTrn->transmission[0]);
        fprintf (midiReportPtr, "   Dispersive Transmission for files 4 / 1 = %f\n", dspTrn->transmission[0]);
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Files 1 and 4 are not compatible");
        *error = 1;
    }
        
    if ((strcmp (dspTrn->filterName[1], dspTrn->filterName[4]) == 0) &&
        (strcmp (dspTrn->grismId[1], "OPEN") == 0) && 
        ((strcmp (dspTrn->grismId[4], "PRISM") == 0) || (strcmp (dspTrn->grismId[4], "GRISM") == 0)))
    {
        dspTrn->transmission[1] = dspTrn->integFlux[4] / dspTrn->integFlux[1];
       cpl_msg_info(cpl_func,"   Dispersive Transmission for files 5 / 2 = %f\n", dspTrn->transmission[1]);
        fprintf (midiReportPtr, "   Dispersive Transmission for files 5 / 2 = %f\n", dspTrn->transmission[1]);
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Files 2 and 5 are not compatible");
        *error = 1;
    }

    if ((strcmp (dspTrn->filterName[2], dspTrn->filterName[5]) == 0) &&
        (strcmp (dspTrn->grismId[2], "OPEN") == 0) && 
        ((strcmp (dspTrn->grismId[5], "PRISM") == 0) || (strcmp (dspTrn->grismId[5], "GRISM") == 0)))
    {
        dspTrn->transmission[2] = dspTrn->integFlux[5] / dspTrn->integFlux[2];
       cpl_msg_info(cpl_func,"   Dispersive Transmission for files 6 / 3 = %f\n", dspTrn->transmission[2]);
        fprintf (midiReportPtr, "   Dispersive Transmission for files 6 / 3 = %f\n", dspTrn->transmission[2]);
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Files 3 and 6 are not compatible");
        *error = 1;
    }

    if (plotFile) 
    {
        dummy = (float *) calloc (3, sizeof (float));
        dummy[0] = 1.0;
        dummy[1] = 2.0;
        dummy[2] = 3.0;
        
        midiCreatePlotFile2D2P ("TransmissivityProfile", "Transmissivity Profile", 
            "Spectroscopy / Photometry: 4/1, 5/2, 6/3", "Transmissivity", 0, dummy, dspTrn->transmission, 0, 3, 1);
        free (dummy);
    }
    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getDspTrnTargetCoords
* Input/Output: See function arguments to avoid duplication
* Description:  This routine uses a 2D Gaussian fit algorithm to search for the target
*                The search area will be confined to a region around the global maximum.
*                In order to distiguish a true global maximum with a bad pixel a filter
*                must first be applied to the entire data set. The Gaussian fit will
*                give the size as well as the coordinates of the target. We choose the
*                size of the background region to be larger than the size of the target 
*                window
*
* History:      
* 18-Aug-04     (csabet) Created
******************************************************************************/
void getDspTrnTargetCoords (
    int            fileNumber,    // In: File number
    char        *grismId,    // In: Grism ID indicates Spectroscopy or Imaging
    float           *image,        // In: Pointer to the image data
    ImageFormat *format,    // In: Pointer to the image format
    MidiCoords    *target,    // Ou: Pointer to the target coordinate structures
    int            *error)        // Ou: Error status
{
  
    //  Local Declarations
    //    ------------------
    const char  routine[] = "getDspTrnTargetCoords";
    int            sizeSearch, dimension, xPinhole, yPinhole;
    double        xTarget, yTarget, sizeXTarget, sizeYTarget;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //    Set dimensionality for a suitable Gaussian fit
    if (strcmp (grismId, "OPEN") == 0) dimension = 2; // Imaging mode
    else if ((strcmp (grismId, "GRISM") == 0) || (strcmp (grismId, "PRISM") == 0)) dimension = 1; // Spectroscopy mode
    else
    {
        sprintf (midiMessage, "Unknown Grism ID %s: ", grismId);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
        return;
    }

    //    If all is well then use the appropriate parameters to find the coordinates of the target
    //    Override dimensionality. CPL Gaussian fit relies on a given search.
    dimension = 1;
    sizeSearch = SIZE_SEARCH_DSP_TRN;
    xPinhole = X_DSP_TRN;
    yPinhole = Y_DSP_TRN;
    midiGaussianFit (fileNumber, dimension, image, format->iXWidth, format->iYWidth, xPinhole, yPinhole, 
        sizeSearch, &xTarget, &yTarget, &sizeXTarget, &sizeYTarget, error);

    //    Save output results
    if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot determine target coordinates");
    else
    {
           /* Fixed aperture used after background subtraction */
           target->xCoord  = xTarget - 7.;
           target->yCoord  = yTarget - 7.;
           target->dxCoord = xTarget + 7.;
           target->dyCoord = yTarget + 7.;

           /*
           target->xCoord = xTarget - 0.5 * sizeXTarget;
           target->yCoord = yTarget - 0.5 * sizeYTarget;
           target->dxCoord = xTarget + 0.5 * sizeXTarget;
           target->dyCoord = yTarget + 0.5 * sizeYTarget;
           */
    }

    return; 
}
//*****************************************************************************

