/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  procFrgHS.c
* Description:  Contains routines for Fringe Track data post processing
*
* History:
* 03-Feb-05     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <cpl.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "memoryHandling.h"
#include "calibration.h"
#include "errorHandling.h"
#include "midiFitsUtility.h"
#include "fitsAnalysisFrgHS.h"
#include "imageProcessing.h"
#include "createProdFrg.h"
#include "createProdFrgUndisp.h"
#include "transferFunction.h"
#include "visibility.h"
#include "diagnostics.h"
#include "estimationFrg.h"
#include "estimationFrgHSUndisp.h"
#include "photometry.h"
#include "procFrgHS.h"
#include "preProcFrgHS.h"
#include "qfits.h"

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables
**********************************************************/

/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  procFrgHS
* Input/Output: See function arguments to avoid duplication
* Description:    This is the main routine for HIGH_SEND fringe track
*                The memory allocation needs the following considerations:
*                
*                Split Interferometry files require     "compressedInterf"    & "formatInterf"
*                One Photometry A file requires        "compressedPhotomA"    & "formatPhotomA"
*                One Photometry B file requires        "compressedPhotomB"    & "formatPhotomB"
*    
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void procFrgHS (
   MidiFiles          *fileNames,    // In: Pointer to midi files structure
   UserOptions        *options,    // In: Pointer to user options
   int                *error,        // Ou: Error status
   cpl_parameterlist  *parlist,
   cpl_frameset       *frameset)
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "procFrgHS";
    ImageFormat        *formatInterf=NULL;    // Applies to interf files
    ImageFormat        *formatPhotomA=NULL;// Applies to photom A files
    ImageFormat        *formatPhotomB=NULL;// Applies to photom B files
    CompressedData    *compressedInterf=NULL, *compressedPhotomA=NULL, *compressedPhotomB=NULL;
    FilterData        *filterInfo=NULL;
    FILE            *signaturePtr=NULL;
    DispersedResult    *dispResult=NULL;
    CorrectedFrames		*corrFrames=NULL;
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Write a signature. Midas checks to see which template is running
    if (strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0)
        signaturePtr = fopen ("MIDI_sig_hsc.log", "w");
    else if (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0)
        signaturePtr = fopen ("MIDI_sig_hss.log", "w");
    else
    {
        sprintf (midiMessage, "Invalid template ... %s", batchTemplate);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
    fclose (signaturePtr);

    //    Reset status
    *error = 0;

    //    Analyse Input files
    //    -------------------
    //    Allocate memory
    formatInterf = callocImageFormat ();
    formatPhotomA = callocImageFormat ();
    formatPhotomB = callocImageFormat ();
    corrFrames = cpl_calloc (1, sizeof (CorrectedFrames));

    corrFrames->CorrInterf=0;
    corrFrames->CorrPhotomA=0;
    corrFrames->CorrPhotomB=0;

    analyseFitsFrgHS (fileNames, formatInterf, formatPhotomA, formatPhotomB, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot analyse FITS files");
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        return;
    }

    //    Reduce Data
    //    -----------
    //  Allocate memory for the compressed data, filter data and initialise to zero
    compressedInterf = callocMidiCompressed (formatInterf);
    compressedPhotomA = callocMidiCompressed (formatPhotomA);
    compressedPhotomB = callocMidiCompressed (formatPhotomB);
    badChannelList = (int *) calloc (formatInterf->iXWidth, sizeof (int)); 
    filterInfo = callocFilterInfo();
    
    //    Preprocessing
    preProcFrgHS (options, filterInfo, fileNames, compressedInterf, compressedPhotomA, 
        compressedPhotomB, formatInterf, formatPhotomA, formatPhotomB, error);
    
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compress FITS files");
        freeCompressedData (formatInterf, compressedInterf);
        freeCompressedData (formatPhotomA, compressedPhotomA);
        freeCompressedData (formatPhotomB, compressedPhotomB);
        free (badChannelList);
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        freeFilterInfo (filterInfo);
        return;
    }

    //    Check Delay Lines for jumps within scans
    checkDelayLineJumps ("INTERF", formatInterf, compressedInterf);

    //    Check Delay Line Consistencies
    checkDelayLineConsistencies ("INTERF", "PHOTOMA", compressedInterf, compressedPhotomA, formatInterf, formatPhotomA, error);
    checkDelayLineConsistencies ("INTERF", "PHOTOMB", compressedInterf, compressedPhotomB, formatInterf, formatPhotomB, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "HIGH_SENSE processing abandonned");
        freeCompressedData (formatInterf, compressedInterf);
        freeCompressedData (formatPhotomA, compressedPhotomA);
        freeCompressedData (formatPhotomB, compressedPhotomB);
        free (badChannelList);
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        freeFilterInfo (filterInfo);
        return;
    }

    //    Check Chopping Statistics. for QCLOG diagnostics
    //    ------------------------------------------------
    reportInterfChopping ("INTERF", formatInterf, compressedInterf);
    reportPhotomChopping ("PHOTOMA", formatPhotomA, compressedPhotomA);
    reportPhotomChopping ("PHOTOMB", formatPhotomB, compressedPhotomB);
    if (options->processing == DISPERSED)
    {
        displayInterfChoppingDisp ("INTERF", formatInterf, compressedInterf);
        displayPhotomChoppingDisp ("PHOTOMA", formatPhotomA, compressedPhotomA);
        displayPhotomChoppingDisp ("PHOTOMB", formatPhotomB, compressedPhotomB);
    }
    
    //    Remove Sky Background and display Target with Sky removed
    dispResult = callocDispResult (formatInterf->iXWidth);
    cleanupFrgHS (options->processing, formatInterf, formatPhotomA, formatPhotomB, compressedInterf,
        compressedPhotomA, compressedPhotomB, dispResult, error, corrFrames);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "HIGH_SENSE processing abandonned");
        freeCompressedData (formatInterf, compressedInterf);
        freeCompressedData (formatPhotomA, compressedPhotomA);
        freeCompressedData (formatPhotomB, compressedPhotomB);
        free (badChannelList);
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        freeFilterInfo (filterInfo);
        freeDispResult (dispResult);
        return;
    }
    //    Display Waterfall data
    prepareWaterfallDisplay (fileNames, formatInterf, compressedInterf);

    //    Frequency Calibration, if it is a CALIB batch
    //    ---------------------------------------------
    if (strcmp (formatInterf->obsCatg, "CALIB") == 0)
        calibrateWaveUsingPolynom (fileNames, formatInterf);
            
    //    Create UNDISPERSED allSpectrum and dcLevels
    //    -------------------------------------------
    getUndispersedPowerSpectrum (formatInterf, compressedInterf);
    
    //    Display Waterpower data
    prepareWaterpowerDisplay (fileNames, formatInterf, compressedInterf->allSpectrum);

    //    Select processing mode
    //    ----------------------
    if (options->processing == DISPERSED)
    {
        postProcFringe (fileNames, filterInfo, compressedInterf,
        		compressedPhotomA, compressedPhotomB, formatInterf,
        		formatPhotomA, formatPhotomB, dispResult, error,
        		parlist, frameset, corrFrames);
        if (*error)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "HIGH_SENSE processing abandonned");
            freeCompressedData (formatInterf, compressedInterf);
            freeCompressedData (formatPhotomA, compressedPhotomA);
            freeCompressedData (formatPhotomB, compressedPhotomB);
            free (badChannelList);
            freeImageFormat (formatInterf);
            freeImageFormat (formatPhotomA);
            freeImageFormat (formatPhotomB);
            freeFilterInfo (filterInfo);
            freeDispResult (dispResult);
            return;
        }
    }
    else if (options->processing == UNDISPERSED)
    {
       postProcFrgHSUndisp (fileNames, compressedInterf, compressedPhotomA, 
                            compressedPhotomB, filterInfo, formatInterf, formatPhotomA, formatPhotomB, error, parlist, frameset);
       if (*error)
       {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "HIGH_SENSE processing abandonned");
            freeCompressedData (formatInterf, compressedInterf);
            freeCompressedData (formatPhotomA, compressedPhotomA);
            freeCompressedData (formatPhotomB, compressedPhotomB);
            free (badChannelList);
            freeImageFormat (formatInterf);
            freeImageFormat (formatPhotomA);
            freeImageFormat (formatPhotomB);
            freeFilterInfo (filterInfo);
            freeDispResult (dispResult);
            return;
        }
    }
    else
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown processing mode requested");
        freeCompressedData (formatInterf, compressedInterf);
        freeCompressedData (formatPhotomA, compressedPhotomA);
        freeCompressedData (formatPhotomB, compressedPhotomB);
        free (badChannelList);
        freeImageFormat (formatInterf);
        freeImageFormat (formatPhotomA);
        freeImageFormat (formatPhotomB);
        freeFilterInfo (filterInfo);
        freeDispResult (dispResult);
        return;
    }

    //    Release memory used in this batch
    freeCompressedData (formatInterf, compressedInterf);
    freeCompressedData (formatPhotomA, compressedPhotomA);
    freeCompressedData (formatPhotomB, compressedPhotomB);
    free (badChannelList);
    freeImageFormat (formatInterf);
    freeImageFormat (formatPhotomA);
    freeImageFormat (formatPhotomB);
    freeFilterInfo (filterInfo);
    freeDispResult (dispResult);
    cpl_free(corrFrames);
    return;
}



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  postProcFringe
* Input/Output: See function arguments to avoid duplication
* Description:  Performs complete post processing for the DISPERSED mode
*
* History:
* 03-Feb-05     (csabet) Created
******************************************************************************/
void postProcFringe (
   MidiFiles         *fileNames,            // In: Pointer to file names
   FilterData        *filterInfo,        // In: Filter data
   CompressedData    *compressedInterf,    // In: Compressed interferometry data
   CompressedData    *compressedPhotomA,    // In: Photometry data
   CompressedData    *compressedPhotomB,    // In: Photometry data
   ImageFormat       *formatInterf,        // In: Image size of interferometry data
   ImageFormat       *formatPhotomA,        // In: Applies to A photom files
   ImageFormat       *formatPhotomB,        // In: Applies to B photom files
   DispersedResult   *dispResult,        // IO: Output results
   int               *error,                // Ou: Status
   cpl_parameterlist *parlist,
   cpl_frameset      *frameset,
   CorrectedFrames   *corrFrames)
{

    //  Local Declarations
    //    ------------------
    const char        routine[] = "postProcFringe";
    CalibratorParam    *calibrator=NULL;
    float            *freqCal=NULL;
    IauExchange      *iauData=NULL;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nProcessing data in DISPERSED mode for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------------- \n");
    fprintf (midiReportPtr, "\nProcessing data in DISPERSED mode for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------------------- \n");

    //    Allocate memory
    formatInterf->fftsize = POST_FFT_SIZE;
    calibrator = callocCalibrator ();
    freqCal = (float *) calloc (formatInterf->iXWidth, sizeof (float));
    
    //  Reset status
    *error = 0;
    dispResult->photomAExists = 0;
    dispResult->photomBExists = 0;
    dispResult->photomIExists = 0;
    dispResult->calibVisExists = 0;
    dispResult->trfExists = 0;

    //    Estimate fringe
    //    ---------------
    estimateFringe (fileNames, filterInfo, compressedInterf, formatInterf, freqCal, 
        dispResult->rawVis2, dispResult->rawVis2Err, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compute Raw Visibilities");
        return;
    }


   cpl_msg_info(cpl_func,"\nRunning Photometry on photom A data: ");
    fprintf (midiReportPtr, "\nRunning Photometry on photom A data: ");
    estimatePhotom ('A', compressedPhotomA, formatPhotomA, dispResult->photomA, dispResult->photomAErr, error);
    if (*error)
    {
       cpl_msg_info(cpl_func,"%s \n", UNAV);
        fprintf (midiReportPtr, "%s \n", UNAV);
    }
    else
    {
        dispResult->photomAExists = 1;
       cpl_msg_info(cpl_func,"\n");
        fprintf (midiReportPtr, "\n");
    } 

   cpl_msg_info(cpl_func,"Running Photometry on Photom B data: ");
    fprintf (midiReportPtr, "Running Photometry on Photom B data: ");
    estimatePhotom ('B', compressedPhotomB, formatPhotomB, dispResult->photomB, dispResult->photomBErr, error);
    if (*error)
    {
       cpl_msg_info(cpl_func,"%s \n", UNAV);
        fprintf (midiReportPtr, "%s \n", UNAV);
    }
    else
    {
        dispResult->photomBExists = 1;
       cpl_msg_info(cpl_func,"\n");
        fprintf (midiReportPtr, "\n");
    } 

   cpl_msg_info(cpl_func,"Running Photometry on Interf data: ");
    fprintf (midiReportPtr, "Running Photometry on Interf data: ");
    estimatePhotom ('I', compressedInterf, formatInterf, dispResult->photomI, dispResult->photomIErr, error);
    if (*error)
    {
       cpl_msg_info(cpl_func,"%s \n", UNAV);
        fprintf (midiReportPtr, "%s \n", UNAV);
    }
    else
    {
        dispResult->photomIExists = 1;
       cpl_msg_info(cpl_func,"\n");
        fprintf (midiReportPtr, "\n");
    } 
        
    //    Compute DISPERSED Normalized but Uncalibrated Visibility
    //    --------------------------------------------------------
    computeUncalibVis (formatInterf, freqCal, dispResult, error);
    if (*error)
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compute Uncalibrated Visibility for this batch");

    //    Compute Transfer Function if observation category is CALIB
    //    ----------------------------------------------------------
    if (strcmp (formatInterf->obsCatg, "CALIB") == 0)
    {
       estimateTransferFunction (formatInterf, fileNames, dispResult, calibrator, error, parlist, frameset);
        if (*error)
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compute Transfer Function for this batch");
    }

    //    Compute Calibrated Visibility if observation category is SCIENCE
    //    ----------------------------------------------------------------
    if (strcmp (formatInterf->obsCatg, "SCIENCE") == 0)
    {
       computeCalibVis (fileNames, formatInterf, dispResult, error, parlist, frameset);
        if (*error)
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot compute Calibrated Visibility for this batch");
    }

    //    Report results
    if (diagnostic) midiReportResultsFringe (freqCal, formatInterf, dispResult);
    reportRejectList (formatInterf, compressedInterf);
    
    //    Create the product files

    iauData=createFrgProd (fileNames, formatInterf, freqCal, dispResult, calibrator,
    		error, corrFrames);

    /*Save the correlated flux*/
    if(iauData != NULL){
    	midiSaveCorrelatedFlux(iauData, dispResult, parlist, frameset);
    }
    if (*error)
    	midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create product files");

    /*  Release memory */
    freeCalibrator (calibrator);
    free (freqCal);
    if(iauData != NULL){
    	freeIauExchange (iauData);
    }
    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  postProcFrgHSUndisp
* Input/Output: See function arguments to avoid duplication
* Description:  Performs complete post processing for the UNDISPERSED mode
*
* History:
* 03-Feb-05     (csabet) Created
******************************************************************************/
void postProcFrgHSUndisp (
   MidiFiles        *fileNames,            // In: Pointer to file names
   CompressedData    *compressedInterf,    // In: Compressed interferometry data
   CompressedData    *compressedPhotomA,    // In: Compressed photometry data
   CompressedData    *compressedPhotomB,    // In: Compressed photometry data
   FilterData        *filterInfo,        // In: Filter information
   ImageFormat       *formatInterf,
   ImageFormat       *formatPhotomA,
   ImageFormat       *formatPhotomB,
   int               *error,                // Ou: Status
   cpl_parameterlist *parlist,
   cpl_frameset      *frameset)
{
   
    //     Local Declarations
    //    ------------------
    const char                routine[] = "postProcFrgHSUndisp";
    RawVisibility            *rawVis=NULL;
    PhotometryResult        *photometry=NULL;
    CalibratedVisibility    *calibVis=NULL;
    CalibratorParam            *calibrator=NULL;
    TransferFunction        *trf=NULL;


    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nProcessing HIGH_SENS data in UNDISPERSED mode for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------------------------- \n");
    fprintf (midiReportPtr, "\nProcessing HIGH_SENS data in UNDISPERSED mode for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------------------------------- \n");

    //    Reset status
    *error = 0;
    
    //    Finalise reject list using Signal / Noise
    getBadScansFromSpectrumUndisp (filterInfo, formatInterf, compressedInterf, error);
    if (*error) return;
    
    //    Compute UNDISPERSED fringe
    //    --------------------------
    rawVis = callocRawVis ();
    photometry = callocPhotometry ();
    estimateFrgHSUndisp (compressedInterf, compressedPhotomA, compressedPhotomB, filterInfo, 
        formatInterf, formatPhotomA, formatPhotomB, rawVis, photometry);
 
     //    Compute UNDISPERSED Uncalibrated Visibility
    //    -------------------------------------------
    computeUncalibVisHSUndisp (rawVis, photometry);

    //    Compute Transfer Function if observation category is CALIB
    //    ----------------------------------------------------------
    calibrator = callocCalibrator ();
    trf = callocTrFunction (6);
    if (strcmp (formatInterf->obsCatg, "CALIB") == 0)
    {
       estimateTransferFunctionUndisp (fileNames, rawVis, trf, calibrator, error,parlist, frameset);
       if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                                         "Cannot compute Transfer Function for this batch");
    }

    //    Compute Calibrated Visibility if observation category is SCIENCE
    //    ----------------------------------------------------------------
    calibVis = callocCalibVis ();
    if (strcmp (formatInterf->obsCatg, "SCIENCE") == 0)
    {
        computeCalibVisHSUndisp (fileNames, rawVis, trf, calibVis, error);
        if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot compute Calibrated Visibility for this batch");
    }
        
    //    Create the product files
    //    ------------------------
    createFrgProdUndisp (fileNames, formatInterf, filterInfo, rawVis, trf, 
        calibrator, calibVis, photometry, error);
    if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create product files");

    //    Release memory
    freeRawVis (rawVis);
    freePhotometry (photometry);
    freeTrFunction (trf);
    freeCalibrator (calibrator);
    freeCalibVis (calibVis);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  cleanupFrgHS
* Input/Output: See function arguments to avoid duplication
* Description:  Removes sky etc from the data
*
* History:
* 06-Dec-05     (csabet) Created
******************************************************************************/
void cleanupFrgHS (
    enum ProcessingMode    processing,
    ImageFormat            *formatInterf,        // In: Interf size parameters
    ImageFormat            *formatPhotomA,        // In: PhotA size parameters
    ImageFormat            *formatPhotomB,        // In: PhotB size parameters
    CompressedData        *compressedInterf,    // Ou: Pointer to the compressed interferometry data structure
    CompressedData        *compressedPhotomA,    // Ou: Pointer to the compressed photom A data structure
    CompressedData        *compressedPhotomB,    // Ou: Pointer to the compressed photom B data structure
    DispersedResult        *dispResult,        // Ou: Output results
    int                    *error,
    CorrectedFrames        *corrFrames)                // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char    routine[] = "cleanupFrgHS";
    int            F, s, t, u, n, f, R, X, transitions, localError;
    double        *array, fluxAve;
    float        *arrayF;
    char        *fileName, *title;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCleaning up HIGH_SENS files for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------- \n");
    fprintf (midiReportPtr, "\nCleaning up HIGH_SENS files for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------------- \n");

    //    Initialise
    localError = 0;
    *error = 0;
    transitions = 0;
        
    //    Check Interf chopping cycle
    //    ---------------------------
    s = t = u = 0;
    for (F = 0; F < formatInterf->numOfFrames; F++)
    {
        if (compressedInterf->tarType[F] == 'S') s++;
        else if (compressedInterf->tarType[F] == 'T') t++;
        else u++;
    }
    //    Report
    if (u || s || (t != formatInterf->numOfFrames))
    {
        sprintf (midiMessage, "Interferometry data is chopped"); 
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
    }
    transitions = countTransitions ('T', formatInterf->numOfFrames, compressedInterf->tarType);
    sprintf (midiMessage, "Target Transitions for Interf   = %d", transitions); 
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    formatInterf->numOfSkyScans = 0;

    //    Check PhotomA chopping cycle
    //    ----------------------------    
    transitions = countTransitions ('T', formatPhotomA->numOfFrames, compressedPhotomA->tarType);
    sprintf (midiMessage, "Target Transitions for Photom A = %d", transitions); 
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    if (transitions < 10)
    {
        sprintf (midiMessage, "Photometry A data is not chopped"); 
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
    }

    //    Check PhotomB chopping cycle
    //    ----------------------------
    transitions = countTransitions ('T', formatPhotomB->numOfFrames, compressedPhotomB->tarType);
    sprintf (midiMessage, "Target Transitions for Photom B = %d", transitions); 
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    if (transitions < 10)
    {
        sprintf (midiMessage, "Photometry B data is not chopped"); 
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
    }

    //    if No error so far
    if (!(*error))
    {
        formatInterf->chopped = 0;
        formatPhotomA->chopped = 1;
        formatPhotomB->chopped = 1;
    }
    else 
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot cleanup HIGH_SENS files");
        *error = 1;
        return;
    }

    //    Correct benign tarType inconsistencies
    //    --------------------------------------
    corrFrames->CorrInterf=0;
    corrFrames->CorrPhotomA = correctTarType ("PHOTOMA",
    		compressedPhotomA->tarType, compressedPhotomA->time,
    		formatPhotomA->numOfFrames, &localError);
    if (localError) *error = 1;

    corrFrames->CorrPhotomB = correctTarType ("PHOTOMB",
    		compressedPhotomB->tarType, compressedPhotomB->time,
    		formatPhotomB->numOfFrames, &localError);
    if (*error || localError)
    {
        *error = 1;
        return;
    }

    //    If all is well, carry out cleaning
    //    ----------------------------------
    //    First compute binned Target 'T' photometries with background
    if (processing == DISPERSED && (strcmp(formatPhotomA->beamCombiner, "HIGH_SENS") == 0))
    {
        computeBinnedPhotom (compressedPhotomA,    compressedPhotomB, formatPhotomA,
            &(dispResult->photomATotal1), &(dispResult->photomATotal2), &(dispResult->photomATotal3),
            &(dispResult->photomBTotal1), &(dispResult->photomBTotal2), &(dispResult->photomBTotal3), error);

        //    Report
       cpl_msg_info(cpl_func,"\nBinned photometric measurements with background \n");
       cpl_msg_info(cpl_func,"----------------------------------------------- \n");
       cpl_msg_info(cpl_func,"Grism ID: %s \n", formatPhotomA->grismId);
        fprintf(midiReportPtr, "\nBinned photometric measurements with background QCLOG \n");
        fprintf(midiReportPtr, "----------------------------------------------- QCLOG \n");
        fprintf(midiReportPtr, "Grism ID: %s QCLOG \n", formatPhotomA->grismId);
        if (strcmp(formatPhotomA->grismId, "PRISM") == 0)
        {
           cpl_msg_info(cpl_func,"ArIII (Channel 113 -> 121) Beam A = %0.2f  Beam B = %0.2f \n", 
                dispResult->photomATotal1, dispResult->photomBTotal1);
           cpl_msg_info(cpl_func,"  SIV (Channel  88 ->  98) Beam A = %0.2f  Beam B = %0.2f \n", 
                dispResult->photomATotal2, dispResult->photomBTotal2);
           cpl_msg_info(cpl_func," NeII (Channel  36 ->  50) Beam A = %0.2f  Beam B = %0.2f \n\n", 
                dispResult->photomATotal3, dispResult->photomBTotal3);

            fprintf(midiReportPtr, "ArIII (Channel 113 -> 121) Beam A = %0.2f  Beam B = %0.2f QCLOG \n", 
                dispResult->photomATotal1, dispResult->photomBTotal1);
            fprintf(midiReportPtr, "  SIV (Channel  88 ->  98) Beam A = %0.2f  Beam B = %0.2f QCLOG \n", 
                dispResult->photomATotal2, dispResult->photomBTotal2);
            fprintf(midiReportPtr, " NeII (Channel  36 ->  50) Beam A = %0.2f  Beam B = %0.2f QCLOG \n\n", 
                dispResult->photomATotal3, dispResult->photomBTotal3);
        }
        else
        {
           cpl_msg_info(cpl_func,"ArIII (Channel  52 ->  68) Beam A = %0.2f  Beam B = %0.2f \n", 
                dispResult->photomATotal1, dispResult->photomBTotal1);
           cpl_msg_info(cpl_func,"  SIV (Channel 116 -> 136) Beam A = %0.2f  Beam B = %0.2f \n", 
                dispResult->photomATotal2, dispResult->photomBTotal2);
           cpl_msg_info(cpl_func," NeII (Channel 217 -> 244) Beam A = %0.2f  Beam B = %0.2f \n\n", 
                dispResult->photomATotal3, dispResult->photomBTotal3);

            fprintf(midiReportPtr, "ArIII (Channel  52 ->  68) Beam A = %0.2f  Beam B = %0.2f QCLOG \n", 
                dispResult->photomATotal1, dispResult->photomBTotal1);
            fprintf(midiReportPtr, "  SIV (Channel 116 -> 136) Beam A = %0.2f  Beam B = %0.2f QCLOG \n", 
                dispResult->photomATotal2, dispResult->photomBTotal2);
            fprintf(midiReportPtr, " NeII (Channel 217 -> 244) Beam A = %0.2f  Beam B = %0.2f QCLOG \n\n", 
                dispResult->photomATotal3, dispResult->photomBTotal3);
        }
    }
    
    removeSkyBackground ("PHOTOMA", processing, formatPhotomA, compressedPhotomA, &localError);
    if (localError) *error = 1;
    removeSkyBackground ("PHOTOMB", processing, formatPhotomB, compressedPhotomB, &localError);
    if (formatPhotomA->numOfFrames != formatPhotomA->numOfFrames || *error || localError)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unequal numOfFrames");
        *error = 1;
        return;
    }

    //    Now compute binned Target 'T' photometries with background removed
    if (processing == DISPERSED && (strcmp(formatPhotomB->beamCombiner, "HIGH_SENS") == 0))
    {
        computeBinnedPhotom (compressedPhotomA,    compressedPhotomB, formatPhotomA,
            &(dispResult->photomATarg1), &(dispResult->photomATarg2), &(dispResult->photomATarg3),
            &(dispResult->photomBTarg1), &(dispResult->photomBTarg2), &(dispResult->photomBTarg3), error);

        //    Report
       cpl_msg_info(cpl_func,"\nBinned photometric measurements without background \n");
       cpl_msg_info(cpl_func,"-------------------------------------------------- \n");
       cpl_msg_info(cpl_func,"Grism ID: %s \n", formatPhotomA->grismId);
        fprintf(midiReportPtr, "\nBinned photometric measurements without background QCLOG \n");
        fprintf(midiReportPtr, "-------------------------------------------------- QCLOG \n");
        fprintf(midiReportPtr, "Grism ID: %s QCLOG \n", formatPhotomA->grismId);
        if (strcmp(formatPhotomA->grismId, "PRISM") == 0)
        {
           cpl_msg_info(cpl_func,"ArIII (Channel 113 -> 121) Beam A = %0.2f  Beam B = %0.2f \n", 
                dispResult->photomATarg1, dispResult->photomBTarg1);
           cpl_msg_info(cpl_func,"  SIV (Channel  88 ->  98) Beam A = %0.2f  Beam B = %0.2f \n", 
                dispResult->photomATarg2, dispResult->photomBTarg2);
           cpl_msg_info(cpl_func," NeII (Channel  36 ->  50) Beam A = %0.2f  Beam B = %0.2f \n\n", 
                dispResult->photomATarg3, dispResult->photomBTarg3);

            fprintf(midiReportPtr, "ArIII (Channel 113 -> 121) Beam A = %0.2f  Beam B = %0.2f QCLOG \n", 
                dispResult->photomATarg1, dispResult->photomBTarg1);
            fprintf(midiReportPtr, "  SIV (Channel  88 ->  98) Beam A = %0.2f  Beam B = %0.2f QCLOG \n", 
                dispResult->photomATarg2, dispResult->photomBTarg2);
            fprintf(midiReportPtr, " NeII (Channel  36 ->  50) Beam A = %0.2f  Beam B = %0.2f QCLOG \n\n", 
                dispResult->photomATarg3, dispResult->photomBTarg3);
        }
        else
        {
           cpl_msg_info(cpl_func,"ArIII (Channel  52 ->  68) Beam A = %0.2f  Beam B = %0.2f \n", 
                dispResult->photomATarg1, dispResult->photomBTarg1);
           cpl_msg_info(cpl_func,"  SIV (Channel 116 -> 136) Beam A = %0.2f  Beam B = %0.2f \n", 
                dispResult->photomATarg2, dispResult->photomBTarg2);
           cpl_msg_info(cpl_func," NeII (Channel 217 -> 244) Beam A = %0.2f  Beam B = %0.2f \n\n", 
                dispResult->photomATarg3, dispResult->photomBTarg3);

            fprintf(midiReportPtr, "ArIII (Channel  52 ->  68) Beam A = %0.2f  Beam B = %0.2f QCLOG \n", 
                dispResult->photomATarg1, dispResult->photomBTarg1);
            fprintf(midiReportPtr, "  SIV (Channel 116 -> 136) Beam A = %0.2f  Beam B = %0.2f QCLOG \n", 
                dispResult->photomATarg2, dispResult->photomBTarg2);
            fprintf(midiReportPtr, " NeII (Channel 217 -> 244) Beam A = %0.2f  Beam B = %0.2f QCLOG \n\n", 
                dispResult->photomATarg3, dispResult->photomBTarg3);
        }
    }
    
    //    Reject frames whose flux are <= 0 or are not 'T'
    for (R = 0; R < formatInterf->numOfRegionsToProcess; R++)
    {
        for (X = 0; X < formatInterf->iXWidth; X++)
        {
            if (badChannelList[X])
                continue;
                
            for (f = 0; f < formatInterf->numOfFrames; f++)
            {
                if ((compressedInterf->tarType[f] == 'T') && 
                    (((compressedInterf->iDispFringe)[R])[X])[f] <= 0.0)
                {
                    compressedInterf->rejectList[X][f] |= BSL_DATA_ERROR;
                    if (diagnostic > 3) 
                    {
                        sprintf (midiMessage, "Negative Flux at: R %d    X %3d    F %4d          Interf %10.2f", 
                            R, X, f, (((compressedInterf->iDispFringe)[R])[X])[f]);
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    }
                }
                //    Reject frames which are not 'T'
                if (compressedInterf->tarType[f] != 'T')
                    compressedInterf->rejectList[X][f] |= BSL_SKY_SCAN;
            }
        }
    }
 
    for (R = 0; R < formatPhotomA->numOfRegionsToProcess; R++)
    {
        for (X = 0; X < formatPhotomA->iXWidth; X++)
        {
            if (badChannelList[X])
                continue;
                    
            for (f = 0; f < formatPhotomA->numOfFrames; f++)
            {
                if ((compressedPhotomA->tarType[f] == 'T') && 
                    (((compressedPhotomA->iDispFringe)[R])[X])[f] <= 0.0)
                {
                    compressedPhotomA->rejectList[X][f] |= BSL_DATA_ERROR;
                    if (diagnostic > 3) 
                    {
                        sprintf (midiMessage, "Negative Flux at: R %d    X %3d    F %4d         PhotomA %10.2f", 
                            R, X, f, (((compressedPhotomA->iDispFringe)[R])[X])[f]);
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    }
                }
                //    Reject frames which are not 'T'
                if (compressedPhotomA->tarType[f] != 'T')
                    compressedPhotomA->rejectList[X][f] |= BSL_SKY_SCAN;
            }
        }
    }
 
    for (R = 0; R < formatPhotomB->numOfRegionsToProcess; R++)
    {
        for (X = 0; X < formatPhotomB->iXWidth; X++)
        {
            if (badChannelList[X])
                continue;
                
            for (f = 0; f < formatPhotomB->numOfFrames; f++)
            {
                if ((compressedPhotomB->tarType[f] == 'T') && 
                    (((compressedPhotomB->iDispFringe)[R])[X])[f] <= 0.0)
                {
                    compressedPhotomB->rejectList[X][f] |= BSL_DATA_ERROR;
                    if (diagnostic > 3) 
                    {
                        sprintf (midiMessage, "Negative Flux at: R %d    X %3d    F %4d         PhotomB %10.2f", 
                            R, X, f, (((compressedPhotomB->iDispFringe)[R])[X])[f]);
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    }
                }
                //    Reject frames which are not 'T'
                if (compressedPhotomB->tarType[f] != 'T')
                    compressedPhotomB->rejectList[X][f] |= BSL_SKY_SCAN;
            }
        }
    }
    
    //    Display data with background removed
    if (plotFile)
    {
        array = (double *) calloc (formatInterf->numOfFrames, sizeof (double));
 
        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatInterf->numOfFrames; f++)
        {
            if (compressedInterf->tarType[f] == 'T')
            {
                array[n] = compressedInterf->iFringe1[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"Interf DATA1 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("InterfCleanDATA1", "Interferometry DATA1 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);
 
        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatInterf->numOfFrames; f++)
        {
            if (compressedInterf->tarType[f] == 'T')
            {
                array[n] = compressedInterf->iFringe2[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"Interf DATA2 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("InterfCleanDATA2", "Interferometry DATA2 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);
 
        n = 0;
        for (f = 0; f < formatInterf->numOfFrames; f++)
        {
            if (compressedInterf->tarType[f] == 'T')
            {
                array[n] = compressedInterf->iFringe[f];
                n++;
            }
        }
        midiCreatePlotFileDouble2D ("InterfCleanDATA1_DATA2", "Interferometry DATA1-DATA2 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);

        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatPhotomA->numOfFrames; f++)
        {
            if (compressedPhotomA->tarType[f] == 'T')
            {
                array[n] = compressedPhotomA->iFringe1[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"PhotomA DATA1 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("PhotomACleanDATA1", "Photometry A DATA1 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);
 
        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatPhotomA->numOfFrames; f++)
        {
            if (compressedPhotomA->tarType[f] == 'T')
            {
                array[n] = compressedPhotomA->iFringe2[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"PhotomA DATA2 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("PhotomACleanDATA2", "Photometry A DATA2 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);

        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatPhotomB->numOfFrames; f++)
        {
            if (compressedPhotomB->tarType[f] == 'T')
            {
                array[n] = compressedPhotomB->iFringe1[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"PhotomB DATA1 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("PhotomBCleanDATA1", "Photometry B DATA1 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);

        n = 0;
        fluxAve = 0.0;
        for (f = 0; f < formatPhotomB->numOfFrames; f++)
        {
            if (compressedPhotomB->tarType[f] == 'T')
            {
                array[n] = compressedPhotomB->iFringe2[f];
                fluxAve += array[n];
                n++;
            }
        }
        fluxAve /= n;
       cpl_msg_info(cpl_func,"PhotomB DATA2 Average Flux = %f \n", fluxAve);
        midiCreatePlotFileDouble2D ("PhotomBCleanDATA2", "Photometry B DATA2 (Sky Removed)", 
            "Frame", "Flux", 0, array, 0, n, 1);

        free (array);
    }

    //    Display for each channel
    if (plotFile && diagnostic > 4)
    {
        arrayF = (float *) calloc (formatInterf->numOfFrames, sizeof (float));
        fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        
        for (R = 0; R < formatInterf->numOfRegionsToProcess; R++)
        {
            for (X = 0; X < formatInterf->iXWidth; X++)
            {
                if (badChannelList[X])
                    continue;
    
                n = 0;
                for (f = 0; f < formatInterf->numOfFrames; f++)
                {
                    if (compressedInterf->tarType[f] == 'T' && !(compressedInterf->rejectList[X][f]))
                    {
                        arrayF[n] = compressedInterf->iDispFringe[R][X][f];
                        n++;
                    }
                }
                sprintf (fileName, "InterfCleanDATA%d_X%d", R+1, X+1);
                sprintf (title, "Interferometry DATA%d for Channel %d (Sky Removed)", R+1, X+1);
                midiCreatePlotFile2D (fileName, title, "Frame", "Flux", 0, arrayF, 0, n, 1, 0);
            }
        }
        for (R = 0; R < formatPhotomA->numOfRegionsToProcess; R++)
        {
            for (X = 0; X < formatPhotomA->iXWidth; X++)
            {
                if (badChannelList[X])
                    continue;
                
                n = 0;
                for (f = 0; f < formatPhotomA->numOfFrames; f++)
                {
                    if (compressedPhotomA->tarType[f] == 'T' && !(compressedPhotomA->rejectList[X][f]))
                    {
                        arrayF[n] = compressedPhotomA->iDispFringe[R][X][f];
                        n++;
                    }
                }
                sprintf (fileName, "PhotomACleanDATA%d_X%d", R+1, X+1);
                sprintf (title, "Photometry A DATA%d for Channel %d (Sky Removed)", R+1, X+1);
                midiCreatePlotFile2D (fileName, title, "Frame", "Flux", 0, arrayF, 0, n, 1, 0);
            }
        }
        for (R = 0; R < formatPhotomB->numOfRegionsToProcess; R++)
        {
            for (X = 0; X < formatPhotomB->iXWidth; X++)
            {
                if (badChannelList[X])
                    continue;
                
                n = 0;
                for (f = 0; f < formatPhotomB->numOfFrames; f++)
                {
                    if (compressedPhotomB->tarType[f] == 'T' && !(compressedPhotomB->rejectList[X][f]))
                    {
                        arrayF[n] = compressedPhotomB->iDispFringe[R][X][f];
                        n++;
                    }
                }
                sprintf (fileName, "PhotomBCleanDATA%d_X%d", R+1, X+1);
                sprintf (title, "Photometry B DATA%d for Channel %d (Sky Removed)", R+1, X+1);
                midiCreatePlotFile2D (fileName, title, "Frame", "Flux", 0, arrayF, 0, n, 1, 0);
            }
        }
        free (fileName);
        free (title);
        free (arrayF);
    }

    //    Display the mean flux as a function of the channel

    if (plotFile)
    {	
       arrayF = (float *) calloc (formatInterf->iXWidth, sizeof (float));
       fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
       title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
       
       for (R = 0; R < formatInterf->numOfRegionsToProcess; R++)
       {
          for (X = 0; X < formatInterf->iXWidth; X++)
          {
             arrayF[X]=0;
             if (badChannelList[X])
                continue;
             
              n = 0;
              for (f = 0; f < formatInterf->numOfFrames; f++)
              {
                 if (compressedInterf->tarType[f] == 'T' && !(compressedInterf->rejectList[X][f]))
                 {
                    arrayF[X] += compressedInterf->iDispFringe[R][X][f];
                    n++;
                 }
              }
              arrayF[X] /=(float)n;
          }
          
          sprintf (fileName, "MeanInterfCleanDATA%d", R+1);
          sprintf (title, "Mean Interferometry DATA%d (Sky Removed)", R+1);
          midiCreatePlotFile2D (fileName, title, "Channel", "Flux", 0, arrayF, 0, formatInterf->iXWidth, 1, 0);
       }
       free (fileName);
       free (title);
       free (arrayF);
    }


    //    Finalise rejection list
    finaliseRejectListInterf ("Combined Interferometry", formatInterf, compressedInterf, error);
    if (*error) return;
    finaliseRejectListPhotom ("Combined Photometry A", formatPhotomA, compressedPhotomA, error);
    if (*error) return;
    finaliseRejectListPhotom ("Combined Photometry B", formatPhotomB, compressedPhotomB, error);
    if (*error) return;

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  finaliseRejectListInterf
* Input/Output: See function arguments to avoid duplication
* Description:    Finalising the badScanList, rejectList and badChannelList
*
* History:
* 14-Mar-06     (csabet) Created
******************************************************************************/
void finaliseRejectListInterf (
    const char            *dataKey,        // In: Data type flag
    ImageFormat     *format,        // In: Points to the image format
    CompressedData    *compressed,    // IO: Pointer to the compressed data structure
    int                *error)            // Ou: Error status
{
    //    Local Declarations
    //--------------------
    const char  routine[] = "finaliseRejectListInterf";
    int            i, f, s, x, numOfGoodFrames, numOfGoodChannels,
                numOfBadScans, minFrames;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine '%s'  \n", routine);

   cpl_msg_info(cpl_func,"\nFinalising Rejection List for %s data \n", dataKey);
   cpl_msg_info(cpl_func,"-------------------------------------------------- \n");
    fprintf (midiReportPtr, "\nFinalising Rejection List for %s data \n", dataKey);
    fprintf (midiReportPtr, "-------------------------------------------------- \n");

    //    Reset
    *error = 0;
    minFrames = MIN_FRAMES_PER_SCAN * format->framesPerScan;
    
    numOfGoodChannels = 0;
    for (x = 0; x < format->iXWidth; x++)
    {
        if (badChannelList[x])
        {
            for (f = 0; f < format->numOfFrames; f++)
                compressed->rejectList[x][f] |= badChannelList[x];
            continue;
        }
            
        numOfBadScans = 0;
        for (s = 0; s < format->numOfScans; s++)
        {
            numOfGoodFrames = 0;
            for (i = 0; i < format->framesPerScan; i++)
            {
                f = i + s * format->framesPerScan;
                if (!(compressed->rejectList[x][f]))
                    numOfGoodFrames++;
            }
            //    If number of good frames in a scan is less than a minimum reject scan
            if (numOfGoodFrames < minFrames)
                numOfBadScans++;
        }
        if (numOfBadScans == format->numOfScans)
            badChannelList[x] |= BSL_DATA_ERROR;
        else
            numOfGoodChannels++;
    }
    
    //    If no good channels return error
    if (!numOfGoodChannels)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "All channels have been rejected");
        *error = 1;
    }

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  finaliseRejectListPhotom
* Input/Output: See function arguments to avoid duplication
* Description:    Finalising the badScanList, rejectList and badChannelList
*                This routine is used for the photometry of the HIGH_SENS
*                as concept of scan is meaningless
*
* History:
* 14-Mar-06     (csabet) Created
******************************************************************************/
void finaliseRejectListPhotom (
    const char            *dataKey,        // In: Data type flag
    ImageFormat     *format,        // In: Points to the image format
    CompressedData    *compressed,    // IO: Pointer to the compressed data structure
    int                *error)            // Ou: Error status
{
    //    Local Declarations
    //--------------------
    const char  routine[] = "finaliseRejectListPhotom";
    int            f, x, numOfBadFrames, numOfGoodChannels;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine '%s'  \n", routine);

   cpl_msg_info(cpl_func,"\nFinalising Rejection List for %s data \n", dataKey);
   cpl_msg_info(cpl_func,"-------------------------------------------------- \n");
    fprintf (midiReportPtr, "\nFinalising Rejection List for %s data \n", dataKey);
    fprintf (midiReportPtr, "-------------------------------------------------- \n");

    //    Reset
    *error = 0;

    numOfGoodChannels = 0;
    for (x = 0; x < format->iXWidth; x++)
    {
        if (badChannelList[x])
        {
            for (f = 0; f < format->numOfFrames; f++)
                compressed->rejectList[x][f] |= badChannelList[x];
            continue;
        }
            
        numOfBadFrames = 0;
        for (f = 0; f < format->numOfFrames; f++)
        {
            if (compressed->rejectList[x][f])
                numOfBadFrames++;
        }
        if (numOfBadFrames == format->numOfFrames)
            badChannelList[x] |= BSL_DATA_ERROR;
        else
            numOfGoodChannels++;
    }
    
    //    If no good channels return error
    if (!numOfGoodChannels)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "All channels have been rejected");
        *error = 1;
    }

    return;
}
/*****************************************************************************/

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the correlated flux.
  @param    iauData    iauData data structure with wavelenth information
  @param    dispResult interferometric structure with correlated flux info
  @return   The CPL error code
 */
/*----------------------------------------------------------------------------*/

cpl_error_code midiSaveCorrelatedFlux(IauExchange *iauData,
		DispersedResult *dispResult, cpl_parameterlist *parlist,
		cpl_frameset *frameset)
{
	cpl_table        * table=NULL;
	cpl_propertylist * qclist=NULL;
	char             * dataname=NULL;

	table=cpl_table_new(iauData->wavelength->nwave);

	cpl_table_new_column(table, "Wavelength",          CPL_TYPE_FLOAT);
	cpl_table_new_column(table, "CorrelatedFlux",      CPL_TYPE_FLOAT);
	cpl_table_new_column(table, "CorrelatedFluxError", CPL_TYPE_FLOAT);
	cpl_table_new_column(table, "Flag",                CPL_TYPE_INT);

	cpl_table_set_column_unit(table ,"Wavelength",          "m");
	cpl_table_set_column_unit(table, "CorrelatedFlux",      "ADU");
	cpl_table_set_column_unit(table, "CorrelatedFluxError", "ADU");

	cpl_table_copy_data_float(table,"Wavelength",iauData->wavelength->eff_wave);
	cpl_table_copy_data_float(table,"CorrelatedFlux",dispResult->rawVis2);
	cpl_table_copy_data_float(table,"CorrelatedFluxError",dispResult->rawVis2Err);
	cpl_table_copy_data_int(table,"Flag",badChannelList);
	/*
	cpl_table_dump_structure(table,NULL);
	cpl_table_dump(table,0,200,NULL);
	 */
	qclist=cpl_propertylist_new();

	cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "CORRELATED_FLUX");
	dataname=cpl_sprintf("MIDI_CorrelatedFlux.fits");

	cpl_dfs_save_table(frameset, NULL, parlist, frameset, NULL, table,
			qclist, "midi_fringe_all",
			qclist, NULL,
			PACKAGE "/" PACKAGE_VERSION,
			dataname);

	cpl_propertylist_delete(qclist);
	cpl_free(dataname);
	cpl_table_delete(table);

	if(cpl_error_get_code()!=CPL_ERROR_NONE)
	{
		cpl_msg_error(cpl_func,"Correlated flux can not be saved!");
		return cpl_error_set(cpl_func, CPL_ERROR_FILE_NOT_CREATED);
	}
	else {
		return CPL_ERROR_NONE;
	}
}

/*****************************************************************************/

