/* $Id: midi_intopd.c,v 1.55 2011-11-21 09:40:20 agabasch Exp $
 *
 * This file is part of the MIDI Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2011-11-21 09:40:20 $
 * $Revision: 1.55 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/
#include <math.h>
#include <cpl.h>
#include "midi_utils.h"
#include "midi_cplutils.h"
#include "cpl_image_filter.h"
#include <fitsio.h>
#include "midiConst.h"
#include "midi_dfs.h"
#include "string.h"
#include "lomb_scargel.h"
#include "midi_cplupgrade.h"

#define DIMENDATA 2 
/*-----------------------------------------------------------------------------
                            Private function prototypes
 -----------------------------------------------------------------------------*/

static int midi_intopd_create(cpl_plugin *);
static int midi_intopd_exec(cpl_plugin *);
static int midi_intopd_destroy(cpl_plugin *);
static int midi_intopd(cpl_frameset *, const cpl_parameterlist *);


static void  midi_intopd_check_parameter(int * llx, int * lly, 
                                         int * urx, int * ury, 
                                         double * fmin, double * fmax,
                                         int * sampling,
                                         cpl_imagelist * imglst_ABOPEN0);

static int append_image_to_table(cpl_table * table, const char * columname, 
                                 cpl_image * image, int row);

static void save_mask_for_ews(cpl_frameset * frameset, const cpl_parameterlist * parlist, 
                              cpl_image ** opdmask,const char * name,
                              char * first_valid_frame);

static int midi_copy_extension(const char * infile , const char * outfile,
		const char * extension_name);

static void midi_intopd_qc_stats(cpl_propertylist * pro_list,
                                 cpl_table * table, const char * column_name);

 
/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

/* These wavelength calibration is taken from the static calibration file */
static double wavecalib_prism_hs_frequency[]={0.0706326774, 0.0707558873, 0.0708812144, 0.0710086692, 0.0711382772, 0.0712700642, 0.0714040373, 0.0715402332, 0.0716786644, 0.0718193590, 0.0719623404, 0.0721076328, 0.0722552605, 0.0724052439, 0.0725576234, 0.0727124054, 0.0728696265, 0.0730293143, 0.0731914915, 0.0733561970, 0.0735234497, 0.0736932899, 0.0738657377, 0.0740408347, 0.0742186024, 0.0743990732, 0.0745822912, 0.0747682797, 0.0749570788, 0.0751487242, 0.0753432522, 0.0755407000, 0.0757411057, 0.0759445081, 0.0761509415, 0.0763604520, 0.0765730812, 0.0767888660, 0.0770078611, 0.0772301053, 0.0774556385, 0.0776845187, 0.0779167818, 0.0781524825, 0.0783916647, 0.0786343910, 0.0788807019, 0.0791306567, 0.0793843101, 0.0796417240, 0.0799029438, 0.0801680280, 0.0804370487, 0.0807100554, 0.0809871170, 0.0812682981, 0.0815536648, 0.0818432783, 0.0821372209, 0.0824355571, 0.0827383534, 0.0830456908, 0.0833576459, 0.0836742903, 0.0839957179, 0.0843219977, 0.0846532210, 0.0849894814, 0.0853308542, 0.0856774376, 0.0860293181, 0.0863866060, 0.0867493857, 0.0871177658, 0.0874918501, 0.0878717455, 0.0882575543, 0.0886494115, 0.0890474028, 0.0894516622, 0.0898623119, 0.0902794776, 0.0907032805, 0.0911338691, 0.0915713642, 0.0920159223, 0.0924676717, 0.0929267693, 0.0933933680, 0.0938676170, 0.0943496952, 0.0948397528, 0.0953379702, 0.0958445247, 0.0963595988, 0.0968833719, 0.0974160647, 0.0979578415, 0.0985089263, 0.0990695319, 0.0996398781, 0.1002201913, 0.1008106958, 0.1014116523, 0.1020233009, 0.1026458890, 0.1032796929, 0.1039249773, 0.1045820472, 0.1052511762, 0.1059326790, 0.1066268812, 0.1073340762, 0.1080546341, 0.1087889046, 0.1095372161, 0.1102999658, 0.1110775425, 0.1118703495, 0.1126788049, 0.1135033426, 0.1143444129, 0.1152024835, 0.1160780271, 0.1169715608, 0.1178835959, 0.1188147173, 0.1197654389, 0.1207363775, 0.1217281474, 0.1227413885, 0.1237767821, 0.1248349940, 0.1259167631, 0.1270228297, 0.1281540054, 0.1293111060, 0.1304949757, 0.1317065291, 0.1329467317, 0.1342165168, 0.1355169386, 0.1368490662, 0.1382140376, 0.1396130370, 0.1410472968, 0.1425181298, 0.1440268952, 0.1455750610, 0.1471641000, 0.1487956338, 0.1504713171, 0.1521929141, 0.1539622963, 0.1557814268, 0.1576523663, 0.1595773159, 0.1615586032, 0.1635986409, 0.1657000609, 0.1678655738, 0.1700981363, 0.1724008044, 0.1747769110, 0.1772299438, 0.1797636156, 0.1823819143, 0.1850890631, 0.1878896053, 0.1907884167, 0.1937905936, 0.1968915361};


static char midi_intopd_description[] =

"This recipe calculates the internal OPD stability of MIDI based on\n"
"the group delay. It follows the paper of Lawson, P.R. 1995\n"
"(J. Opt. Soc. Am. A, Vol. 12, No. 2, p. 366 - 374) and analizes\n"
"the fringes in the frequency domain by the usage of the Scargle-Lomb\n"
"algorithm (ApJ, vol. 263, Dec. 15, 1982, p. 835-853).\n"
"\n"
"Input files:\n\n"
"  DO category:               Type:       Explanation:         Required:\n"
"  INTERNAL_OPD               Raw         Raw data frame          Y\n\n"
"Output files:\n\n"
"  DO category:               Data type:  Explanation:\n"
"  MIDI_INTOPD                FITS table  Internal OPD offset of MIDI\n"
"  MIDI_MASK_INTOPD           FITS table  Mask for signal extraction\n"
"  MIDI_MASK_DATA1            FITS image  Mask for DATA1 signal extraction\n"
"  MIDI_MASK_DATA2            FITS image  Mask for DATA2 signal extraction\n\n";

/*-----------------------------------------------------------------------------
                                Function code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 * @defgroup midi_intopd    Midi Intopd
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Build the list of available plugins, for this module. 
  @param    list    the plugin list
  @return   0 if everything is ok, 1 otherwise
  @note     Only this function is exported

  Create the recipe instance and make it available to the application using the 
  interface. 
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe  *   recipe = cpl_calloc(1, sizeof *recipe );
    cpl_plugin  *   plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                    CPL_PLUGIN_API,
                    MIDI_BINARY_VERSION,
                    CPL_PLUGIN_TYPE_RECIPE,
                    "midi_intopd",
                    "Derives the internal OPD stability",
                    midi_intopd_description,
                    "Armin Gabasch",
                    PACKAGE_BUGREPORT,
                    midi_get_license(),
                    midi_intopd_create,
                    midi_intopd_exec,
                    midi_intopd_destroy)) {    
        cpl_msg_error(cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where(cpl_func);                          
        return 1;                                               
    }                                                    

    if (cpl_pluginlist_append(list, plugin)) {                 
        cpl_msg_error(cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where(cpl_func);                         
        return 1;                                              
    }                                                          
    
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Setup the recipe options    
  @param    plugin  the plugin
  @return   0 if everything is ok

  Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int midi_intopd_create(cpl_plugin * plugin)
{
    cpl_recipe    * recipe;                                               
    cpl_parameter * p;
                                                                       
    /* Do not create the recipe if an error code is already set */     
    if (cpl_error_get_code() != CPL_ERROR_NONE) {                      
        cpl_msg_error(cpl_func, "%s():%d: An error is already set: %s",
                      cpl_func, __LINE__, cpl_error_get_where());      
        return (int)cpl_error_get_code();                              
    }                                                                  
                                                                       
    if (plugin == NULL) {                                              
        cpl_msg_error(cpl_func, "Null plugin");                        
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                 
    }                                                                  
                                                                       
    /* Verify plugin type */                                           
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {       
        cpl_msg_error(cpl_func, "Plugin is not a recipe");             
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);              
    }                                                                  
                                                                       
    /* Get the recipe */                                               
    recipe = (cpl_recipe *)plugin;                                     
                                                                       
    /* Create the parameters list in the cpl_recipe object */          
    recipe->parameters = cpl_parameterlist_new();                      
    if (recipe->parameters == NULL) {                                  
        cpl_msg_error(cpl_func, "Parameter list allocation failed");   
        cpl_ensure_code(0, (int)CPL_ERROR_ILLEGAL_OUTPUT);             
    }                                                                  

    /* Fill the parameters list */

    p = cpl_parameter_new_value("midi.midi_intopd.llx", 
            CPL_TYPE_INT, "Lower left x-value of the window where the signal resides", "midi.midi_intopd",60);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "llx");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);

    p = cpl_parameter_new_value("midi.midi_intopd.lly", 
            CPL_TYPE_INT, "Lower left y-value of the window where the signal resides", "midi.midi_intopd",11);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "lly");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);


    p = cpl_parameter_new_value("midi.midi_intopd.urx", 
            CPL_TYPE_INT, "Upper right x-value of the window where the signal resides", "midi.midi_intopd",130);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "urx");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);


    p = cpl_parameter_new_value("midi.midi_intopd.ury", 
            CPL_TYPE_INT, "Upper right y-value of the window where the signal resides", "midi.midi_intopd",18);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "ury");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);



    p = cpl_parameter_new_value("midi.midi_intopd.lomb_fmin", 
            CPL_TYPE_DOUBLE, "Minimum frequency of the Lomb evaluation window", "midi.midi_intopd",70.);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "fmin");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);

    p = cpl_parameter_new_value("midi.midi_intopd.lomb_fmax", 
            CPL_TYPE_DOUBLE, "Maximum frequency of the Lomb evaluation window", "midi.midi_intopd",130.);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "fmax");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);


    p = cpl_parameter_new_value("midi.midi_intopd.lomb_sampling", 
            CPL_TYPE_INT, "Sampling rate inside the Lomb evaluation window", "midi.midi_intopd",12000);
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "sampling");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);

    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Execute the plugin instance given by the interface
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_intopd_exec(cpl_plugin * plugin)
{

    cpl_recipe * recipe;                                                   
    int recipe_status;                                                     
    cpl_errorstate initial_errorstate = cpl_errorstate_get();              
                                                                           
    /* Return immediately if an error code is already set */               
    if (cpl_error_get_code() != CPL_ERROR_NONE) {                          
        cpl_msg_error(cpl_func, "%s():%d: An error is already set: %s",    
                      cpl_func, __LINE__, cpl_error_get_where());          
        return (int)cpl_error_get_code();                                  
    }                                                                      
                                                                           
    if (plugin == NULL) {                                                  
        cpl_msg_error(cpl_func, "Null plugin");                            
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
                                                                           
    /* Verify plugin type */                                               
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {           
        cpl_msg_error(cpl_func, "Plugin is not a recipe");                 
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);                  
    }                                                                      
                                                                           
    /* Get the recipe */                                                   
    recipe = (cpl_recipe *)plugin;                                         
                                                                           
    /* Verify parameter and frame lists */                                 
    if (recipe->parameters == NULL) {                                      
        cpl_msg_error(cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
    if (recipe->frames == NULL) {                                          
        cpl_msg_error(cpl_func, "Recipe invoked with NULL frame set");     
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
                                                                           
    /* Invoke the recipe */                                                
    recipe_status = midi_intopd(recipe->frames, recipe->parameters);
                                                                           
    /* Ensure DFS-compliance of the products */                            
    if (cpl_dfs_update_product_header(recipe->frames)) {                   
        if (!recipe_status) recipe_status = (int)cpl_error_get_code();                         
    }                                                                      
                                                                           
    if (!cpl_errorstate_is_equal(initial_errorstate)) {                    
        /* Dump the error history since recipe execution start.            
           At this point the recipe cannot recover from the error */       
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);          
    }                                                                      
                                                                           
    return recipe_status;                                                  
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Destroy what has been created by the 'create' function
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_intopd_destroy(cpl_plugin * plugin)
{
    cpl_recipe * recipe;                                          
                                                                  
    if (plugin == NULL) {                                         
        cpl_msg_error(cpl_func, "Null plugin");                   
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);            
    }                                                             
                                                                  
    /* Verify plugin type */                                      
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {  
        cpl_msg_error(cpl_func, "Plugin is not a recipe");        
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);         
    }                                                             
                                                                  
    /* Get the recipe */                                          
    recipe = (cpl_recipe *)plugin;                                
                                                                  
    cpl_parameterlist_delete(recipe->parameters);             
                                                                  
    return 0;                                                    
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    frameset   the frames list
  @param    parlist    the parameters list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_intopd(cpl_frameset            * frameset,
                            const cpl_parameterlist * parlist)
{

   int                     llx=0;
   int                     lly=0;
   int                     urx=0;
   int                     ury=0;
   cpl_image             * opdmask[DIMENDATA]={NULL,NULL};
   cpl_image             * dummy_image=NULL;
   cpl_image             * dummy_image_data1=NULL;
   cpl_image             * dummy_image_data2=NULL;
   int                     Cgood=0; 
   int                     Csaturated=0; 

   cpl_image             * dummy_image_ycollapsed=NULL;
   cpl_image             * mask_goodvalues=NULL;
   char                  * first_valid_frame=NULL;
   int                     isFirst=0;   
   char                  * filename=NULL;
   double                  image_max_data1=0;
   double                  image_max_data2=0;
   int                     ext_imaging_data=0;       
   cpl_table             * timetable=NULL;


   cpl_frame             * cur_frame=NULL;
   int                     dimenDATA=DIMENDATA; /* Assumption: HS Data */
   cpl_imagelist         * imglst_ABOPEN[DIMENDATA]={NULL,NULL}; /* Assumption: HS Data */
   cpl_imagelist         * imglst_ABOPEN_ycollapsed[DIMENDATA]={NULL,NULL}; /* Assumption: HS Data */

   cpl_errorstate          prestate = cpl_errorstate_get();
   int                     i=0; 
   /* int                     j=0; */
   char                  * tag=NULL;

   cpl_table             * table=NULL;
   char                  * dataname=NULL;
   cpl_propertylist      * qclist=NULL;
   /* cpl_propertylist      * pro_list=NULL; */
   /* cpl_mask              * mask=NULL; */
   cpl_table             * intopd_table=NULL;
   
   cpl_array             * frequency=NULL;

   cpl_array             * dummy_frequency=NULL;
   cpl_array             * dummy_fringe=NULL;

   cpl_array            ** fringe=NULL;
   cpl_array            ** lomb_out=NULL;
   cpl_size                lomb_maxpos=0;
   int                     size_fringe=0;    
   int                     size_imglist_ABOPEN_ycollapsed=0;
   double                  fmin=0.;
   double                  fmax=0.;
   int                     sampling=0;
   int                     workcounter=0;


   

/* Check if siutable files are present in the SOF */
   if(midi_check_sof(frameset,MIDI_INTERNAL_OPD)<1)
   {
      return (int)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
                                        "SOF has no appropriate AOPEN fitsfiles! Aborting!");
   }

   /* RETRIEVE THE INPUT PARAMETERS */
   
   llx = cpl_parameter_get_int(cpl_parameterlist_find_const(parlist, "midi.midi_intopd.llx"));
   lly = cpl_parameter_get_int(cpl_parameterlist_find_const(parlist, "midi.midi_intopd.lly"));
   urx = cpl_parameter_get_int(cpl_parameterlist_find_const(parlist, "midi.midi_intopd.urx"));
   ury = cpl_parameter_get_int(cpl_parameterlist_find_const(parlist, "midi.midi_intopd.ury"));

   fmin = cpl_parameter_get_double(cpl_parameterlist_find_const(parlist, "midi.midi_intopd.lomb_fmin"));
   fmax = cpl_parameter_get_double(cpl_parameterlist_find_const(parlist, "midi.midi_intopd.lomb_fmax"));
   sampling = cpl_parameter_get_int(cpl_parameterlist_find_const(parlist, "midi.midi_intopd.lomb_sampling"));


   if (!cpl_errorstate_is_equal(prestate)) {
      return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(), "Could not retrieve the input parameters");
   }

   /* Identify the RAW and CALIB frames in the input frameset */
   cpl_ensure_code(midi_dfs_set_groups(frameset) == CPL_ERROR_NONE,
                   cpl_error_get_code());
   

   for (i=0; i<dimenDATA;i++){
      imglst_ABOPEN[i]=cpl_imagelist_new();
      imglst_ABOPEN_ycollapsed[i]=cpl_imagelist_new();
      
   }
   
   timetable=cpl_table_new(0);
   cpl_table_new_column(timetable,"TIME",CPL_TYPE_DOUBLE); 		
   cpl_table_set_column_unit(timetable,"TIME","DAY");
   
   cur_frame = cpl_frameset_get_position(frameset, 0);
   if (cur_frame == NULL) {
      return (int)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
                                        "SOF does not have any file");
   }
   
/*    Walk through the whole Set of Frames SOF and */
/*    append the images from the tables to the various imagelists */
/*    as well as the the timetable*/     
   cpl_frameset_iterator * it = cpl_frameset_iterator_new(frameset);
   cur_frame = cpl_frameset_iterator_get(it);
   while(cur_frame)
   {
      
      tag = (char*)cpl_frame_get_tag(cur_frame);
      if (strcmp(tag, MIDI_INTERNAL_OPD)) {
         cpl_frameset_iterator_advance(it, 1);
         cur_frame = cpl_frameset_iterator_get(it);
         continue;
      }

      /* Save the name of the first valid frame to be EWS compatible in the product*/
      /* Check the recipe input parameters */

      if(isFirst<1){
         isFirst=1;
         first_valid_frame=cpl_sprintf("%s", cpl_frame_get_filename(cur_frame));         
      }
      cpl_msg_info(cpl_func, "Processing file  %s",cpl_frame_get_filename(cur_frame));
      
      /* Load extension  Imaging Data  */ 
      ext_imaging_data=cpl_fits_find_extension(cpl_frame_get_filename(cur_frame),"IMAGING_DATA");
      table = cpl_table_load(cpl_frame_get_filename(cur_frame), ext_imaging_data, 1);
      if (table == NULL) {
         return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                           "Could not load the table");
      }
      cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());      
      
/*       Load columns DATA... */
      for (i=0; i<dimenDATA;i++){
         dataname=cpl_sprintf("DATA%d",i+1);           
         
         /*         cpl_msg_info(cpl_func, "Scanning for dataname ..."); */
         if (cpl_table_has_column(table,dataname)){
            table_to_imglst(dataname,imglst_ABOPEN[i],table);
         }
         cpl_msg_info(cpl_func, "Number of so far processed %s Frames: % " CPL_SIZE_FORMAT " ",dataname,cpl_imagelist_get_size(imglst_ABOPEN[i]));
         cpl_free(dataname);
      }
      cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
      
      /* Load column TIME... */
      
      if (cpl_table_has_column(table,"TIME")){
         timetable_to_cpltable("TIME",table,timetable);
      }
      cpl_msg_info(cpl_func, "Number of so far processed Timestamps: % " CPL_SIZE_FORMAT " ",cpl_table_get_nrow(timetable));
      cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());

      cpl_table_delete(table);

      cpl_frameset_iterator_advance(it, 1);
      cur_frame = cpl_frameset_iterator_get(it);
   }
   cpl_frameset_iterator_delete(it);

   midi_intopd_check_parameter(&llx, &lly, &urx, &ury, &fmin, &fmax, &sampling, imglst_ABOPEN[0]);

/* Create the mask */
   cpl_msg_info(cpl_func, "Creating the mask for signal extraction:");
   cpl_msg_info(cpl_func, "Signal in x: pixel %3d to pixel %3d (all inclusive)",llx,urx);
   cpl_msg_info(cpl_func, "Signal in y: pixel %3d to pixel %3d (all inclusive)",lly,ury);

   mask_goodvalues=cpl_image_new((urx-llx)+1,(ury-lly)+1,  CPL_TYPE_DOUBLE);
   cpl_image_add_scalar(mask_goodvalues,1);  	  
   dummy_image=cpl_imagelist_get(imglst_ABOPEN[1],1);
   for (i=0; i<dimenDATA;i++){
      opdmask[i]=cpl_image_new(cpl_image_get_size_x(dummy_image),cpl_image_get_size_y(dummy_image),CPL_TYPE_DOUBLE );
      cpl_image_copy(opdmask[i],mask_goodvalues,llx,lly);
   }
   cpl_image_delete(mask_goodvalues);

   cpl_msg_info(cpl_func,"Saving the mask file ...");
   save_mask_for_ews(frameset,parlist,opdmask,"opdmask", first_valid_frame);
   cpl_free(first_valid_frame);

   
/*    Multiply the mask files to the single images in the imagelists */
   cpl_msg_info(cpl_func,"Multiplying the mask with the images ...");

   for (i=0; i<dimenDATA;i++){
      cpl_imagelist_multiply_image(imglst_ABOPEN[i],opdmask[i]);
   }

/*    Collapse the single images in the imagelist in y-direction */
/*    and remove saturated images */

   cpl_msg_info(cpl_func,"Collapsing images in y-direction and removing saturated images...");
   Cgood=0;
   Csaturated=0;
   for (i=0; i<cpl_imagelist_get_size(imglst_ABOPEN[0]);i++){
      dummy_image_data1=cpl_imagelist_get(imglst_ABOPEN[0],i);
      dummy_image_data2=cpl_imagelist_get(imglst_ABOPEN[1],i);
      image_max_data1=cpl_image_get_max(dummy_image_data1);
      image_max_data2=cpl_image_get_max(dummy_image_data2);
      if(image_max_data1>=PIXEL_SATURATION || image_max_data2>=PIXEL_SATURATION ){
         cpl_table_set_invalid(timetable,"TIME",i); 		
         Csaturated++;
         continue;
      }

      dummy_image_ycollapsed=cpl_image_collapse_create(dummy_image_data1,0);
      cpl_imagelist_set(imglst_ABOPEN_ycollapsed[0],dummy_image_ycollapsed,Cgood);
      
      dummy_image_ycollapsed=cpl_image_collapse_create(dummy_image_data2,0);
      cpl_imagelist_set(imglst_ABOPEN_ycollapsed[1],dummy_image_ycollapsed,Cgood);
      Cgood++;
   }
   
   if(Csaturated>0){
      cpl_msg_warning(cpl_func,"%d saturated images skipped",Csaturated);
   }
   else {
      cpl_msg_info(cpl_func,"No saturated images found");
   }

   /* If all frames are saturated, exit! */
   if(cpl_imagelist_get_size(imglst_ABOPEN_ycollapsed[0])<1){
      
      cpl_msg_error(cpl_func,"All frames are saturated! Exiting...");
      /* ---------- Free the memory ---------- */
      for (i=0; i<dimenDATA;i++){
         while(cpl_imagelist_get_size(imglst_ABOPEN[i])>0){
            cpl_image_delete(cpl_imagelist_unset(imglst_ABOPEN[i],0));
         }
         while(cpl_imagelist_get_size(imglst_ABOPEN_ycollapsed[i])>0){
            cpl_image_delete(cpl_imagelist_unset(imglst_ABOPEN_ycollapsed[i],0));
         }
         
         
         cpl_imagelist_delete(imglst_ABOPEN[i]);
         cpl_imagelist_delete(imglst_ABOPEN_ycollapsed[i]);
         cpl_image_delete(opdmask[i]);
      }
      cpl_table_delete(timetable);
      /* ---------- Free the memory ---------- */
      return -1;
   }
   
   /* Clean the timetable where the corresponding images are saturated */  
   cpl_table_erase_invalid(timetable);   	

   cpl_msg_info(cpl_func,"Subtracting DATA2 from DATA1 to remove the background and extract the fringes ...");
   cpl_imagelist_subtract(imglst_ABOPEN_ycollapsed[0], imglst_ABOPEN_ycollapsed[1]);

/* For Diagnostic during development */
   if(0){
/*    cpl_imagelist_save(imglst_ABOPEN_ycollapsed[0], "armin.fits", CPL_BPP_IEEE_FLOAT, NULL, CPL_IO_CREATE); */
/*    for (i=0; i<cpl_imagelist_get_size(imglst_ABOPEN_ycollapsed[0]);i=i+50){ */
      for (i=0; i<30;i=i+1){
         filename = cpl_sprintf("slice_%d.fits",i);
      cpl_image_save(cpl_imagelist_get(imglst_ABOPEN_ycollapsed[0],i), filename, CPL_BPP_IEEE_FLOAT, NULL, CPL_IO_CREATE);	 
      cpl_free(filename);
      }
      
      cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
      
/*    cpl_table_dump_structure(timetable,NULL); */
/*    cpl_table_dump(timetable,0,100,NULL); */
/*    cpl_table_save(timetable,NULL,NULL,"timetable.fits",CPL_IO_CREATE);	  */
/*    cpl_image_dump_window(cpl_imagelist_get(imglst_ABOPEN_ycollapsed[0],10),1,1,172,1,stdout); */
   }

   size_fringe=sizeof(wavecalib_prism_hs_frequency)/sizeof(double);

   dummy_frequency =cpl_array_new(size_fringe, CPL_TYPE_DOUBLE);
   cpl_array_copy_data_double(dummy_frequency, wavecalib_prism_hs_frequency);
   frequency=cpl_array_extract(dummy_frequency, llx-1,urx-llx+1); 	
   cpl_array_delete(dummy_frequency);      
   
   fringe=cpl_malloc(cpl_imagelist_get_size(imglst_ABOPEN_ycollapsed[0]) * sizeof(cpl_array *));
   for (i=0; i<cpl_imagelist_get_size(imglst_ABOPEN_ycollapsed[0]);i=i+1){
      dummy_fringe =cpl_array_new(size_fringe, CPL_TYPE_DOUBLE);      
      dummy_image_ycollapsed=cpl_image_cast(cpl_imagelist_get(imglst_ABOPEN_ycollapsed[0],i),CPL_TYPE_DOUBLE);
      cpl_array_copy_data_double(dummy_fringe,  cpl_image_get_data_double(dummy_image_ycollapsed)); 
      fringe[i]=cpl_array_extract(dummy_fringe, llx-1,urx-llx+1); 
      cpl_image_delete(dummy_image_ycollapsed);
      cpl_array_delete(dummy_fringe);
   }

/*    cpl_msg_info(cpl_func,"frequency: %d fringe: %d ", cpl_array_get_size(frequency), cpl_array_get_size(fringe[1])); */
/*    for (i=0; i<cpl_array_get_size(fringe[1]);i=i+1){ */
/*       cpl_msg_info(cpl_func,"frequency, fringe %d %g, %g",i,  */
/*                    cpl_array_get_double(frequency,i,NULL), */
/*                    cpl_array_get_double(fringe[1],i,NULL)); */
/*    } */
   

   lomb_out=cpl_malloc(2 * sizeof(cpl_array *));

   cpl_table_new_column(timetable,"PATH_DIFFERENCE",CPL_TYPE_DOUBLE); 
   cpl_table_new_column(timetable,"LOMB_POWER",CPL_TYPE_DOUBLE); 
   cpl_table_set_column_unit  	(timetable, "PATH_DIFFERENCE","m"); 
   cpl_table_set_column_unit  	(timetable, "TIME","day"); 


   size_imglist_ABOPEN_ycollapsed=cpl_imagelist_get_size(imglst_ABOPEN_ycollapsed[0]);
   workcounter=0;   

   cpl_msg_info(cpl_func," ");
   cpl_msg_info(cpl_func,"Deriving the OPD stability based on the paper of Lawson, P.R. 1995:");
   cpl_msg_info(cpl_func,"J. Opt. Soc. Am. A, Vol. 12, No. 2, p. 366 - 374");
   cpl_msg_info(cpl_func,"Fringes are analyzed in the frequency domain by the usage of the");
   cpl_msg_info(cpl_func,"Scargle-Lomb algorithm:  ApJ, vol. 263, Dec. 15, 1982, p. 835-853.");
   cpl_msg_info(cpl_func," ");
   cpl_msg_info(cpl_func,"The detector-channel to lambda/frequency mapping is not derived"); 
   cpl_msg_info(cpl_func,"from the analyzed images, but the values are static to this recipe"); 
   cpl_msg_info(cpl_func,"(see variable wavecalib_prism_hs_frequency)");
   cpl_msg_info(cpl_func," ");

   for (i=0; i<size_imglist_ABOPEN_ycollapsed;i=i+1){
/*    for (i=19; i<20;i=i+1){ */
      lomb_maxpos=0;   
      lomb(fmin,fmax,sampling,frequency,fringe[i],lomb_out); 
      cpl_array_get_maxpos (lomb_out[1], &lomb_maxpos);
      cpl_table_set_double (timetable, "PATH_DIFFERENCE", i,  cpl_array_get_double(lomb_out[0], lomb_maxpos,NULL)/1e6);
      cpl_table_set_double (timetable, "LOMB_POWER", i,  cpl_array_get_max(lomb_out[1]));


/*       cpl_msg_info(cpl_func,"Time, Power, Frequency: %.15e  %.10e  %.5e", */
/*                    cpl_table_get_double(timetable, "TIME", i, NULL), */
/*                    cpl_array_get_max(lomb_out[1]),  */
/*                    cpl_array_get_double(lomb_out[0], lomb_maxpos,NULL)/1e6);  */

/*       for(j=0;j<cpl_array_get_size(lomb_out[0]);j++){ */
/*          cpl_msg_info(cpl_func,"frequency, fringe %g, %g", */
/*                       cpl_array_get_double(lomb_out[0],j,NULL), */
/*                       cpl_array_get_double(lomb_out[1],j,NULL)); */
         
/*       } */
      cpl_array_delete(lomb_out[0]);
      cpl_array_delete(lomb_out[1]);

      if(i%(int)(size_imglist_ABOPEN_ycollapsed/10) == 0)
      {
         cpl_msg_info(cpl_func,"%3d per cent done",workcounter);
         workcounter+=10;
      }
     
   }
   cpl_msg_info(cpl_func," All done!");



/* Save it to a fits table */
   

/* Propertylist to write to the extension */
   qclist=cpl_propertylist_new();

   cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_INTOPD");
   cpl_propertylist_append_string(qclist, "EXTNAME", "OPD_ACCURACY");
   
   /*Derive QC parameters*/
   midi_intopd_qc_stats(qclist,timetable,"PATH_DIFFERENCE");
   
   
   cpl_dfs_save_table(frameset, NULL, parlist, frameset, NULL, timetable, 
                      qclist, "midi_intopd",
                      qclist, NULL, 
                      PACKAGE "/" PACKAGE_VERSION,
                      "midi_intopd.fits");
   cpl_table_delete(intopd_table);   
   

/* ---------- Free the memory ---------- */

   for (i=0; i<cpl_imagelist_get_size(imglst_ABOPEN_ycollapsed[0]);i=i+1){
      cpl_array_delete(fringe[i]);
   }

   for (i=0; i<dimenDATA;i++){
      while(cpl_imagelist_get_size(imglst_ABOPEN[i])>0){
         cpl_image_delete(cpl_imagelist_unset(imglst_ABOPEN[i],0));
      }
      while(cpl_imagelist_get_size(imglst_ABOPEN_ycollapsed[i])>0){
         cpl_image_delete(cpl_imagelist_unset(imglst_ABOPEN_ycollapsed[i],0));
      }


      cpl_imagelist_delete(imglst_ABOPEN[i]);
      cpl_imagelist_delete(imglst_ABOPEN_ycollapsed[i]);
      cpl_image_delete(opdmask[i]);
   }
   cpl_table_delete(timetable);

   cpl_array_delete(frequency);
   cpl_free(fringe);
   cpl_free(lomb_out); 
   cpl_propertylist_delete(qclist);


/* ---------- Free the memory ---------- */


   return (int)cpl_error_get_code();
}


static void  midi_intopd_check_parameter(int * llx, int * lly, 
                                         int * urx, int * ury, 
                                         double * fmin, double * fmax, int * sampling, 
                                         cpl_imagelist * imglst_ABOPEN0)
{
   int size_x=0;
   int size_y=0;
   size_x=cpl_image_get_size_x(cpl_imagelist_get(imglst_ABOPEN0,1));
   size_y=cpl_image_get_size_y(cpl_imagelist_get(imglst_ABOPEN0,1));

   cpl_msg_info(cpl_func, "Checking recipe parameters ...");

   if (*llx < 1)
   {
      *llx=1;
      cpl_msg_warning(cpl_func, "recipe parameters llx is reset to %d",*llx);
   }
   if (*lly < 1 )
   {
      *lly=1;
      cpl_msg_warning(cpl_func, "recipe parameters lly is reset to %d",*lly);
   }
   if (*urx <= *llx)
   {
      *urx=*llx+1;
      cpl_msg_warning(cpl_func, "recipe parameters urx is reset to %d",*urx);
   }
   if (*ury <= *lly )
   {
      *ury=*lly+1;
      cpl_msg_warning(cpl_func, "recipe parameters ury is reset to %d",*ury);
   }

   if (*llx > size_x)
   {
      *llx=size_x-1;
      cpl_msg_warning(cpl_func, "recipe parameters llx is reset to %d",*llx);
   }
   if (*lly > size_y)
   {
      *lly=size_y-1;
      cpl_msg_warning(cpl_func, "recipe parameters lly is reset to %d",*lly);
   }

   if (*urx > size_x)
   {
      *urx=size_x;
      cpl_msg_warning(cpl_func, "recipe parameters urx is reset to %d",*urx);
   }
   if (*ury > size_y)
   {
      *ury=size_y;
      cpl_msg_warning(cpl_func, "recipe parameters ury is reset to %d",*ury);
   }


   if (*sampling<1)
   {
      *sampling=1;
      cpl_msg_warning(cpl_func, "recipe parameters sampling is reset to %d",*sampling);
   }

   if (*fmin < 0)
   {
      *fmin=0.;
      cpl_msg_warning(cpl_func, "recipe parameters fmin is reset to %g",*fmin);
   }

   if (*fmax < *fmin)
   {
      *fmax=*fmin+2*DBL_MIN*(*sampling);
      cpl_msg_warning(cpl_func, "recipe parameters fmax is reset to %g",*fmax);
   }
   
   return;   
}


void save_mask_for_ews(cpl_frameset * frameset, const cpl_parameterlist * parlist, 
                       cpl_image ** opdmask ,const char * name, char  * first_valid_frame)
{

   char                  * fitsname;
   cpl_table             * dummy_table=NULL;
   cpl_propertylist      * qclist=NULL;
   cpl_image             * opdmask_float[DIMENDATA]; /* Assumption: HS Data */       

   qclist = cpl_propertylist_new();

   cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_MASK_DATA1");
   fitsname = cpl_sprintf("%s%s",name, "_DATA1.fits");
   if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  opdmask[0],
                          CPL_BPP_IEEE_FLOAT, name,
                          qclist, NULL,
                          PACKAGE "/" PACKAGE_VERSION,
                          fitsname)) {
      /* Propagate the error */
      (void)cpl_error_set_where(cpl_func);
   }
   cpl_free(fitsname);


   cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_MASK_DATA2");
   fitsname = cpl_sprintf("%s%s",name, "_DATA2.fits");
   if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL,  opdmask[1],
                          CPL_BPP_IEEE_FLOAT, name,
                          qclist, NULL,
                          PACKAGE "/" PACKAGE_VERSION,
                          fitsname)) {
      /* Propagate the error */
      (void)cpl_error_set_where(cpl_func);
   }
   cpl_free(fitsname);


/* Save it to a fits table */
   
   dummy_table=cpl_table_new(1);

   cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_MASK_INTOPD");   
/* Propertylist to write to the extension */
   cpl_propertylist_append_string (qclist, "EXTNAME", "IMAGING_DATA");

   opdmask_float[0]=cpl_image_cast(opdmask[0],CPL_TYPE_FLOAT);
   opdmask_float[1]=cpl_image_cast(opdmask[1],CPL_TYPE_FLOAT);



   append_image_to_table(dummy_table,"DATA1",opdmask_float[0],0);
   append_image_to_table(dummy_table,"DATA2",opdmask_float[1],0);


   fitsname = cpl_sprintf("%s%s",name, ".fits");

   cpl_dfs_save_table(frameset, NULL, parlist, frameset, NULL, dummy_table, 
                      qclist, name,
                      qclist, NULL, 
                      PACKAGE "/" PACKAGE_VERSION,
                      fitsname);
   cpl_table_delete(dummy_table);   
   cpl_free(fitsname);
   
   cpl_image_delete(opdmask_float[0]);
   cpl_image_delete(opdmask_float[1]);
   cpl_propertylist_delete(qclist);

   /* Append the IMAGING_DETECTOR table from the first valid image */
   /*  midi_copy_extension() is a workaround to cpl_table_save() as
    *  the latter is modifiing the content during saving */
   
   midi_copy_extension(first_valid_frame, "opdmask.fits", "IMAGING_DETECTOR");

   return;

}   


/*----------------------------------------------------------------------------*/
/**
  @brief    copy/appends an extension of infile to outfile
  @param    infile         the filename with the extension
  @param    outfile        the filename to which the extension should be copied 
  @param    extension_name the name of the extension
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/

static int midi_copy_extension(const char * infile , const char * outfile, const char * extension_name)   
{

/*    cpl_propertylist      * pHeader= NULL; */
/*    cpl_table             * pTable= NULL; */
/*    int                     ext_imaging_detector=-1;  */
   
/*    ext_imaging_detector=cpl_fits_find_extension(infile, extension_name); */

/*    cpl_msg_info(cpl_func,"%d",ext_imaging_detector); */

/*    pHeader = cpl_propertylist_load(infile, ext_imaging_detector ); */
/*    pTable  = cpl_table_load(infile, ext_imaging_detector, 1 ); */

/*    if (pHeader ==NULL || pTable==NULL) */
/*    { */
/*       cpl_msg_error(cpl_func,"Error loading the extended header or table of %s",infile);  */
      
/*       cpl_propertylist_delete(pHeader); */
/*       cpl_table_delete(pTable);  */
      
/*       return -1; */
/*    } */
 
/*    if (CPL_ERROR_NONE != cpl_table_save(pTable, NULL, pHeader, outfile, CPL_IO_EXTEND )) */
/*    { */
/*       cpl_msg_error(cpl_func,"Error when saving Extension: %s %s",cpl_error_get_message(),cpl_error_get_where()); */
/*    } */


/*    return 0; */


   fitsfile * fptrin=NULL;
   fitsfile * fptrout=NULL;
   int status=0;
   
   fits_open_diskfile(&fptrin, infile, READONLY, &status);
   fits_open_diskfile(&fptrout, outfile, READWRITE, &status);
   fits_movnam_hdu(fptrin, ANY_HDU, (char *)extension_name, 0, &status);
   fits_copy_hdu(fptrin, fptrout, 0, &status);
   fits_close_file(fptrin, &status);
   fits_close_file(fptrout, &status);
   
   if  (status != 0){
      cpl_msg_error(cpl_func,"A problem occured while copying the EXTENSION: %s", extension_name);
      return 1;
   }
   else{
      return 0;
   }
}



/*----------------------------------------------------------------------------*/
/**
  @brief    Inserts an image into a table
  @param    table       the table
  @param    columname   the name of the column
  @param    image       the image to insert
  @param    raw         the insert raw 
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/

static int append_image_to_table(cpl_table * table, const char * columname, cpl_image * image, int row)
{
   
   cpl_array        * array_dimension=NULL;   
   cpl_array        * array_dummy=NULL;
   
   array_dimension=cpl_array_new(2,CPL_TYPE_INT);
   cpl_array_set(array_dimension, 0,cpl_image_get_size_x(image));
   cpl_array_set(array_dimension, 1,cpl_image_get_size_y(image));
   
   cpl_table_new_column_array(table, columname, CPL_TYPE_FLOAT, cpl_image_get_size_x(image)*cpl_image_get_size_y(image));
   cpl_table_set_column_dimensions(table,columname,array_dimension);
   array_dummy = cpl_array_wrap_float(cpl_image_get_data_float(image), cpl_image_get_size_x(image)*cpl_image_get_size_y(image));    
   cpl_table_set_array(table, columname, row, array_dummy);
   cpl_array_unwrap(array_dummy);
   
   
   cpl_array_delete(array_dimension);
   
   return 0;  
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Calculates statistics as min, max, ... and writes QC parameters.
  @param    pro_list    property list where the QC parameters are appended
  @param    table       table where the statistics are calculated
  @return   NONE
 */
/*----------------------------------------------------------------------------*/
static void midi_intopd_qc_stats(cpl_propertylist * pro_list, cpl_table * table,
		const char * column_name)
{
	double table_max=0.;
	double table_min=0.;
	double table_mean=0.;
	double table_median=0.;
	double table_stdev=0.;

	table_min=cpl_table_get_column_min(table,column_name);
	table_max=cpl_table_get_column_max(table,column_name);
	table_mean=cpl_table_get_column_mean(table,column_name);
	table_median=cpl_table_get_column_median(table,column_name);
	table_stdev=cpl_table_get_column_stdev(table,column_name);

	cpl_propertylist_update_double(pro_list, "ESO QC INTOPD MIN",table_min);
	cpl_propertylist_update_double(pro_list, "ESO QC INTOPD MAX",table_max);
	cpl_propertylist_update_double(pro_list, "ESO QC INTOPD MEAN",table_mean);
	cpl_propertylist_update_double(pro_list, "ESO QC INTOPD MEDIAN",table_median);
	cpl_propertylist_update_double(pro_list, "ESO QC INTOPD STDEV",table_stdev);

	return;
}

/**@}*/
